package com.xxh.cloud.basic.application.controller;


import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.xxh.cloud.basic.framework.modules.base.security.SignCertificateGetParam;
import com.xxh.cloud.basic.framework.modules.base.security.SignCertificateModel;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.HttpContextUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.framework.security.bean.RsaPubPriKeyModel;
import com.xxh.cloud.framework.security.utils.CacheKeys;
import com.xxh.cloud.framework.security.utils.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

@Log4j2
@RestController
@RequestMapping("/app/user/security")
@Api(tags="App_安全服务")
public class SecurityAppController {

    @Autowired
    private RedisUtils redisUtils;

    @PostMapping("/getCertificate")
    @ApiOperation(value="获取签名证书",response = SignCertificateModel.class )
    public Result<SignCertificateModel> getCertificate(@RequestBody SignCertificateGetParam params) {
        ValidatorUtils.validateEntity(params);
        String key= CacheKeys.getRsaKey();
        String token = HttpContextUtils.getRequest().getHeader("token");
        if (StringUtils.hasText(token)) {
            key= CacheKeys.getRsaKey(token, true);
        }
        RsaPubPriKeyModel model= (RsaPubPriKeyModel) redisUtils.get(key);

        if(model==null) {
            model = SecurityUtils.generateRSAKey(DateUtil.format(new Date(), DatePattern.PURE_DATETIME_FORMAT));
            redisUtils.set(key, model);
            SignCertificateModel data= new SignCertificateModel(model.getReqPubKey(),model.getRespPriKey(),model.getVersion());
            return new Result<SignCertificateModel>().ok(data);
        }

        if(StrUtil.isNotEmpty(params.getVersion()) && params.getVersion().equals(model.getVersion())){
            SignCertificateModel data= new SignCertificateModel(model.getVersion());
            return new Result<SignCertificateModel>().ok(data);
        }
        System.out.println("redis_key:"+ key);
        System.out.println("获取的加密key:"+ JSONObject.toJSONString(model));
        SignCertificateModel data= new SignCertificateModel(model.getReqPubKey(),model.getRespPriKey(),model.getVersion());
        return new Result<SignCertificateModel>().ok(data);
    }
}
