package com.xxh.cloud.basic.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.NoticeInfo;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.NoticeAddDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.NoticeEditDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.NoticeQueryDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.NoticeUpdateStatusDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.NoticeVO;
import com.xxh.cloud.basic.framework.modules.base.service.NoticeInfoService;
import com.xxh.cloud.basic.framework.modules.biz.helper.NoticeMessageConvertHelper;
import com.xxh.cloud.basic.framework.modules.biz.service.NoticeBizService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

@Log4j2
@Service
public class NoticeBizServiceImpl implements NoticeBizService {

    @Autowired
    private NoticeInfoService noticeInfoService;
    @Autowired
    private AdminUserClientService adminUserClientService;

    @Override
    public PageData<NoticeVO> query(NoticeQueryDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        IPage<NoticeInfo> page= noticeInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<NoticeInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .like(StrUtil.isNotEmpty(params.getTitle()),"title",params.getTitle())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .and(i->i.like("to_user_ids",loginUser.getId()).or().eq("to_user_ids", "ALL"))
                        .ge(StrUtil.isNotEmpty(params.getNoticeDateStart()),"create_date",params.getNoticeDateStart())
                        .le(StrUtil.isNotEmpty(params.getNoticeDateEnd()),"create_date",params.getNoticeDateEnd())
                        .orderByDesc("create_date")
        );
        if (page == null || CollUtil.isEmpty(page.getRecords())) {
            return null;
        }

        Set<String> userIds=new HashSet<>();

        page.getRecords().forEach(item->{
            userIds.add(item.getCreator());
            userIds.add(item.getUpdater());
        });

        Map<String, AdminUserBO> userMap = adminUserClientService.getUserMapByIds(userIds);
        PageData<NoticeVO> data= new PageConverter<NoticeInfo, NoticeVO>() {
            @Override
            public NoticeVO converter(NoticeInfo entity) {
                return NoticeMessageConvertHelper.getNoticeVO(entity, userMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public PageData<NoticeVO> queryByAdmin(NoticeQueryDTO params) {
        IPage<NoticeInfo> page= noticeInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<NoticeInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .like(StrUtil.isNotEmpty(params.getTitle()),"title",params.getTitle())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .ge(StrUtil.isNotEmpty(params.getNoticeDateStart()),"create_date",params.getNoticeDateStart())
                        .le(StrUtil.isNotEmpty(params.getNoticeDateEnd()),"create_date",params.getNoticeDateEnd())
                        .orderByDesc("create_date")
        );
        if (page == null || CollUtil.isEmpty(page.getRecords())) {
            return null;
        }
        Set<String> userIds=new HashSet<>();
        page.getRecords().forEach(item->{
            userIds.add(item.getCreator());
            userIds.add(item.getUpdater());
        });

        Map<String, AdminUserBO> userMap = adminUserClientService.getUserMapByIds(userIds);
        PageData<NoticeVO> data= new PageConverter<NoticeInfo, NoticeVO>() {
            @Override
            public NoticeVO converter(NoticeInfo entity) {
                return NoticeMessageConvertHelper.getNoticeVO(entity, userMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public NoticeVO add(NoticeAddDTO params) {
        ValidatorUtils.validateEntity(params);
        NoticeInfo entity = NoticeMessageConvertHelper.getNoticeInfo(params);
        noticeInfoService.save(entity);
        return NoticeMessageConvertHelper.getNoticeVO(entity, new HashMap<>());
    }


    @Override
    public void edit(NoticeEditDTO params) {
        ValidatorUtils.validateEntity(params);
        NoticeInfo entity = noticeInfoService.getById(params.getId());
        if (entity == null || entity.getRowsStatus().equals(YN.NO.getCode())) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        NoticeMessageConvertHelper.getNoticeInfo(params, entity);
        noticeInfoService.updateById(entity);
    }

    @Override
    public void updateStatus(NoticeUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        NoticeInfo entity = noticeInfoService.getById(params.getId());
        if (entity == null || entity.getRowsStatus().equals(YN.NO.getCode())) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setStatus(params.getStatus());
        noticeInfoService.updateById(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        NoticeInfo entity = noticeInfoService.getById(id);
        if (entity == null || entity.getRowsStatus().equals(YN.NO.getCode())) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        noticeInfoService.updateById(entity);
    }

    @Override
    public NoticeVO getById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        NoticeInfo entity = noticeInfoService.getById(id);
        if(entity==null || entity.getRowsStatus().equals(YN.NO.getCode())){
            return new NoticeVO();
        }
        Set<String> userIds=new HashSet<>();
        userIds.add(entity.getCreator());
        userIds.add(entity.getUpdater());
        Map<String, AdminUserBO> userMap = adminUserClientService.getUserMapByIds(userIds);
        return NoticeMessageConvertHelper.getNoticeVO(entity, userMap);
    }
}
