package com.xxh.cloud.basic.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.Params;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.ParamsAddDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.ParamsListDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.ParamsQueryDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.ParamsUpdateDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.ParamsVO;
import com.xxh.cloud.basic.framework.modules.base.service.ParamsService;
import com.xxh.cloud.basic.framework.modules.biz.helper.ParamsConvertHelper;
import com.xxh.cloud.basic.framework.modules.biz.service.ParamsBizService;
import com.xxh.cloud.basic.framework.modules.cache.helper.CacheBasicKeyHelper;
import com.xxh.cloud.basic.framework.modules.cache.service.ParamsCacheService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class ParamsBizServiceImpl implements ParamsBizService {

    @Autowired
    private ParamsService paramsService;
    @Autowired
    private ParamsCacheService paramsCacheService;

    @Override
    public PageData<ParamsVO> query(ParamsQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<Params> page= paramsService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<Params>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getParamType()),"param_type",params.getParamType())
                        .eq(StrUtil.isNotEmpty(params.getParamValueType()),"param_value_type",params.getParamValueType())
                        .in(CollUtil.isNotEmpty(params.getParamValueTypeList()),"param_value_type",params.getParamValueTypeList())
                        .orderByDesc("create_date")
        );
        PageData<ParamsVO> data= new PageConverter<Params, ParamsVO>() {
            @Override
            public ParamsVO converter(Params entity) {
                return ParamsConvertHelper.getParamsVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<ParamsVO> list(ParamsListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<Params> list= paramsService.list(
                new QueryWrapper<Params>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .in(CollUtil.isNotEmpty(params.getIds()),"id",params.getIds())
                        .eq(StrUtil.isNotEmpty(params.getParamType()),"param_type",params.getParamType())
                        .eq(StrUtil.isNotEmpty(params.getParamValueType()),"param_value_type",params.getParamValueType())
                        .in(CollUtil.isNotEmpty(params.getParamValueTypeList()),"param_value_type",params.getParamValueTypeList())
                        .orderByDesc("create_date")
        );

        List<ParamsVO> data=new ArrayList<>();
        if(CollUtil.isNotEmpty(list)){
            list.forEach(item->{
                data.add(ParamsConvertHelper.getParamsVOFull(item));
            });
        }
        return data;
    }

    @Override
    public ParamsVO add(ParamsAddDTO params) {
        ValidatorUtils.validateEntity(params);
        //todo: 判断参数编号
        Params entity=new Params();
        entity.setId(params.getId());
        entity.setParamType(params.getParamType());
        entity.setParamValueType(params.getParamValueType());
        if(MapUtil.isNotEmpty(params.getParamValue())){
            entity.setParamValue(JSON.toJSONString(params.getParamValue()));
        }
        entity.setRowsStatus(YN.YES.getCode());
        paramsService.save(entity);
        return ParamsConvertHelper.getParamsVO(entity);
    }

    @Override
    public ParamsVO update(ParamsUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        Params entity= paramsService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(MapUtil.isNotEmpty(params.getParamValue())){
            entity.setParamValue(JSON.toJSONString(params.getParamValue()));
        }
        paramsService.updateById(entity);
        // 清缓存
        paramsCacheService.removeById(entity.getId());
        return ParamsConvertHelper.getParamsVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Params entity= paramsService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        paramsService.updateById(entity);
        // 清缓存
        paramsCacheService.removeById(entity.getId());
    }

    @Override
    public ParamsVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Params entity= paramsService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return ParamsConvertHelper.getParamsVOFull(entity);
    }




}
