package com.xxh.cloud.basic.framework.modules.biz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.Company;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.NoticeInfo;
import com.xxh.cloud.basic.framework.modules.base.entity.dos.VisitVerificationConfig;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.VisitVerificationAddDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.VisitVerificationEditDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.VisitVerificationGetDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.VisitVerificationQueryDTO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.CompanyVO;
import com.xxh.cloud.basic.framework.modules.base.service.VisitVerificationConfigService;
import com.xxh.cloud.basic.framework.modules.biz.entity.PatientRecord;
import com.xxh.cloud.basic.framework.modules.biz.helper.CompanyConvertHelper;
import com.xxh.cloud.basic.framework.modules.biz.helper.PatientRecordApiHelper;
import com.xxh.cloud.basic.framework.modules.biz.helper.VisitVerificationConvertHelper;
import com.xxh.cloud.basic.framework.modules.biz.mapper.TaskPatientBasicMapper;
import com.xxh.cloud.basic.framework.modules.biz.service.VisitVerificationConfigBizService;
import com.xxh.cloud.basic.framework.modules.cache.service.VisitVerificationCacheService;
import com.xxh.cloud.basic.framework.modules.constant.BasicVisitConstant;
import com.xxh.cloud.framework.common.constant.DataSourceTypeEnum;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.logging.Handler;
import java.util.stream.Collectors;

@Log4j2
@Service
public class VisitVerificationConfigBizServiceImpl implements VisitVerificationConfigBizService {

    @Autowired
    private VisitVerificationConfigService visitVerificationConfigService;
    @Resource
    private TaskPatientBasicMapper taskPatientBasicMapper;

    @Override
    public void syncPatientRecordBlackList() {
        // 调用API查询
        List<PatientRecord> records = PatientRecordApiHelper.patientRecordBlackList();
        if (CollUtil.isEmpty(records)) {
            return;
        }
        List<VisitVerificationConfig> entityList = new ArrayList<>();
        for (PatientRecord record : records) {
            if (StrUtil.isNotEmpty(record.getPatientId())) {
                VisitVerificationConfig entity = new VisitVerificationConfig();
                entity.setId(IdUtil.getSnowflake(7,2).nextIdStr());
                entity.setVisitType(BasicVisitConstant.VisitType.PATIENT.getCode());
                entity.setVisitId(record.getPatientId());
                entity.setVerType(BasicVisitConstant.VerType.BLACK_LIST.getCode());
                entity.setAppScope(BasicVisitConstant.AppScope.ALL.getCode());
                entity.setBizScope(BasicVisitConstant.BizScope.ALL.getCode());
                entity.setDataSource(BasicVisitConstant.DataSource.WBMD_BLACK_LIST.getCode());
                entity.setRowsStatus(YN.YES.getCode());
                entityList.add(entity);
            }
            if (StrUtil.isNotEmpty(record.getIdCardNo())) {
                VisitVerificationConfig entity = new VisitVerificationConfig();
                entity.setId(IdUtil.getSnowflake(7,2).nextIdStr());
                entity.setVisitType(BasicVisitConstant.VisitType.ID_CARD.getCode());
                entity.setVisitId(record.getIdCardNo());
                entity.setVerType(BasicVisitConstant.VerType.BLACK_LIST.getCode());
                entity.setAppScope(BasicVisitConstant.AppScope.ALL.getCode());
                entity.setBizScope(BasicVisitConstant.BizScope.ALL.getCode());
                entity.setDataSource(BasicVisitConstant.DataSource.WBMD_BLACK_LIST.getCode());
                entity.setRowsStatus(YN.YES.getCode());
                entityList.add(entity);
            }
        }
        // 删除
        visitVerificationConfigService.remove(new QueryWrapper<VisitVerificationConfig>()
                .eq("ver_type", BasicVisitConstant.VerType.BLACK_LIST.getCode())
                .eq("data_source", BasicVisitConstant.DataSource.WBMD_BLACK_LIST.getCode())
        );
        // 新增
        if (CollUtil.isNotEmpty(entityList)) {
            visitVerificationConfigService.saveBatch(entityList);
        }

    }

    @Override
    public void syncPatient4IdCard() {
        List<VisitVerificationConfig> entityList = visitVerificationConfigService.list(new QueryWrapper<VisitVerificationConfig>()
                .eq("visit_type", BasicVisitConstant.VisitType.ID_CARD.getCode()));
        if (CollUtil.isEmpty(entityList)) {
            return;
        }
        Set<String> idCardSet = new HashSet<>();
        entityList.forEach(item->{
            idCardSet.add(item.getVisitId());
        });
        // 查询关联的患者id
        List<PatientRecord> patients = taskPatientBasicMapper.listTaskPatient(new QueryWrapper<PatientRecord>()
                .in("id_card_no", idCardSet));
        Map<String, List<PatientRecord>> patientIdCardMap = new HashMap<>();
        if (CollUtil.isEmpty(patients)) {
            return;
        }
        patients.forEach(p->{
            List<PatientRecord> l;
            if (patientIdCardMap.containsKey(p.getIdCardNo())) {
                l = patientIdCardMap.get(p.getIdCardNo());
            } else {
                l = new ArrayList<>();
            }
            l.add(p);
            patientIdCardMap.put(p.getIdCardNo(), l);
        });

        List<VisitVerificationConfig> saveEntityList = new ArrayList<>();
        for (VisitVerificationConfig entity : entityList) {
            if (patientIdCardMap.containsKey(entity.getVisitId())) {
                List<PatientRecord> patientList = patientIdCardMap.get(entity.getVisitId());
                for (PatientRecord record : patientList) {
                    long exist = visitVerificationConfigService.count(new QueryWrapper<VisitVerificationConfig>()
                            .eq("visit_type", BasicVisitConstant.VisitType.PATIENT.getCode())
                            .eq("visit_id", record.getPatientId())
                            .eq("ver_type", entity.getVerType()));
                    // 根据身份证新增患者
                    if (exist <= 0) {
                        VisitVerificationConfig saveEntity = new VisitVerificationConfig();
                        BeanUtil.copyProperties(entity, saveEntity); // 复制相同的规则
                        saveEntity.setId(IdUtil.getSnowflake(7,2).nextIdStr());
                        saveEntity.setVisitType(BasicVisitConstant.VisitType.PATIENT.getCode());
                        saveEntity.setVisitId(record.getPatientId());
                        saveEntityList.add(saveEntity);
                    }
                }
            }
        }
        if (CollUtil.isNotEmpty(saveEntityList)) {
            visitVerificationConfigService.saveBatch(saveEntityList);
        }
    }

    @Override
    public void save(VisitVerificationAddDTO params) {
        VisitVerificationConfig entity = new VisitVerificationConfig();
        entity.setId(IdUtil.getSnowflake(9,3).nextIdStr());
        entity.setVisitType(params.getVisitType());
        entity.setVisitId(params.getVisitId());
        entity.setVerType(params.getVerType());
        entity.setAppScope(params.getAppScope());
        entity.setBizScope(params.getBizScope());
        entity.setVerificationConfig(params.getVerificationConfig());
        entity.setDataSource(BasicVisitConstant.DataSource.SYS_ADD.getCode());
        entity.setRowsStatus(YN.YES.getCode());
        visitVerificationConfigService.save(entity);
    }

    @Override
    public void edit(VisitVerificationEditDTO params) {
        VisitVerificationConfig entity = visitVerificationConfigService.getById(params.getId());
        if (null == entity || YN.NO.getCode().equals(entity.getRowsStatus())) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setAppScope(params.getAppScope());
        entity.setBizScope(params.getBizScope());
        entity.setVerificationConfig(params.getVerificationConfig());
        visitVerificationConfigService.updateById(entity);
    }

    @Override
    public void del(VisitVerificationGetDTO params) {
        visitVerificationConfigService.removeById(params.getId());
    }

    @Override
    public PageData<VisitVerificationBO> query(VisitVerificationQueryDTO params) {
        IPage<VisitVerificationConfig> page= visitVerificationConfigService.page(new Page<>(params.getPage(), params.getRows()), new QueryWrapper<VisitVerificationConfig>()
                .eq("rows_status", YN.YES.getCode())
                .eq(StrUtil.isNotEmpty(params.getVisitType()),"visit_type",params.getVisitType())
                .eq(StrUtil.isNotEmpty(params.getVisitId()),"visit_id",params.getVisitId())
                .eq(StrUtil.isNotEmpty(params.getVerType()),"ver_type",params.getVerType())
                .orderByDesc("create_date")
        );
        PageData<VisitVerificationBO> data= new PageConverter<VisitVerificationConfig, VisitVerificationBO>() {
            @Override
            public VisitVerificationBO converter(VisitVerificationConfig entity) {
                return VisitVerificationConvertHelper.getVisitVerificationVO(entity);
            }
        }.getPageData(page);
        return data;
    }


}
