package com.xxh.cloud.basic.framework.modules.client.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.basic.client.api.bean.bos.ParamsBO;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.ParamsVO;
import com.xxh.cloud.basic.framework.modules.cache.service.ParamsCacheService;
import com.xxh.cloud.basic.framework.modules.client.helper.BasicClientConvertHelper;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.constant.ParamValTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class ParamsClientLocalService implements ParamsClientService {

    @Autowired
    private ParamsCacheService paramsCacheService;

    @Override
    public ParamsBO getParamsById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        ParamsVO vo = paramsCacheService.getById(id);
        return BasicClientConvertHelper.getParamsBO(vo);
    }

    @Override
    public List<ParamsBO> getParamsListByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        List<ParamsVO> list = paramsCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(BasicClientConvertHelper::getParamsBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, ParamsBO> getParamsMapByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new HashMap<>();
        }
        List<ParamsVO> list = paramsCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        return new HashMap<>(list.stream().collect(Collectors.toMap(ParamsVO::getId, BasicClientConvertHelper::getParamsBO)));
    }

    @Override
    public <T> T getParamConfig(String paramValType, T bean) {
        ParamsBO params= this.getParamsById(paramValType);
        if(params==null){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR);
        }
        if(MapUtil.isEmpty(params.getParamValue())){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR);
        }
        T config= BeanUtil.fillBeanWithMap(params.getParamValue(),bean,false);
        return config;
    }

    @Override
    public <T> T getParamConfig(String id, String paramValType, T bean) {
        ParamsBO params= this.getParamsById(id);
        if(params==null){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR);
        }
        if(!params.getParamValueType().equals(paramValType)){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR);
        }
        if(MapUtil.isEmpty(params.getParamValue())){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR);
        }
        T config= BeanUtil.fillBeanWithMap(params.getParamValue(),bean,false);
        return config;
    }

    @Override
    public <T> T getParamConfig(Map<String, Object> valMap, T bean) {
        if(MapUtil.isEmpty(valMap)){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR);
        }
        T config= BeanUtil.fillBeanWithMap(valMap,bean,false);
        return config;
    }

}
