package com.xxh.cloud.basic.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.basic.client.api.VisitVerificationClientService;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.client.api.bean.dto.VisitVerificationSearchDTO;
import com.xxh.cloud.basic.framework.modules.biz.entity.PatientRecord;
import com.xxh.cloud.basic.framework.modules.biz.helper.VisitVerificationConvertHelper;
import com.xxh.cloud.basic.framework.modules.biz.mapper.TaskPatientBasicMapper;
import com.xxh.cloud.basic.framework.modules.cache.service.VisitVerificationCacheService;
import com.xxh.cloud.basic.framework.modules.constant.BasicVisitConstant;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class VisitVerificationClientLocalService implements VisitVerificationClientService {

    @Autowired
    private VisitVerificationCacheService visitVerificationCacheService;
    @Resource
    private TaskPatientBasicMapper taskPatientBasicMapper;
    @Autowired
    private RedisUtils redisUtils;

    @Override
    public Map<String, List<VisitVerificationBO>> getVisitVerification(VisitVerificationSearchDTO params) {
        return visitVerificationCacheService.getVisitVerification(params);
    }

    @Override
    public Map<String, List<VisitVerificationBO>> getVisitVerificationByPatient(List<String> patientIds) {
        VisitVerificationSearchDTO dto = new VisitVerificationSearchDTO(BasicVisitConstant.VisitType.PATIENT.getCode(), patientIds);
        Map<String, List<VisitVerificationBO>> map = visitVerificationCacheService.getVisitVerification(dto);
        for (String patientId : patientIds) {
            if (MapUtil.isEmpty(map) || !map.containsKey(patientId)) {
                // 再校验对应的身份证
                String idCard = this.getRelIdCard(patientId);
                VisitVerificationSearchDTO dtoByIdCard = new VisitVerificationSearchDTO(BasicVisitConstant.VisitType.ID_CARD.getCode(), idCard);
                Map<String, List<VisitVerificationBO>> mapIdCard = visitVerificationCacheService.getVisitVerification(dtoByIdCard);
                if (MapUtil.isNotEmpty(mapIdCard)) {
                    mapIdCard.forEach((k, v) -> {
                        map.put(patientId, v);
                    });
                }
            }
        }
        return map;
    }

    private String getRelIdCard(String patientId) {
        String cacheKey = VisitVerificationConvertHelper.getPatientRelIdCardCacheKey(patientId);
        if (redisUtils.hasKey(cacheKey)) {
            return (String) redisUtils.get(cacheKey);
        } else {
            List<PatientRecord> patientRecords = taskPatientBasicMapper.listTaskPatient(new QueryWrapper<PatientRecord>()
                    .eq("patient_id", patientId)
            );
            if (CollUtil.isNotEmpty(patientRecords)) {
                String idCard = patientRecords.get(0).getIdCardNo();
                redisUtils.set(cacheKey, idCard, 24 * 3600);
                return idCard;
            } else {
                redisUtils.set(cacheKey, "empty", 24 * 3600);
                return "";
            }
        }
    }
}
