package com.xxh.cloud.basic.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.basic.framework.modules.biz.service.CompanyBizService;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.CompanyVO;
import com.xxh.cloud.basic.framework.modules.constant.CompanySysType;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:basic:company")
@RestController
@RequestMapping("/sys/basic/company")
@Api(tags="运营端_管理,机构管理")
public class CompanyManagerController {

    @Autowired
    private CompanyBizService companyBizService;


    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:basic:company:query")
    public Result<PageData<CompanyVO>> query(@RequestBody CompanyQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<CompanyVO> data= companyBizService.query(params);
        return new Result<PageData<CompanyVO>>().ok(data);
    }

    @PostMapping("tree")
    @ApiOperation("树)")
    @SaCheckPermission("sys:basic:company:tree")
    public Result<List<CompanyVO>> tree(@RequestBody CompanyTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<CompanyVO> data= companyBizService.tree(params);
        return new Result<List<CompanyVO>>().ok(data);
    }

    @PostMapping("add")
    @ApiOperation("新增")
    @SaCheckPermission("sys:basic:company:add")
    public Result<CompanyVO> add(@RequestBody CompanyAddDTO params) {
        if(StrUtil.isEmpty(params.getSysType())){
            params.setSysType(CompanySysType.Company_Merchant.getCode());
        }
        if(StrUtil.isEmpty(params.getId())){
            params.setId(IdUtil.getSnowflake(2,1).nextIdStr());
        }
        ValidatorUtils.validateEntity(params);
        CompanyVO data= companyBizService.add(params);
        return new Result<CompanyVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改")
    @SaCheckPermission("sys:basic:company:update")
    public Result<CompanyVO> update(@RequestBody CompanyUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        CompanyVO data= companyBizService.update(params);
        return new Result<CompanyVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:basic:company:info")
    public Result<CompanyVO> info(@RequestBody CompanyInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        CompanyVO data=companyBizService.infoById(params.getId());
        return new Result<CompanyVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:basic:company:del")
    public Result<String> del(@RequestBody CompanyInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        companyBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }



}
