package com.xxh.cloud.basic.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.basic.framework.modules.biz.service.NoticeBizService;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.NoticeVO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

//@SaCheckPermission("sys:basic:notice")
@RestController
@RequestMapping("/sys/basic/notice")
@Api(tags="运营端_管理,公告管理")
public class NoticeManagerController {

    @Autowired
    private NoticeBizService noticeBizService;



    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:basic:notice:query")
    public Result<PageData<NoticeVO>> query(@RequestBody NoticeQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<NoticeVO> data= noticeBizService.query(params);
        return new Result<PageData<NoticeVO>>().ok(data);
    }

    @PostMapping("queryByAdmin")
    @ApiOperation("查询ByAdmin")
    @SaCheckPermission("sys:basic:notice:queryByAdmin")
    public Result<PageData<NoticeVO>> queryByAdmin(@RequestBody NoticeQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<NoticeVO> data= noticeBizService.query(params);
        return new Result<PageData<NoticeVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增")
    @SaCheckPermission("sys:basic:notice:add")
    public Result<NoticeVO> add(@RequestBody NoticeAddDTO params) {
        ValidatorUtils.validateEntity(params);
        NoticeVO data= noticeBizService.add(params);
        return new Result<NoticeVO>().ok(data);
    }



    @PostMapping("updateStatus")
    @ApiOperation("修改状态")
    @SaCheckPermission("sys:basic:notice:updateStatus")
    public Result<String> updateStatus(@RequestBody NoticeUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        noticeBizService.updateStatus(params);
        return new Result<String>().ok("ok");
    }


    @PostMapping("update")
    @ApiOperation("修改")
    @SaCheckPermission("sys:basic:notice:update")
    public Result<String> update(@RequestBody NoticeEditDTO params) {
        ValidatorUtils.validateEntity(params);
        noticeBizService.edit(params);
        return new Result<String>().ok("ok");
    }


    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:basic:notice:info")
    public Result<NoticeVO> info(@RequestBody NoticeGetDTO params) {
        ValidatorUtils.validateEntity(params);
        NoticeVO data=noticeBizService.getById(params.getId());
        return new Result<NoticeVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:basic:notice:del")
    public Result<String> del(@RequestBody NoticeGetDTO params) {
        ValidatorUtils.validateEntity(params);
        noticeBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }




}
