package com.xxh.cloud.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.basic.framework.modules.biz.service.ProjectBizService;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.ProjectVO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


//@SaCheckPermission("sys:basic:project")
@RestController
@RequestMapping("/sys/basic/project")
@Api(tags="运营端_管理,项目管理")
public class ProjectManagerController {

    @Autowired
    private ProjectBizService projectBizService;

    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:basic:project:query")
    public Result<PageData<ProjectVO>> query(@RequestBody ProjectInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<ProjectVO> data= projectBizService.query(params);
        return new Result<PageData<ProjectVO>>().ok(data);
    }


    @PostMapping("list")
    @ApiOperation("查询")
    @SaCheckPermission("sys:basic:project:list")
    public Result<List<ProjectVO>> list(@RequestBody ProjectInfoListDTO params) {
        ValidatorUtils.validateEntity(params);
        List<ProjectVO> data= projectBizService.list(params);
        return new Result<List<ProjectVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增")
    @SaCheckPermission("sys:basic:project:add")
    public Result<String> add(@RequestBody ProjectInfoAddDTO params) {
        ValidatorUtils.validateEntity(params);
        projectBizService.add(params);
        return new Result<String>().ok("ok");
    }


    @PostMapping("update")
    @ApiOperation("修改")
    @SaCheckPermission("sys:basic:project:update")
    public Result<String> update(@RequestBody ProjectInfoEditDTO params) {
        ValidatorUtils.validateEntity(params);
        projectBizService.update(params);
        return new Result<String>().ok("ok");
    }

    @PostMapping("updateStatus")
    @ApiOperation("修改状态")
    @SaCheckPermission("sys:basic:project:updateStatus")
    public Result<String> updateStatus(@RequestBody ProjectInfoEditStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        projectBizService.updateStatus(params);
        return new Result<String>().ok("ok");
    }


    @PostMapping("updateDef")
    @ApiOperation("修改默认状态")
    @SaCheckPermission("sys:basic:project:updateDef")
    public Result<String> updateDef(@RequestBody ProjectInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        projectBizService.updateDef(params.getCode());
        return new Result<String>().ok("ok");
    }




    @PostMapping("updateDataKey")
    @ApiOperation("修改数据KEY")
    @SaCheckPermission("sys:basic:project:updateDataKey")
    public Result<String> updateDataKey(@RequestBody ProjectInfoEditKeyDTO params) {
        ValidatorUtils.validateEntity(params);
        projectBizService.updateDataKey(params);
        return new Result<String>().ok("ok");
    }



    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:basic:project:info")
    public Result<ProjectVO> info(@RequestBody ProjectInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        ProjectVO data=projectBizService.getById(params.getCode());
        return new Result<ProjectVO>().ok(data);
    }

    @PostMapping("infoDataKey")
    @ApiOperation("获取数据KEY")
    @SaCheckPermission("sys:basic:project:infoDataKey")
    public Result<ProjectVO> infoDataKey(@RequestBody ProjectInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        ProjectVO data=projectBizService.getDataKeyById(params.getCode());
        return new Result<ProjectVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:basic:project:del")
    public Result<String> del(@RequestBody ProjectInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        projectBizService.delById(params.getCode());
        return new Result<String>().ok("ok");
    }

}
