package com.xxh.cloud.basic.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.basic.framework.modules.biz.service.SiteMessageBizService;
import com.xxh.cloud.basic.framework.modules.base.entity.dto.*;
import com.xxh.cloud.basic.framework.modules.base.entity.vos.SiteMessageVO;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

//@SaCheckPermission("sys:basic:siteMessage")
@RestController
@RequestMapping("/sys/basic/siteMessage")
@Api(tags="运营端_管理,站内信息管理")
public class SiteMessageManagerController {

    @Autowired
    private SiteMessageBizService siteMessageBizService;


    @PostMapping("query")
    @ApiOperation(value = "查询",response =SiteMessageVO.class)
    @SaCheckPermission("sys:basic:siteMessage:query")
    public Result<PageData<SiteMessageVO>> query(@RequestBody SiteMessageQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<SiteMessageVO> data= siteMessageBizService.query(params);
        return new Result<PageData<SiteMessageVO>>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation(value ="查看",response =SiteMessageVO.class)
    @SaCheckPermission("sys:basic:siteMessage:info")
    public Result<SiteMessageVO> info(@RequestBody SiteMessageByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        SiteMessageVO data= siteMessageBizService.getById(params.getId());
        return new Result<SiteMessageVO>().ok(data);
    }



    @PostMapping("updateReadStatus")
    @ApiOperation("更新读取状态")
    @SaCheckPermission("sys:basic:siteMessage:updateReadStatus")
    public Result<String> updateReadStatus(@RequestBody SiteMessageByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        siteMessageBizService.updateReadStatus(params.getId());
        return new Result<String>().ok("ok");
    }


    @PostMapping("updateReadStatusBatch")
    @ApiOperation("批量更新读取状态")
    @SaCheckPermission("sys:basic:siteMessage:updateReadStatusBatch")
    public Result<String> updateReadStatusBatch(@RequestBody SiteMessageByIdsDTO params) {
        ValidatorUtils.validateEntity(params);
        siteMessageBizService.updateReadStatusBatch(params.getIds());
        return new Result<String>().ok("ok");
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:basic:siteMessage:del")
    public Result<String> del(@RequestBody SiteMessageByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        siteMessageBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }

    @PostMapping("delBatch")
    @ApiOperation("批量删除")
    @SaCheckPermission("sys:basic:siteMessage:delBatch")
    public Result<String> delBatch(@RequestBody SiteMessageByIdsDTO params) {
        ValidatorUtils.validateEntity(params);
        siteMessageBizService.delByIds(params.getIds());
        return new Result<String>().ok("ok");
    }








}
