package com.xxh.cloud.customer.application.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.AddressAddDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.AddressInfoByIdDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.AddressUpdateDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.AddressUpdateDefDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.vos.AddressVO;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerAddrInfoService;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerAddrBizService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@SaCheckLogin
@RestController
@RequestMapping("/app/customer/addr")
@Api(tags="客户端,客户地址服务")
public class CustomerAddrAppController {


    @Autowired
    private CustomerAddrBizService customerAddrBizService;

    @PostMapping("list")
    @ApiOperation("我的地址列表")
    public Result<List<AddressVO>> list() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        List<AddressVO> data=customerAddrBizService.listByUser(loginUser.getId());
        return new Result<List<AddressVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增地址")
    public Result<AddressVO> add(@RequestBody AddressAddDTO params) {
        LoginUser loginUser=  AuthContextUtils.getCurrentLoginUserCheck();
        params.setUserId(loginUser.getId());
        params.setCompanyNo(loginUser.getCompanyNo());
        ValidatorUtils.validateEntity(params);
        AddressVO data= customerAddrBizService.add(params);
        return new Result<AddressVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改地址")
    public Result<AddressVO> update(@RequestBody AddressUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        AddressVO data= customerAddrBizService.update(params);
        return new Result<AddressVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取地址")
    public Result<AddressVO> info(@RequestBody AddressInfoByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        AddressVO data= customerAddrBizService.infoById(params.getId());
        return new Result<AddressVO>().ok(data);
    }

    @PostMapping("del")
    @ApiOperation("删除地址")
    public Result<String> del(@RequestBody AddressInfoByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        customerAddrBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }



    @PostMapping("updateDef")
    @ApiOperation("更新默认地址")
    public Result<String> updateDef(@RequestBody AddressUpdateDefDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        customerAddrBizService.updateDef(params);
        return new Result<String>().ok("ok");
    }







}
