package com.xxh.cloud.customer.application.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;

import com.xxh.cloud.basic.client.api.VisitVerificationClientService;
import com.xxh.cloud.basic.client.api.bean.bos.VisitVerificationBO;
import com.xxh.cloud.basic.client.api.bean.dto.VisitVerificationSearchDTO;
import com.xxh.cloud.basic.framework.modules.constant.BasicVisitConstant;
import com.xxh.cloud.customer.client.api.bean.bos.MemberFamilyBO;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.customer.framework.modules.biz.bean.vos.MemberFamilyVO;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerFamilyMemberAuthService;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerFamilyMemberBizService;
import com.xxh.cloud.customer.framework.modules.cache.helper.CacheCustomerKeyHelper;
import com.xxh.cloud.framework.common.config.BaseProjectConfig;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.third.framework.modules.captcha.service.ImageCaptchaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

@SaCheckLogin
@RestController
@RequestMapping("/app/customer/family")
@Api(tags="客户端,客户家庭成员服务")
public class CustomerFamilyMemberAppController {

    @Autowired
    private CustomerFamilyMemberBizService customerFamilyMemberBizService;
    @Autowired
    private CustomerFamilyMemberAuthService customerFamilyMemberAuthService;
    @Autowired
    private ImageCaptchaService imageCaptchaService;

    @Autowired
    private BaseProjectConfig baseProjectConfig;
    @Autowired
    private VisitVerificationClientService visitVerificationClientService;
    @Autowired
    private RedisUtils redisUtils;

    @PostMapping("listMember")
    @ApiOperation("我的家庭成员列表")
    public Result<List<MemberFamilyVO>> listMember() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        
        // zs20240702 查询前同步互联网医院家庭数据
        customerFamilyMemberBizService.addFromHis(loginUser);

        MemberFamilyListDTO params=new MemberFamilyListDTO();
        params.setUserId(loginUser.getId());
        List<MemberFamilyVO> data= customerFamilyMemberBizService.list(params);

        return new Result<List<MemberFamilyVO>>().ok(getValidData(loginUser.getId(), data));
    }

    // 排除黑名单家属数据
    private List<MemberFamilyVO> getValidData(String userId, List<MemberFamilyVO> data) {
        List<MemberFamilyVO> list = new ArrayList<>();
        if (CollUtil.isNotEmpty(data)) {
            // 黑名单患者过滤
            List<String> patientIds = new ArrayList<>();
            data.forEach(i -> patientIds.add(i.getPatHisNo()));

            VisitVerificationSearchDTO visitVerificationSearchDTO = new VisitVerificationSearchDTO(BasicVisitConstant.VisitType.PATIENT.getCode(), patientIds);
            Map<String, List<VisitVerificationBO>> verificationMap = visitVerificationClientService.getVisitVerification(visitVerificationSearchDTO);
            Set<String> invalidPatientIds = getInvalidPatientIds(verificationMap);

            data.forEach(i -> {
                if (invalidPatientIds.contains(i.getPatHisNo())) { // 患者在黑名单且被选中则清除选中
                    if (YN.YES.getCode().equals(i.getChooseFlg())) {
                        String redisKey = CacheCustomerKeyHelper.getCustomerFamilyMemberChoose(userId);
                        redisUtils.delete(redisKey);
                    }
                } else {
                    list.add(i);
                }
            });
        }
        return list;
    }

    // 获取黑名单数据
    private Set<String> getInvalidPatientIds(Map<String, List<VisitVerificationBO>> verificationMap) {
        Set<String> invalidPatientIds = new HashSet<>();
        if (CollUtil.isNotEmpty(verificationMap)) {
            verificationMap.forEach((k, v) -> {
                for (VisitVerificationBO bo : v) {
                    if (BasicVisitConstant.VerType.BLACK_LIST.getCode().equals(bo.getVerType())) {
                        invalidPatientIds.add(k);
                        break;
                    }
                }
            });
        }
        return invalidPatientIds;
    }

    @PostMapping("delMember")
    @ApiOperation("删除家庭成员列表")
    public Result<String> delMember(@RequestBody MemberFamilyInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        customerFamilyMemberBizService.delById(params.getFamilyMemberId());
        return new Result<String>().ok("");
    }

    @PostMapping("addFromHis")
    @ApiOperation("同步家庭成员信息")
    public Result<String> addFromHis() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        customerFamilyMemberBizService.addFromHis(loginUser);
        return new Result<String>().ok("OK");
    }

    @PostMapping("sendAuthCode")
    @ApiOperation("发送家人授权验证码")
    public Result<String> sendAuthCode(@RequestBody MemberFamilyAuthSmsDTO params) {
        ValidatorUtils.validateEntity(params);
        // 校验图片验证码
        imageCaptchaService.validate(params.getCaptchaVerification(),params.getCaptchaCode());
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        customerFamilyMemberBizService.sendAuthCode(loginUser, params);
        return new Result<String>().ok("OK");
    }

    @PostMapping("auth")
    @ApiOperation("确认/取消授权")
    public Result<String> auth(@RequestBody MemberFamilyAuthDTO params) {
        if(baseProjectConfig.isCustFamilyBindAuth()){
            if(StrUtil.isEmpty(params.getAuthCode())){
                throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"授权码不能为空");
            }
            customerFamilyMemberAuthService.auth(params);
        }else {
            customerFamilyMemberAuthService.bind(params);
        }
        return new Result<String>().ok("OK");
    }



    @PostMapping("choose")
    @ApiOperation("选中家庭成员")
    public Result<String> choose(@RequestBody MemberFamilyInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        customerFamilyMemberBizService.choose(loginUser, params);
        return new Result<String>().ok("OK");
    }

    @PostMapping("addMember")
    @ApiOperation("新增家庭成员列表")
    public Result<MemberFamilyVO> addMember(@RequestBody MemberFamilyAddDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
//        customerFamilyMemberInfoService.add()
//        MemberFamilyListDTO params=new MemberFamilyListDTO();
//        List<MemberFamilyVO> data= customerFamilyMemberInfoService.list(params);
        return new Result<MemberFamilyVO>().ok(null);
    }

    @PostMapping("updateMember")
    @ApiOperation("修改家庭成员列表")
    public Result<MemberFamilyVO> updateMember(@RequestBody MemberFamilyUpdateDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
//        customerFamilyMemberInfoService.add()
//        MemberFamilyListDTO params=new MemberFamilyListDTO();
//        List<MemberFamilyVO> data= customerFamilyMemberInfoService.list(params);
        return new Result<MemberFamilyVO>().ok(null);
    }


    @PostMapping("unbindMember")
    @ApiOperation("解绑家庭成员列表")
    public Result<String> Member(@RequestBody MemberFamilyInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        customerFamilyMemberBizService.unbindById(params.getFamilyMemberId());
        return new Result<String>().success();
    }



}
