package com.xxh.cloud.customer.application.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.customer.framework.modules.check.service.CustomerUserValidateService;
import com.xxh.cloud.customer.framework.modules.login.entity.dto.BindByMobileDTO;
import com.xxh.cloud.customer.framework.modules.login.entity.dto.CustomerUserBindThirdByMobileCodeDTO;
import com.xxh.cloud.customer.framework.modules.login.entity.dto.CustomerUserBindThirdByWeixinMiniMobileDTO;
import com.xxh.cloud.customer.framework.modules.login.entity.vos.CustomerUserLoginVO;
import com.xxh.cloud.customer.framework.modules.login.service.CustomerUserLoginThirdService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.third.framework.modules.captcha.entity.enums.SmsCaptchaBizTypeEnum;
import com.xxh.cloud.third.framework.modules.captcha.service.SmsCaptchaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@RestController
@RequestMapping("/app/customer/thirdUser")
@Api(tags="客户端,三方用户服务")
public class CustomerThirdUserAppController {


    @Autowired
    private SmsCaptchaService smsCaptchaService;
    @Autowired
    private CustomerUserLoginThirdService customerUserLoginThirdService;

    @Autowired
    private CustomerUserValidateService customerUserValidateService;



    @PostMapping("bindByMobileCode")
    @ApiOperation("三方手机号验证码绑定用户")
    public Result<CustomerUserLoginVO> bindByMobileCode(@RequestBody CustomerUserBindThirdByMobileCodeDTO params) {
        ValidatorUtils.validateEntity(params);
        customerUserValidateService.checkThirdLogin();
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        if(StrUtil.isEmpty(loginUser.getThirdUserId())){
            throw  new BusinessException(1,"3third");
        }
        smsCaptchaService.validate(SmsCaptchaBizTypeEnum.SC_Bind.getCode(),params.getMobile(),params.getCode());
        CustomerUserLoginVO data= customerUserLoginThirdService.bind(new BindByMobileDTO(loginUser.getThirdUserId(),
                params.getMobile(),
                customerUserValidateService.isRegThirdBindAfter()));
        AuthContextUtils.logoutByToken(loginUser.getToken());
        String token= AuthContextUtils.login(data.getCompanyNo(),null,data.getId(),data.getUserName(),data.getThirdId());
        data.setToken(token);
        return new Result<CustomerUserLoginVO>().ok(data);
    }




    @PostMapping("bindByWxMiniMobile")
    @ApiOperation("三方微信手机号加密方式绑定用户")
    public Result<CustomerUserLoginVO> bindByWxMiniMobile(@RequestBody CustomerUserBindThirdByWeixinMiniMobileDTO params) {
        ValidatorUtils.validateEntity(params);
        customerUserValidateService.checkThirdLogin();
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        if(StrUtil.isEmpty(loginUser.getThirdUserId())){
            throw  new BusinessException(1,"3third");
        }
//        smsCaptchaService.validate(SmsCaptchaBizTypeEnum.SC_Bind.getCode(),params.getMobile(),params.getCode());
//        CustomerUserLoginVO data= customerUserLoginThirdService.bind(new BindByMobileDTO(loginUser.getThirdUserId(),
//                params.getMobile(),
//                customerUserValidateService.isRegThirdBindAfter()));
//        AuthContextUtils.logoutByToken(loginUser.getToken());
//        String token= AuthContextUtils.login(data.getCompanyNo(),null,data.getId(),data.getThirdId());
//        data.setToken(token);
        return new Result<CustomerUserLoginVO>().ok(null);
    }







}
