package com.xxh.cloud.customer.application.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.annotation.SaIgnore;

import com.xxh.cloud.customer.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.customer.framework.modules.biz.bean.vos.CustomerUserVO;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerUserInfoService;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.third.framework.modules.captcha.entity.enums.SmsCaptchaBizTypeEnum;
import com.xxh.cloud.third.framework.modules.captcha.service.SmsCaptchaService;
import com.xxh.cloud.third.framework.modules.thirdHis.model.GenerateSchemeModel;
import com.xxh.cloud.third.framework.modules.thirdHis.service.ThirdHisService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaCheckLogin
@RestController
@RequestMapping("/app/customer/user")
@Api(tags="客户端,客户用户服务")
public class CustomerUserAppController {

    @Autowired
    private CustomerUserInfoService customerUserInfoService;
    @Autowired
    private SmsCaptchaService smsCaptchaService;
    @Autowired
    private ThirdHisService thirdHisService;

    @PostMapping("getUserInfo")
    @ApiOperation("获取用户信息")
    public Result<CustomerUserVO> getUserInfo() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        CustomerUserVO data= customerUserInfoService.infoById(loginUser.getId(),true);
        return new Result<CustomerUserVO>().ok(data);
    }


    @PostMapping("updateUserInfo")
    @ApiOperation("修改用户信息")
    public Result<CustomerUserVO> updateUserInfo(@RequestBody UserCustomerUpdateByMyDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        UserCustomerUpdateDTO updateParam=new UserCustomerUpdateDTO();
        updateParam.setId(loginUser.getId());
        updateParam.setNickName(params.getNickName());
        updateParam.setRealName(params.getRealName());
        updateParam.setHeadUrl(params.getHeadUrl());
        updateParam.setCustomerExtend(params.getCustomerExtend());
        updateParam.setDesensitized(true);
        CustomerUserVO data= customerUserInfoService.update(updateParam);
        return new Result<CustomerUserVO>().ok(data);
    }


    @PostMapping("updatePassword")
    @ApiOperation("修改密码")
    public Result<CustomerUserVO> updatePassword(@RequestBody UserCustomerUpdatePasswordDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        params.setId(loginUser.getId());
        params.setDesensitized(true);
        CustomerUserVO data= customerUserInfoService.updatePassword(params);
        return new Result<CustomerUserVO>().ok(data);
    }


    @PostMapping("updateMobile")
    @ApiOperation("修改手机号")
    public Result<CustomerUserVO> updateMobile(@RequestBody UserCustomerUpdateMobileDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUserCheck();
        params.setId(loginUser.getId());
        params.setDesensitized(true);
        smsCaptchaService.validate(SmsCaptchaBizTypeEnum.SC_UpdateMobile.getCode(),params.getMobile(),params.getCode());
        CustomerUserVO data= customerUserInfoService.updateMobile(params);
        return new Result<CustomerUserVO>().ok(data);
    }

    @SaIgnore
    @PostMapping("logout")
    @ApiOperation("退出登录")
    public Result<String> logout() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        if (null != loginUser) {
            AuthContextUtils.logout(loginUser.getId());
        }
        return new Result<String>().ok("logout ok");
    }

    @PostMapping("getSchemeUrl")
    @ApiOperation("获取schemeUrl")
    public Result<String> getSchemeUrl(@RequestBody SchemeUrlDTO params) {
        GenerateSchemeModel date = thirdHisService.generateScheme(params.getDeptCode(), params.getDoctorCode());
        return new Result<String>().ok(date.getOpenlink());
    }
}
