package com.xxh.cloud.customer.application.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.annotation.SaIgnore;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerThirdUserInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerThirdUserInfoService;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerFamilyMemberBizService;
import com.xxh.cloud.customer.framework.modules.check.service.CustomerUserValidateService;
import com.xxh.cloud.customer.framework.modules.login.entity.dto.BindByMobileDTO;
import com.xxh.cloud.customer.framework.modules.login.entity.dto.CustomerUserLoginThirdDTO;
import com.xxh.cloud.customer.framework.modules.login.entity.vos.CustomerUserLoginVO;
import com.xxh.cloud.customer.framework.modules.login.service.CustomerUserLoginThirdService;
import com.xxh.cloud.framework.common.config.ProjectConfig;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.utils.HttpContextUtils;
import com.xxh.cloud.framework.common.utils.IpUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.bean.constant.LogLoginOutTypeEnum;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.LoginLogAddDTO;
import com.xxh.cloud.log.framework.modules.biz.service.LoginLogInfoBizService;
import com.xxh.cloud.third.framework.modules.captcha.service.SmsCaptchaService;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.entity.bos.ThirdUnifyUserLoginBO;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.entity.dto.BindMobileByWxDTO;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.entity.dto.ThirdUnifyUserLoginDTO;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.service.ThirdUnifyUserLoginFactoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@SaIgnore
@RestController
@RequestMapping("/app/customer/third")
@Api(tags="客户端,客户小程序登录")
public class CustomerUserLogin2ThirdAppController {


    @Autowired
    private CustomerUserLoginThirdService customerUserLoginThirdService;
    @Autowired
    private ProjectConfig projectConfig;
    @Autowired
    private SmsCaptchaService smsCaptchaService;
    @Autowired
    private ThirdUnifyUserLoginFactoryService thirdUnifyUserLoginFactoryService;
    @Autowired
    private CustomerUserValidateService customerUserValidateService;
    @Autowired
    private CustomerThirdUserInfoService customerThirdUserInfoService;
    @Autowired
    private LoginLogInfoBizService loginLogInfoBizService;
    @Autowired
    private CustomerFamilyMemberBizService customerFamilyMemberBizService;

    @PostMapping("login")
    @ApiOperation("三方登录")
    public Result<CustomerUserLoginVO> login(@RequestBody CustomerUserLoginThirdDTO params) {
        ValidatorUtils.validateEntity(params);
        customerUserValidateService.checkThirdLogin();
        ThirdUnifyUserLoginBO thirdUnifyUserLogin= thirdUnifyUserLoginFactoryService.login(new ThirdUnifyUserLoginDTO(params.getParamsId(),params.getCode()));

        params.setOpenId(thirdUnifyUserLogin.getOpenid());
        params.setUnionId(thirdUnifyUserLogin.getUnionid());
        params.setReg(customerUserValidateService.isRegThirdLoginAfter());

        CustomerUserLoginVO data= customerUserLoginThirdService.login(params);
        String token= AuthContextUtils.login(data.getCompanyNo(),null,data.getId(),data.getUserName(),data.getThirdId());
        data.setToken(token);

        // zs20240628 拉取家属信息
        LoginUser loginUser=  new LoginUser();
        loginUser.setId(data.getId());
        loginUser.setThirdUserId(data.getThirdId());
        customerFamilyMemberBizService.addFromHis(loginUser);
        
        // 登录日志
        LoginLogAddDTO loginLogAddDTO = new LoginLogAddDTO();
        loginLogAddDTO.setUserId(data.getId());
        loginLogAddDTO.setUserName(data.getUserName());
        loginLogAddDTO.setOpType(LogLoginOutTypeEnum.LL_Login.getCode());
        loginLogAddDTO.setOpWay("微信");
        loginLogAddDTO.setOpIp(IpUtils.getIpAddr());
        loginLogAddDTO.setOpUserAgent(HttpContextUtils.getRequest().getHeader("User-Agent"));
        loginLogAddDTO.setOpPlatform("user");
        loginLogInfoBizService.add(loginLogAddDTO);

        return new Result<CustomerUserLoginVO>().ok(data);
    }

    @SaCheckLogin
    @PostMapping("bindMobileStepByWx")
    @ApiOperation(value = "第三方平台用户绑定手机号(微信授权)")
    public Result<Object> bindMobileStepByWx(@RequestBody BindMobileByWxDTO param) {
        ValidatorUtils.validateEntity(param);
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        CustomerThirdUserInfo thirdUser= customerThirdUserInfoService.getById(loginUser.getThirdUserId());
        if (thirdUser != null) {
            param.setOpenId(thirdUser.getOpenId());
            String mobile = thirdUnifyUserLoginFactoryService.bindMobileByWx(param);
            customerUserLoginThirdService.bind(new BindByMobileDTO(loginUser.getThirdUserId(), mobile, true));
        }
        return new Result<>().success();
    }


}
