package com.xxh.cloud.customer.framework.modules.biz.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerAddrInfo;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerFamilyInfo;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerFamilyMemberInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerFamilyInfoService;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerFamilyMemberInfoService;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.FamilyInfoAddDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.FamilyInfoQueryDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.FamilyInfoUpdateDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.MemberFamilyListDTO;
import com.xxh.cloud.customer.framework.modules.biz.bean.vos.FamilyVO;
import com.xxh.cloud.customer.framework.modules.biz.helper.CustomerFamilyHelper;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerFamilyBizService;
import com.xxh.cloud.customer.framework.modules.cache.helper.CacheCustomerKeyHelper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Log4j2
@Service
public class CustomerFamilyBizServiceImpl implements CustomerFamilyBizService {

    @Autowired
    private CustomerFamilyInfoService customerFamilyInfoService;
    @Autowired
    private CustomerFamilyMemberInfoService customerFamilyMemberInfoService;


    @Override
    public PageData<FamilyVO> query(FamilyInfoQueryDTO params) {
        return null;
    }

    @Override
    public List<FamilyVO> list(MemberFamilyListDTO params) {
        return null;
    }

    @Override
    public List<FamilyVO> listByUser(String userId) {
        return null;
    }

    @Override
    public FamilyVO add(FamilyInfoAddDTO params) {
        CustomerFamilyInfo entity = CustomerFamilyHelper.getCustomerFamilyInfo_Add(params);
        customerFamilyInfoService.save(entity);
        return CustomerFamilyHelper.getFamilyVO(entity);
    }

    @Override
    public FamilyVO update(FamilyInfoUpdateDTO params) {
        CustomerFamilyInfo entity= customerFamilyInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        CustomerFamilyHelper.getCustomerAddrInfo_Update(entity, params);
        customerFamilyInfoService.updateById(entity);
        return CustomerFamilyHelper.getFamilyVO(entity);
    }

    @Override
    public FamilyVO infoById(String id) {
        CustomerFamilyInfo entity= customerFamilyInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return CustomerFamilyHelper.getFamilyVO(entity);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        CustomerFamilyInfo entity= customerFamilyInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 校验是否包含家庭成员
        long count = customerFamilyMemberInfoService.count(new QueryWrapper<CustomerFamilyMemberInfo>()
                .eq("family_id", id)
                .eq("rows_status",YN.YES.getCode()));
        if (count > 0) {
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_EXISTS,"家庭中已经加入了家庭成员");
        }
        entity.setRowsStatus(YN.NO.getCode());
        customerFamilyInfoService.updateById(entity);
    }
}
