package com.xxh.cloud.customer.framework.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.company.cloud.sdk.weixin.mp.model.usermanage.UserBasicInfoReqModel;
import com.company.cloud.sdk.weixin.mp.model.usermanage.UserBasicInfoResp;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerFamilyMemberInfo;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerThirdUserInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerFamilyMemberInfoService;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerThirdUserInfoService;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.*;
import com.xxh.cloud.customer.framework.modules.biz.bean.vos.CheckSmsRulesVO;
import com.xxh.cloud.customer.framework.modules.biz.bean.vos.MemberFamilyVO;
import com.xxh.cloud.customer.framework.modules.biz.helper.CustomerFamilyHelper;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerFamilyMemberBizService;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerDwdPatientBizService;
import com.xxh.cloud.customer.framework.modules.cache.helper.CacheCustomerKeyHelper;
import com.xxh.cloud.customer.framework.modules.constant.RelationTypeEnum;
import com.xxh.cloud.customer.framework.modules.constant.XXCustUserErrorEnum;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.third.framework.modules.sms.model.SmsRequestMessage;
import com.xxh.cloud.third.framework.modules.sms.service.SmsSendService;
import com.xxh.cloud.third.framework.modules.thirdHis.HisApiHelper;
import com.xxh.cloud.third.framework.modules.thirdHis.HisWebApiHelper;
import com.xxh.cloud.third.framework.modules.thirdHis.dto.HisPatientDTO;
import com.xxh.cloud.third.framework.modules.thirdHis.model.*;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.WeixinApiHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Log4j2
@Service
public class CustomerFamilyMemberBizServiceImpl implements CustomerFamilyMemberBizService {


    @Autowired
    private CustomerFamilyMemberInfoService customerFamilyMemberInfoService;
    @Autowired
    private CustomerThirdUserInfoService customerThirdUserInfoService;
    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private SmsSendService smsSendService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private CustomerDwdPatientBizService dwdPatientBizService;

    @Override
    public List<MemberFamilyVO> list(MemberFamilyListDTO params) {
        List<MemberFamilyVO> result = new ArrayList<>();
        List<CustomerFamilyMemberInfo> list = customerFamilyMemberInfoService.list(new QueryWrapper<CustomerFamilyMemberInfo>()
                .eq(StrUtil.isNotEmpty(params.getUserId()), "user_id", params.getUserId())
                .eq(StrUtil.isNotEmpty(params.getPatientId()), "patient_id", params.getPatientId())
                .in(CollUtil.isNotEmpty(params.getPatientIds()), "patient_id", params.getPatientIds())
                .eq(StrUtil.isNotEmpty(params.getRelationType()), "relation_type", params.getRelationType())
                .eq(StrUtil.isNotEmpty(params.getPatientType()), "patient_type", params.getPatientType())
                .eq(StrUtil.isNotEmpty(params.getPatHisNo()), "pat_his_no", params.getPatHisNo())
                .in(CollUtil.isNotEmpty(params.getPatHisNos()), "pat_his_no", params.getPatHisNos())
                .eq(StrUtil.isNotEmpty(params.getIsAuth()), "is_auth", params.getIsAuth())
                .eq(StrUtil.isNotEmpty(params.getMainPersonId()), "main_person_id", params.getMainPersonId())
                .in(CollUtil.isNotEmpty(params.getMainPersonIds()), "main_person_id", params.getMainPersonIds())
                .eq("rows_status", YN.YES.getCode()));
        // 查询选中的家庭成员
        MemberFamilyVO choose = this.getChoose(params.getUserId());
        if (CollUtil.isNotEmpty(list)) {
            result.addAll(list.stream().map(item-> CustomerFamilyHelper.getMemberFamilyVO(item, choose))
                    .collect(Collectors.toList()));
        }
        return result;
    }

    @Override
    public List<MemberFamilyVO> listWithOpenid(List<MemberFamilyVO> vos) {
        if (CollUtil.isNotEmpty(vos)) {
            Set<String> userIds = new HashSet<>();
            vos.forEach(item->{
                userIds.add(item.getUserId());
            });
            List<CustomerThirdUserInfo> thirdUsers = customerThirdUserInfoService.list(new QueryWrapper<CustomerThirdUserInfo>()
                .in("user_id", userIds));
            Map<String, CustomerThirdUserInfo> thirdUserMaps = new HashMap<>();
            if (CollUtil.isNotEmpty(thirdUsers)) {
                thirdUserMaps.putAll(thirdUsers.stream().collect(
                        Collectors.toMap(CustomerThirdUserInfo::getUserId, item->item)));
            }
            for (MemberFamilyVO vo : vos) {
                if (thirdUserMaps.containsKey(vo.getUserId())) {
                    CustomerThirdUserInfo thirdUser = thirdUserMaps.get(vo.getUserId());
                    vo.setThirdUserId(thirdUser.getId());
                    vo.setUserOpenId(thirdUser.getOpenId());
                }
            }
        }
        return vos;
    }

    @Override
    public MemberFamilyVO add(MemberFamilyAddDTO params) {


        return null;
    }

    // @Override
    // public void addBatch(List<MemberFamilyAddDTO> params) {
    //     if (CollUtil.isEmpty(params)) {
    //         return;
    //     }

    //     List<String> patHisNoList = new ArrayList<>();
    //     List<String> queryParams = new ArrayList<>();
    //     params.forEach(i -> {
    //         patHisNoList.add(i.getPatHisNo());
    //         queryParams.add(i.getUserId() + "-" + i.getPatHisNo());
    //     });

    //     // 查询现有的家庭成员
    //     Map<String, CustomerFamilyMemberInfo> familyMemberMaps = new HashMap<>();
    //     List<CustomerFamilyMemberInfo> familyMemberInfos = customerFamilyMemberInfoService.list(new QueryWrapper<CustomerFamilyMemberInfo>()
    //             .in("CONCAT(user_id, '-', pat_his_no)", queryParams).eq("rows_status", YN.YES.getCode()));
    //     if (CollUtil.isNotEmpty(familyMemberInfos)) {
    //         familyMemberInfos.forEach(item->{
    //             familyMemberMaps.put(item.getUserId() + "-" + item.getPatHisNo(), item);
    //         });
    //     }

    //     // 根据同步过来的患者编号[patHisNo] 获取 dwd主患者表id
    //     Map<String, String> patientMap = dwdPatientBizService.getMainPersonIdsByPatientIds(patHisNoList);

    //     // 数据库操作
    //     List<CustomerFamilyMemberInfo> updateList = new ArrayList<>();
    //     List<CustomerFamilyMemberInfo> saveList = new ArrayList<>();
    //     for (MemberFamilyAddDTO patInfo : params) {
    //         if (familyMemberMaps.containsKey(patInfo.getUserId() + "-" + patInfo.getPatHisNo())) {
    //             // updateList.add();
    //         } else {
    //             saveList.add(CustomerFamilyHelper.getCustomerFamilyMemberInfo(patInfo));
    //         }
    //     }
    //     // 新增
    //     if (CollUtil.isNotEmpty(saveList)) {
    //         saveList.forEach(i -> i.setMainPersonId(patientMap.get(i.getPatHisNo())));
    //         customerFamilyMemberInfoService.saveBatch(saveList);
    //     }
    //     // 更新
    //     if (CollUtil.isNotEmpty(updateList)) {

    //     }
    // }

    @Override
    public void addBatch(List<MemberFamilyAddDTO> params) {
        if (CollUtil.isEmpty(params)) {
            return;
        }

        List<String> patHisNoList = new ArrayList<>();
        List<String> queryParams = new ArrayList<>();
        params.forEach(i -> {
            patHisNoList.add(i.getPatHisNo());
            queryParams.add(i.getUserId());
        });

        // 查询现有的家庭成员
        Map<String, Map<String, CustomerFamilyMemberInfo>> familyMemberMaps = new HashMap<>(); // {user_id: {pat_his_no: entity}}；确定是否新增，且修改本人关系
        // 查询用户对应的所有家属，用于更新本人关系
        List<CustomerFamilyMemberInfo> familyMemberInfos = customerFamilyMemberInfoService.list(new QueryWrapper<CustomerFamilyMemberInfo>()
                .in("user_id", queryParams).eq("rows_status", YN.YES.getCode()));
        if (CollUtil.isNotEmpty(familyMemberInfos)) {
            familyMemberInfos.forEach(item->{
                Map<String, CustomerFamilyMemberInfo> m = familyMemberMaps.get(item.getUserId());
                if (null == m) {
                    m = new HashMap<>();
                    familyMemberMaps.put(item.getUserId(), m);
                }
                m.put(item.getPatHisNo(), item);
            });
        }

        // 数据库操作
        List<CustomerFamilyMemberInfo> updateList = new ArrayList<>(); // 需要更新的家属信息（只更新关系）
        List<CustomerFamilyMemberInfo> saveList = new ArrayList<>(); // 需要新增的家属信息
        Map<String, CustomerFamilyMemberInfo> saveMap = new HashMap<>();

        Map<String, List<CustomerFamilyMemberInfo>> relationMap = new HashMap<>(); // {user_id, [entity]}  用于处理哪个家属为本人, entity列表为key对应的所有家属信息
        for (MemberFamilyAddDTO patInfo : params) {
             
            List<CustomerFamilyMemberInfo> l = relationMap.get(patInfo.getUserId());
            if (null == l) {
                l = new ArrayList<>();
                relationMap.put(patInfo.getUserId(), l);
            }

            if (familyMemberMaps.containsKey(patInfo.getUserId()) && familyMemberMaps.get(patInfo.getUserId()).containsKey(patInfo.getPatHisNo())) {
                List<CustomerFamilyMemberInfo> vs = new ArrayList<>();
                familyMemberMaps.get(patInfo.getUserId()).forEach((k, v) -> {
                    vs.add(v);
                });
                if (CollUtil.isNotEmpty(vs)) {
                    l.addAll(vs);
                }
            } else {
                CustomerFamilyMemberInfo entity = CustomerFamilyHelper.getCustomerFamilyMemberInfo(patInfo);
                saveList.add(entity);
                saveMap.put(entity.getId(), entity);
                l.add(entity);
            }
        }

        if (CollUtil.isNotEmpty(relationMap)) {
            relationMap.forEach((k, v) -> {
                CustomerFamilyMemberInfo selfEntity = null; // 系统设置的本人关系的家属信息，可能没有该信息
                CustomerFamilyMemberInfo updateEntity = null; // 除本人关系外时间最早的家属信息
                for (CustomerFamilyMemberInfo i : v) {
                    // 若标记不为1且为本人，则为用户自己设置的本人，不进行处理
                    if (!"1".equals(i.getRelationFlag()) && RelationTypeEnum.self.getCode().equals(i.getRelationType())) {
                        selfEntity = null;
                        updateEntity = null;
                        break;
                    }
                    // 此处本人为系统设置
                    if (RelationTypeEnum.self.getCode().equals(i.getRelationType())) {
                        selfEntity = i;
                        continue;
                    }
                    // 此处是除本人关系家属外最早的家属信息
                    if (null == updateEntity) {
                        updateEntity = i;
                        continue;
                    }

                    String date1 = StrUtil.isBlank(updateEntity.getCreateTime()) ? String.valueOf(updateEntity.getCreateDate().getTime()) : updateEntity.getCreateTime();
                    String date2 = StrUtil.isBlank(i.getCreateTime()) ? String.valueOf(i.getCreateDate().getTime()) : i.getCreateTime();

                    if (date1.compareTo(date2) > 0) {
                        updateEntity = i;
                    }
                }
                // 系统设置的本人关系的家属与最早的家属比较早晚，最早的设置为本人，晚的取消本人关系
                if (null != selfEntity && null != updateEntity) {
                    String date1 = StrUtil.isBlank(selfEntity.getCreateTime()) ? String.valueOf(selfEntity.getCreateDate().getTime()) : selfEntity.getCreateTime();
                    String date2 = StrUtil.isBlank(updateEntity.getCreateTime()) ? String.valueOf(updateEntity.getCreateDate().getTime()) : updateEntity.getCreateTime();

                    if (date1.compareTo(date2) > 0) {
                        selfEntity.setMemberRelation("");
                        selfEntity.setRelationName("");
                        selfEntity.setRelationType("");
                        updateList.add(updateEntity);

                        updateEntity.setMemberRelation(RelationTypeEnum.self.getName());
                        updateEntity.setRelationName(RelationTypeEnum.self.getName());
                        updateEntity.setRelationType(RelationTypeEnum.self.getCode());
                        if (null == saveMap.get(updateEntity.getId())) { // 在saveMap中则直接随其新增到数据库
                            updateList.add(updateEntity);
                        }
                    }
                } else if (null != updateEntity) {
                    updateEntity.setMemberRelation(RelationTypeEnum.self.getName());
                    updateEntity.setRelationName(RelationTypeEnum.self.getName());
                    updateEntity.setRelationType(RelationTypeEnum.self.getCode());
                    if (null == saveMap.get(updateEntity.getId())) { // 在saveMap中则直接随其新增到数据库
                        updateList.add(updateEntity);
                    }
                }
            });
        }
        
        // 根据同步过来的患者编号[patHisNo] 获取 dwd主患者表id
        Map<String, String> patientMap = dwdPatientBizService.getMainPersonIdsByPatientIds(patHisNoList);

        // 新增
        if (CollUtil.isNotEmpty(saveList)) {
            saveList.forEach(i -> i.setMainPersonId(patientMap.get(i.getPatHisNo())));
            customerFamilyMemberInfoService.saveBatch(saveList);
        }
        // 更新
        if (CollUtil.isNotEmpty(updateList)) {
            customerFamilyMemberInfoService.updateBatchById(updateList);
        }
    }

    @Override
    public MemberFamilyVO update(MemberFamilyUpdateDTO params) {


        return null;
    }

    @Override
    public MemberFamilyVO infoById(String id) {


        return null;
    }

    @Override
    public void delById(String id) {
        CustomerFamilyMemberInfo entity = customerFamilyMemberInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        customerFamilyMemberInfoService.updateById(entity);
    }

    // @Override
    // public void addFromHis(LoginUser loginUser) {
    //     CustomerThirdUserInfo thirdUser=customerThirdUserInfoService.getById(loginUser.getThirdUserId());
    //     if(thirdUser==null){
    //         throw new BusinessException(XXCustUserErrorEnum.ThirdUser_NotExists);
    //     }
    //     HealthyFrontendConfig healthyFrontendConfig = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
    //     // unionId为空，重新获取一次
    //     if (StrUtil.isEmpty(thirdUser.getUnionId())) {
    //         getUnionid(thirdUser, healthyFrontendConfig);
    //     }
    //     // RespBaseThird<PatBindModel> resp = HisApiHelper.getPatBind(healthyFrontendConfig, thirdUser.getUnionId());  // 海慈
    //     RespBaseThird<PatBindModel> resp = HisWebApiHelper.getPatBind(healthyFrontendConfig, thirdUser.getUnionId());  // 互联网医院
    //     if (resp.getCode() != 0) {
    //         log.error(JSONObject.toJSONString(resp));
    //         throw new BusinessException(XXCustUserErrorEnum.FAMILY_CREATE_ERR);
    //     }
    //     if (CollUtil.isNotEmpty(resp.getData().getData())) {
    //         // 查询现有的家庭成员
    //         Map<String, CustomerFamilyMemberInfo> familyMemberMaps = new HashMap<>();
    //         List<CustomerFamilyMemberInfo> familyMemberInfos = customerFamilyMemberInfoService.list(new QueryWrapper<CustomerFamilyMemberInfo>()
    //                 .eq("user_id", loginUser.getId()).eq("rows_status", YN.YES.getCode()));
    //         if (CollUtil.isNotEmpty(familyMemberInfos)) {
    //             familyMemberInfos.forEach(item->{
    //                 familyMemberMaps.put(item.getCardNo(), item);
    //             });
    //         }

    //         // 20240314 根据同步过来的患者编号[patHisNo] 获取 dwd主患者表id
    //         List<String> patHisNoList = new ArrayList<>();
    //         resp.getData().getData().forEach(i -> patHisNoList.add(i.getPatHisNo()));
    //         Map<String, String> patientMap = dwdPatientBizService.getMainPersonIdsByPatientIds(patHisNoList);

    //         // 数据库操作
    //         List<CustomerFamilyMemberInfo> updateList = new ArrayList<>();
    //         List<CustomerFamilyMemberInfo> saveList = new ArrayList<>();
    //         for (PatInfoModel patInfo : resp.getData().getData()) {
    //             if (familyMemberMaps.containsKey(patInfo.getIdNo())) {
    //                 updateList.add(CustomerFamilyHelper.getCustomerFamilyMemberInfo(
    //                         familyMemberMaps.get(patInfo.getIdNo()), patInfo));
    //             } else {
    //                 saveList.add(CustomerFamilyHelper.getCustomerFamilyMemberInfo(loginUser.getId(), patInfo));
    //             }
    //         }
    //         // 新增
    //         if (CollUtil.isNotEmpty(saveList)) {
    //             saveList.forEach(i -> i.setMainPersonId(patientMap.get(i.getPatHisNo())));

    //             customerFamilyMemberInfoService.saveBatch(saveList);
    //         }
    //         // 更新,先不做更新处理 20231110 mj-li
    //         // zs20240318 更新，主要更新患者主id
    //         if (CollUtil.isNotEmpty(updateList)) {
    //             updateList.forEach(i -> {
    //                 i.setMainPersonId(patientMap.get(i.getPatHisNo()));
                    
    //                 // 同步过来的关系有问题，都是他人，所以不更新关系字段
    //                 i.setMemberRelation(null);
    //                 i.setRelationType(null);
    //                 i.setRelationName(null);
    //             });

    //             customerFamilyMemberInfoService.updateBatchById(updateList);
    //         }
    //     }
    // }

    @Override
    public void addFromHis(LoginUser loginUser) {
        CustomerThirdUserInfo thirdUser=customerThirdUserInfoService.getById(loginUser.getThirdUserId());
        if(thirdUser==null){
            throw new BusinessException(XXCustUserErrorEnum.ThirdUser_NotExists);
        }
        HealthyFrontendConfig healthyFrontendConfig = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        // unionId为空，重新获取一次
        if (StrUtil.isEmpty(thirdUser.getUnionId())) {
            getUnionid(thirdUser, healthyFrontendConfig);
        }

        // RespBaseThird<PatBindModel> resp = HisApiHelper.getPatBind(healthyFrontendConfig, thirdUser.getUnionId());  // 海慈
        // RespBaseThird<PatBindModel> resp = HisWebApiHelper.getPatBind(healthyFrontendConfig, thirdUser.getUnionId());  // 互联网医院
        HisPatientDTO hisPatientDTO = new HisPatientDTO();
        hisPatientDTO.setOpenid(thirdUser.getOpenId());
        RespBaseThird<PatBindModel> resp = HisWebApiHelper.getPatBindByThirdId(healthyFrontendConfig, hisPatientDTO);  // 互联网医院
        if (resp.getCode() != 0 || CollUtil.isEmpty(resp.getData().getData())) {
            hisPatientDTO.setOpenid(null); // 按unionid查
            hisPatientDTO.setUnionid(thirdUser.getUnionId());
            resp = HisWebApiHelper.getPatBindByThirdId(healthyFrontendConfig, hisPatientDTO);  // 互联网医院
        }

        if (resp.getCode() != 0) {
            log.error(JSONObject.toJSONString(resp));
            throw new BusinessException(XXCustUserErrorEnum.FAMILY_CREATE_ERR);
        }
        if (CollUtil.isNotEmpty(resp.getData().getData())) {
            // 查询现有的家庭成员
            Map<String, CustomerFamilyMemberInfo> familyMemberMaps = new HashMap<>();
            List<CustomerFamilyMemberInfo> familyMemberInfos = customerFamilyMemberInfoService.list(new QueryWrapper<CustomerFamilyMemberInfo>()
                    // .eq("rows_status", YN.YES.getCode())
                    .eq("user_id", loginUser.getId())
            );
            if (CollUtil.isNotEmpty(familyMemberInfos)) {
                familyMemberInfos.forEach(item->{
                    familyMemberMaps.put(item.getCardNo(), item);
                });
            }

            // 20240314 根据同步过来的患者编号[patHisNo] 获取 dwd主患者表id
            List<String> patHisNoList = new ArrayList<>();
            resp.getData().getData().forEach(i -> patHisNoList.add(i.getPatHisNo()));
            Map<String, String> patientMap = dwdPatientBizService.getMainPersonIdsByPatientIds(patHisNoList);

            // 数据库操作
            Boolean updateThirdUserFlag = false;
            List<CustomerFamilyMemberInfo> updateList = new ArrayList<>();
            List<CustomerFamilyMemberInfo> saveList = new ArrayList<>();
            Map<String, PatInfoModel> patInfoMap = new HashMap<>();
            for (PatInfoModel patInfo : resp.getData().getData()) {
                patInfoMap.put(patInfo.getIdNo(), patInfo);

                CustomerFamilyMemberInfo cfm = null;
                if (familyMemberMaps.containsKey(patInfo.getIdNo())) {
                    cfm = CustomerFamilyHelper.getCustomerFamilyMemberInfo(familyMemberMaps.get(patInfo.getIdNo()), patInfo);
                    if (StrUtil.isBlank(cfm.getIsAuth())) {
                        cfm.setIsAuth(YN.NO.getCode());
                    }
                    updateList.add(cfm);
                } else {
                    cfm = CustomerFamilyHelper.getCustomerFamilyMemberInfo(loginUser.getId(), patInfo);
                    saveList.add(cfm);
                }

                // 更新登录用户unionid信息
                // 若返回openid与登录用户一致则更新家属关系为本人
                if (thirdUser.getOpenId().equals(patInfo.getOpenId())) {
                    if (StrUtil.isBlank(thirdUser.getUnionId()) && StrUtil.isNotBlank(patInfo.getUnionid())) {
                        updateThirdUserFlag = true;
                        thirdUser.setUnionId(patInfo.getUnionid());
                    }
                }
            }

            // 需要解绑的家属
            List<CustomerFamilyMemberInfo> unbindList = new ArrayList<>();
            familyMemberMaps.forEach((k, v) -> {
                if (!patInfoMap.containsKey(k)) {
                    v.setRowsStatus(YN.NO.getCode());
                    v.setIsAuth(YN.NO.getCode());
                    v.setMemberRelation("");
                    v.setRelationType("");
                    v.setRelationName("");
                    unbindList.add(v);
                }
            });

            // 新增
            if (CollUtil.isNotEmpty(saveList)) {
                saveList.forEach(i -> i.setMainPersonId(patientMap.get(i.getPatHisNo())));

                // if (saveList.size() == 1) {
                //     CustomerFamilyMemberInfo cfm = saveList.get(0);
                //     cfm.setIsAuth(YN.YES.getCode());
                //     cfm.setMemberRelation(RelationTypeEnum.self.getName());
                //     cfm.setRelationName(RelationTypeEnum.self.getName());
                //     cfm.setRelationType(RelationTypeEnum.self.getCode());
                // }

                customerFamilyMemberInfoService.saveBatch(saveList);
            }
            // 更新,先不做更新处理 20231110 mj-li
            // zs20240318 更新，主要更新患者主id
            if (CollUtil.isNotEmpty(updateList)) {
                updateList.forEach(i -> {
                    i.setMainPersonId(patientMap.get(i.getPatHisNo()));
                    
                    // 同步过来的关系有问题，都是他人，所以不更新关系字段
                    // i.setMemberRelation(null);
                    // i.setRelationType(null);
                    // i.setRelationName(null);
                });

                customerFamilyMemberInfoService.updateBatchById(updateList);
            }

            // 更新登录用户unionid信息
            if (updateThirdUserFlag) {
                customerThirdUserInfoService.updateById(thirdUser);
            }

            // 解绑同步时不存在的家属
            if (CollUtil.isNotEmpty(unbindList)) {
                customerFamilyMemberInfoService.updateBatchById(unbindList);
            }
        }
    }

    @Override
    public void sendAuthCode(LoginUser loginUser, MemberFamilyAuthSmsDTO params) {
        CustomerFamilyMemberInfo entity = customerFamilyMemberInfoService.getById(params.getFamilyMemberId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(!entity.getUserId().equals(loginUser.getId())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 校验短信发送的规则
        checkSmsRules(entity.getMobile());
        String authCode = RandomUtil.randomNumbers(6);
        String notice = StrUtil.format("验证码为:{}，您正在授权绑定家庭关系，若非本人操作，请勿泄露。", authCode);
        log.info(StrUtil.format("家庭成员{}-authCode:{}",params.getFamilyMemberId(),authCode));
//        authCode = "666666"; // 未接短信模块之前 验证码666666
        // 发送短信
        List<SmsRequestMessage> messages = new ArrayList<>();
        messages.add(new SmsRequestMessage(entity.getMobile(),notice));
        smsSendService.sendHisSms(messages);
        //------------------
        String redisKey = CacheCustomerKeyHelper.getCustomerFamilyMemberAuthKey(params.getFamilyMemberId());
        redisUtils.set(redisKey, authCode, 600);
    }

    @Override
    public void choose(LoginUser loginUser, MemberFamilyInfoDTO params) {
        CustomerFamilyMemberInfo entity = customerFamilyMemberInfoService.getById(params.getFamilyMemberId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(!entity.getUserId().equals(loginUser.getId())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 如果患者没有主id则补刀同步一次，再没有则返回异常
        if (StrUtil.isBlank(entity.getMainPersonId())) {
            List<String> patientIds = new ArrayList<>();
            patientIds.add(entity.getPatHisNo());
            Map<String, String> patientMap = dwdPatientBizService.getMainPersonIdsByPatientIds(patientIds);
            if (CollUtil.isEmpty(patientMap) || StrUtil.isBlank(patientMap.get(entity.getPatHisNo()))) {
                throw new BusinessException("数据同步中，请稍后选择");
            }

            entity.setMainPersonId(patientMap.get(entity.getPatCardNo()));
            customerFamilyMemberInfoService.updateById(entity);
        }
        
        String redisKey = CacheCustomerKeyHelper.getCustomerFamilyMemberChoose(loginUser.getId());
        redisUtils.set(redisKey, CustomerFamilyHelper.getMemberFamilyVO(entity, null), 3600 * 24 * 30);
    }

    @Override
    public MemberFamilyVO getChoose(String userId) {
        String redisKey = CacheCustomerKeyHelper.getCustomerFamilyMemberChoose(userId);
        if (redisUtils.hasKey(redisKey)) {
            Object obj = redisUtils.get(redisKey);
            MemberFamilyVO result = (MemberFamilyVO) obj;
            if (null != result && StrUtil.isBlank(result.getMainPersonId())) {
                List<String> patientIds = new ArrayList<>();
                patientIds.add(result.getPatHisNo());
                Map<String, String> patientMap = dwdPatientBizService.getMainPersonIdsByPatientIds(patientIds);
                if (CollUtil.isEmpty(patientMap) || StrUtil.isBlank(patientMap.get(result.getPatHisNo()))) {
                    throw new BusinessException("数据同步中，请稍后选择");
                }
                customerFamilyMemberInfoService.update(new UpdateWrapper<CustomerFamilyMemberInfo>()
                        .set("main_person_id", patientMap.get(result.getPatCardNo()))
                        .eq("id", result.getId())
                );

                result.setMainPersonId(patientMap.get(result.getPatCardNo()));
                redisUtils.set(redisKey, result, 3600 * 24 * 30);
            }
            return result;
        } else {
            return null;
        }
    }

    private void checkSmsRules(String mobile) {
        String redisKey = CacheCustomerKeyHelper.getCustomerFamilyMemberSmsRulesKey(mobile);
        CheckSmsRulesVO checkVo = new CheckSmsRulesVO();
        Date nowDate = new Date();
        if (redisUtils.hasKey(redisKey)) {
            checkVo = (CheckSmsRulesVO) redisUtils.get(redisKey);
            long timeInterval = nowDate.getTime() - checkVo.getLastTimestamp();
            if (checkVo.getCount() >= 5) {
                throw new BusinessException(XXCustUserErrorEnum.SMS_TIMES_OVER_LIMIT);
            } else if (checkVo.getCount() >= 2) {
                // 第二次重发校验10min间隔
                if (timeInterval < 600 * 1000) {
                    throw new BusinessException(XXCustUserErrorEnum.SMS_SEND_WAIT);
                }
            } else  {
                // 第一次重发校验30s间隔
                if (timeInterval < 30 * 1000) {
                    throw new BusinessException(XXCustUserErrorEnum.SMS_SEND_WAIT);
                }
            }
            checkVo.setCount(checkVo.getCount() + 1);
        } else {
            checkVo.setCount(1);
        }
        checkVo.setLastTimestamp(nowDate.getTime());
        redisUtils.set(redisKey, checkVo, getTodayRemainSecondNum());

    }

    // 获取今天剩余的秒数
    private int getTodayRemainSecondNum() {
        long current = System.currentTimeMillis();    //当前时间毫秒数
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DAY_OF_MONTH, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        long tomorrowZero = calendar.getTimeInMillis();
        long remainSecond = (tomorrowZero - current) / 1000;
        return (int)remainSecond;
    }


    // 重新获取unionid
    private void getUnionid(CustomerThirdUserInfo thirdUser, HealthyFrontendConfig healthyFrontend){
        // 调用his获取accessToken
        RespBaseThird<HisAccessTokenModel> acResp = HisApiHelper.getAccessToken(healthyFrontend);
        // 目前调用微信端获取accessToken
//        RespBaseThird<HisAccessTokenModel> acResp = WeixinApiHelper.getAccessToken(healthyFrontend, config);
        UserBasicInfoResp userBasicInfoResp = new UserBasicInfoResp();
        if (acResp.getData() != null) {
            // 获取unionid
            UserBasicInfoReqModel userBasicInfoReqModel = new UserBasicInfoReqModel();
            userBasicInfoReqModel.setAccess_token(acResp.getData().getAccToken());
            userBasicInfoReqModel.setOpenid(thirdUser.getOpenId());
            RespBaseThird<UserBasicInfoResp> unionidResp = WeixinApiHelper.getUnionid(healthyFrontend, userBasicInfoReqModel);
            if (unionidResp.getData() != null && StrUtil.isNotEmpty(unionidResp.getData().getUnionid())) {
                thirdUser.setUnionId(unionidResp.getData().getUnionid());
                customerThirdUserInfoService.updateById(thirdUser);
            } else {
                throw new BusinessException(XXCustUserErrorEnum.UnionId_NotExists);
            }
        }
    }

    @Override
    public void updateSelfRelationByIds(List<String> familyMemberIds) {
        if (CollUtil.isEmpty(familyMemberIds)) {
            return;
        }
        customerFamilyMemberInfoService.update(new UpdateWrapper<CustomerFamilyMemberInfo>()
                .set("member_relation", RelationTypeEnum.self.getName())
                .set("relation_name", RelationTypeEnum.self.getName())
                .set("relation_type", RelationTypeEnum.self.getCode())
                .set("update_date", DateUtil.date())
                .set("is_auth", YN.YES.getCode())
                .in("id", familyMemberIds)
        );
    }

    @Override
    public void unbindById(String id) {
        CustomerFamilyMemberInfo entity = customerFamilyMemberInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setMemberRelation("");
        entity.setRelationName("");
        entity.setRelationType("");
        entity.setIsAuth(YN.NO.getCode());
        customerFamilyMemberInfoService.updateById(entity);
    }
}
