package com.xxh.cloud.customer.framework.modules.check.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerUserInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerUserInfoService;
import com.xxh.cloud.customer.framework.modules.check.entity.dto.CustomerUserEmailCheckDTO;
import com.xxh.cloud.customer.framework.modules.check.entity.dto.CustomerUserMobileCheckDTO;
import com.xxh.cloud.customer.framework.modules.check.entity.dto.CustomerUserUserNameCheckDTO;
import com.xxh.cloud.customer.framework.modules.check.service.CustomerUserRegister2AddCheckService;
import com.xxh.cloud.customer.framework.modules.check.service.CustomerUserValidateService;
import com.xxh.cloud.customer.framework.modules.constant.XXCustUserErrorEnum;
import com.xxh.cloud.framework.common.constant.DataScopeMode;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CustomerUserRegister2AddCheckServiceImpl implements CustomerUserRegister2AddCheckService {


    @Autowired
    private CustomerUserInfoService customerUserInfoService;
    @Autowired
    private CustomerUserValidateService customerUserValidateService;



    /**
     * ------------------------------------------------------------
     * ---------------------------用户名----------------------------
     * ------------------------------------------------------------
     * */
    private CustomerUserInfo get_userName(String userName){
        CustomerUserInfo data= customerUserInfoService.getOne(new QueryWrapper<CustomerUserInfo>()
                .eq("user_name",userName)
        );
        return data;
    }

    private CustomerUserInfo get_userName(String companyNo,String userName){
        if(StrUtil.isEmpty(companyNo)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"机构不能为空");
        }
        CustomerUserInfo data= customerUserInfoService.getOne(new QueryWrapper<CustomerUserInfo>()
                .eq("company_no",companyNo)
                .eq("user_name",userName)
        );
        return data;
    }


    @Override
    public CustomerUserInfo getByUserName(String companyNo, String userName) {
        DataScopeMode dataScope = customerUserValidateService.getCustDataScope();
        if(DataScopeMode.CompanyDataScope==dataScope){
            return this.get_userName(companyNo,userName);
        }else {
            return this.get_userName(userName);
        }
    }



    private boolean check_userName(String companyNo, String userName){
        if(StrUtil.isEmpty(companyNo)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"机构不能为空");
        }
        long num= customerUserInfoService.count(new QueryWrapper<CustomerUserInfo>()
                .eq("company_no",companyNo)
                .eq("user_name",userName)
        );
        return num>0?false:true;
    }

    private boolean check_userName(String userName){
        long num= customerUserInfoService.count(new QueryWrapper<CustomerUserInfo>()
                .eq("user_name",userName)
        );
        return num>0?false:true;
    }


    @Override
    public boolean checkUserName(CustomerUserUserNameCheckDTO params) {
        ValidatorUtils.validateEntity(params);
        DataScopeMode dataScope = customerUserValidateService.getCustDataScope();
        if(DataScopeMode.CompanyDataScope==dataScope){
            return this.check_userName(params.getCompanyNo(),params.getUserName());
        }else {
            return this.check_userName(params.getUserName());
        }
    }

    @Override
    public boolean checkUserName(String companyNo, String userName) {
        return this.checkUserName(new CustomerUserUserNameCheckDTO(companyNo,null));
    }


    @Override
    public void validateUserName(CustomerUserUserNameCheckDTO params) {
        ValidatorUtils.validateEntity(params);
        boolean flag=this.checkUserName(params);
        if(!flag){
            throw new BusinessException(XXCustUserErrorEnum.UserName_Exists);
        }
    }

    @Override
    public void validateUserName(String companyNo, String userName) {
        this.validateUserName(new CustomerUserUserNameCheckDTO(companyNo,userName));
    }


    /**
     * ------------------------------------------------------------
     * ---------------------------手机号----------------------------
     * ------------------------------------------------------------
     * */
    private boolean check_mobile(String companyNo, String mobile){
        if(StrUtil.isEmpty(companyNo)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"机构不能为空");
        }
        long num= customerUserInfoService.count(new QueryWrapper<CustomerUserInfo>()
                .eq("company_no",companyNo)
                .eq("mobile",mobile)
        );
        return num>0?false:true;
    }

    private boolean check_mobile(String mobile){
        long num= customerUserInfoService.count(new QueryWrapper<CustomerUserInfo>()
                .eq("mobile",mobile)
        );
        return num>0?false:true;
    }


    private CustomerUserInfo get_mobile(String companyNo, String mobile){
        if(StrUtil.isEmpty(companyNo)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"机构不能为空");
        }
        CustomerUserInfo data= customerUserInfoService.getOne(new QueryWrapper<CustomerUserInfo>()
                .eq("company_no",companyNo)
                .eq("mobile",mobile)
        );
        return data;
    }

    private CustomerUserInfo get_mobile(String mobile){
        CustomerUserInfo data=  customerUserInfoService.getOne(new QueryWrapper<CustomerUserInfo>()
                .eq("mobile",mobile)
        );
        return data;
    }


    @Override
    public CustomerUserInfo getByMobile(String companyNo, String mobile) {
        DataScopeMode dataScope = customerUserValidateService.getCustDataScope();
        if (DataScopeMode.CompanyDataScope == dataScope) {
            return this.get_mobile(companyNo, mobile);
        } else {
            return this.get_mobile(mobile);
        }
    }


    @Override
    public boolean checkMobile(CustomerUserMobileCheckDTO params) {
        ValidatorUtils.validateEntity(params);
        DataScopeMode dataScope = customerUserValidateService.getCustDataScope();
        if (DataScopeMode.CompanyDataScope == dataScope) {
            return this.check_mobile(params.getCompanyNo(), params.getMobile());
        } else {
            return this.check_mobile(params.getMobile());
        }
    }

    @Override
    public boolean checkMobile(String companyNo, String mobile) {
        return this.checkMobile(new CustomerUserMobileCheckDTO(companyNo,mobile));
    }



    @Override
    public void validateMobile(CustomerUserMobileCheckDTO params) {
        ValidatorUtils.validateEntity(params);
        boolean flag=this.checkMobile(params);
        if(!flag){
            throw new BusinessException(XXCustUserErrorEnum.Mobile_Exists);
        }
    }

    @Override
    public void validateMobile(String companyNo, String mobile) {
        this.validateMobile(new CustomerUserMobileCheckDTO(companyNo,mobile));
    }


    /**
     * ------------------------------------------------------------
     * ---------------------------邮箱----------------------------
     * ------------------------------------------------------------
     * */
    private boolean check_email(String companyNo, String email){
        if(StrUtil.isEmpty(companyNo)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"机构不能为空");
        }
        long num= customerUserInfoService.count(new QueryWrapper<CustomerUserInfo>()
                .eq("company_no",companyNo)
                .eq("email",email)
        );
        return num>0?false:true;
    }

    private boolean check_email(String email){
        long num= customerUserInfoService.count(new QueryWrapper<CustomerUserInfo>()
                .eq("email",email)
        );
        return num>0?false:true;
    }



    private CustomerUserInfo get_email(String companyNo, String email){
        if(StrUtil.isEmpty(companyNo)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"机构不能为空");
        }
        CustomerUserInfo data= customerUserInfoService.getOne(new QueryWrapper<CustomerUserInfo>()
                .eq("company_no",companyNo)
                .eq("email",email)
        );
        return data;
    }

    private CustomerUserInfo get_email(String email){
        CustomerUserInfo data= customerUserInfoService.getOne(new QueryWrapper<CustomerUserInfo>()
                .eq("email",email)
        );
        return data;
    }


    @Override
    public CustomerUserInfo getByEmail(String companyNo, String email) {
        DataScopeMode dataScope = customerUserValidateService.getCustDataScope();
        if (DataScopeMode.CompanyDataScope == dataScope) {
            return this.get_email(companyNo, email);
        } else {
            return this.get_email(email);
        }
    }


    @Override
    public boolean checkEmail(CustomerUserEmailCheckDTO params) {
        ValidatorUtils.validateEntity(params);
        DataScopeMode dataScope = customerUserValidateService.getCustDataScope();
        if (DataScopeMode.CompanyDataScope == dataScope) {
            return this.check_email(params.getCompanyNo(), params.getEmail());
        } else {
            return this.check_email(params.getEmail());
        }
    }

    @Override
    public boolean checkEmail(String companyNo, String email) {
        return this.checkEmail(new CustomerUserEmailCheckDTO(companyNo,email));
    }

    @Override
    public void validateEmail(CustomerUserEmailCheckDTO params) {
        ValidatorUtils.validateEntity(params);
        boolean flag=this.checkEmail(params);
        if(!flag){
            throw new BusinessException(XXCustUserErrorEnum.Email_Exists);
        }
    }

    @Override
    public void validateEmail(String companyNo, String email) {
        this.validateEmail(new CustomerUserEmailCheckDTO(companyNo,email));
    }




}
