package com.xxh.cloud.customer.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.company.cloud.sdk.weixin.mp.model.msgmanage.SendTempMsgResp;
import com.company.cloud.sdk.weixin.mp.model.usermanage.UserBasicInfoResp;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.customer.client.api.CustomerThirdClientService;
import com.xxh.cloud.customer.client.api.bean.bos.SendTempResultBO;
import com.xxh.cloud.customer.client.api.bean.dto.CustomerUserLoginThirdClientDTO;
import com.xxh.cloud.customer.client.api.bean.dto.SendTempClientDTO;
import com.xxh.cloud.customer.client.api.bean.dto.ThirdUserUpdateClientDTO;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerThirdUserInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerThirdUserInfoService;
import com.xxh.cloud.customer.framework.modules.biz.bean.dto.ThirdUserUpdateDTO;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerFamilyMemberBizService;
import com.xxh.cloud.customer.framework.modules.biz.service.CustomerThirdUserBizService;
import com.xxh.cloud.customer.framework.modules.check.service.CustomerUserValidateService;
import com.xxh.cloud.customer.framework.modules.login.entity.dto.CustomerUserLoginThirdDTO;
import com.xxh.cloud.customer.framework.modules.login.entity.vos.CustomerUserLoginVO;
import com.xxh.cloud.customer.framework.modules.login.service.CustomerUserLoginThirdService;
import com.xxh.cloud.framework.common.param.bos.WxApiConfig;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdWX.dto.SendTempMsgDTO;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.WeixinApiHelper;
import com.xxh.cloud.third.framework.modules.thirdWX.service.ThirdWeixinAccessTokenService;
import com.xxh.cloud.third.framework.modules.thirdWX.service.ThirdWeixinLoginMpService;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

@Slf4j
@Service
public class CustomerThirdClientServiceImpl implements CustomerThirdClientService {

    private Lock lock = new ReentrantLock();

    @Autowired
    private CustomerThirdUserInfoService customerThirdUserInfoService;
    @Autowired
    private CustomerThirdUserBizService customerThirdUserBizService;
    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private ThirdWeixinAccessTokenService thirdWeixinAccessTokenService;
    @Autowired
    private CustomerUserLoginThirdService customerUserLoginThirdService;
    @Autowired
    private ThirdWeixinLoginMpService thirdWeixinLoginMpService;
    @Autowired
    private CustomerUserValidateService customerUserValidateService;
    @Autowired
    private CustomerFamilyMemberBizService customerFamilyMemberBizService;

    @Override
    public void updateThirdUser(ThirdUserUpdateClientDTO params) {
        ThirdUserUpdateDTO dto = new ThirdUserUpdateDTO();
        dto.setThirdUserId(params.getThirdUserId());
        dto.setOpenId(params.getOpenId());
        dto.setThirdUserStatus(params.getThirdUserStatus());
        customerThirdUserBizService.updateThirdUser(dto);
    }

    @Override
    public SendTempResultBO getSendTempResult(SendTempClientDTO dto) {
        // 查询三方用户
        CustomerThirdUserInfo thirdUser = customerThirdUserInfoService.getOne(new QueryWrapper<CustomerThirdUserInfo>()
                .eq("open_id", dto.getOpenid()));

        // 查村配置
        WxApiConfig wxConfig= paramsClientService.getParamConfig(thirdUser.getParamsId(), new WxApiConfig());
        HealthyFrontendConfig healthyFrontend = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        // 查询accesstoken 目前调用跳板机调用微信端
        String accessToken = thirdWeixinAccessTokenService.getAccessToken(wxConfig);

        SendTempMsgDTO params = new SendTempMsgDTO();
        params.setAccessToken(accessToken);
        params.setOpenid(dto.getOpenid());
        params.setTemplateId(dto.getTemplateId());
        params.setUrl(dto.getUrl());
        params.setTempData(dto.getTempData());
        params.setAppid(wxConfig.getAppid());
        params.setSecret(wxConfig.getSecret());
        RespBaseThird<SendTempMsgResp> resp = WeixinApiHelper.sendTempMsg(healthyFrontend, params);
        if (resp.getCode() != 0) {
            log.error(JSONObject.toJSONString(resp));
            return new SendTempResultBO(resp.getCode(), resp.getMsg(), null);
        }
        return new SendTempResultBO(resp.getData().getErrcode(), resp.getData().getErrmsg(), resp.getData().getMsgid());
    }

    @Override
    public List<SendTempResultBO> getSendTempResults(List<SendTempClientDTO> dtos) {
        List<SendTempResultBO> result = new ArrayList<>();
        if (CollUtil.isNotEmpty(dtos)) {
            for (SendTempClientDTO dto : dtos) {
                result.add(getSendTempResult(dto));
                try {
                    Thread.sleep(300);
                } catch (Exception e) {
                    // do nothing
                }
            }
        }
        return result;
    }

    // @Transactional(rollbackFor = Exception.class)
    @Override
    public void register(CustomerUserLoginThirdClientDTO params) {
        try {
            lock.lock();
            CustomerUserLoginThirdDTO dto = new CustomerUserLoginThirdDTO();
            dto.setParamsId(params.getParamsId());

            // 通过openid拉取unionid
            WxApiConfig wxConfig= paramsClientService.getParamConfig(params.getParamsId(), new WxApiConfig());
            UserBasicInfoResp userBasicInfoResp = thirdWeixinLoginMpService.getUserUnionId(wxConfig, null, null, params.getOpenId());
            
            // 注册用户信息
            dto.setOpenId(params.getOpenId());
            dto.setUnionId(userBasicInfoResp.getUnionid());
            dto.setReg(customerUserValidateService.isRegThirdLoginAfter());
            CustomerUserLoginVO data = customerUserLoginThirdService.login(dto);
            
            // 拉取家属信息
            LoginUser loginUser=  new LoginUser();
            loginUser.setId(data.getId());
            loginUser.setThirdUserId(data.getThirdId());
            customerFamilyMemberBizService.addFromHis(loginUser);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            lock.unlock();
        }
        
    }
}
