package com.xxh.cloud.customer.framework.modules.login.service.impl;


import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerThirdUserInfo;
import com.xxh.cloud.customer.framework.modules.base.entity.CustomerUserInfo;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerThirdUserInfoService;
import com.xxh.cloud.customer.framework.modules.base.service.CustomerUserInfoService;
import com.xxh.cloud.customer.framework.modules.check.service.CustomerUserRegister2AddCheckService;
import com.xxh.cloud.customer.framework.modules.check.service.CustomerUserValidateService;
import com.xxh.cloud.customer.framework.modules.constant.CustomerUserStatusEnum;
import com.xxh.cloud.customer.framework.modules.constant.XXCustUserConstant;
import com.xxh.cloud.customer.framework.modules.constant.XXCustUserErrorEnum;
import com.xxh.cloud.customer.framework.modules.login.entity.dto.BindByMobileDTO;
import com.xxh.cloud.customer.framework.modules.login.entity.dto.CustomerUserLoginThirdDTO;
import com.xxh.cloud.customer.framework.modules.login.entity.vos.CustomerUserLoginVO;
import com.xxh.cloud.customer.framework.modules.login.helper.CustomerUserLoginHelper;
import com.xxh.cloud.customer.framework.modules.login.service.CustomerUserLoginThirdService;
import com.xxh.cloud.customer.framework.modules.register.service.CustomerUserRegisterMobileService;
import com.xxh.cloud.customer.framework.modules.register.service.CustomerUserRegisterUserNameService;
import com.xxh.cloud.framework.common.constant.DataScopeMode;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Log4j2
@Service
public class CustomerUserLoginThirdServiceImpl implements CustomerUserLoginThirdService {


    @Autowired
    private CustomerThirdUserInfoService customerThirdUserInfoService;
    @Autowired
    private CustomerUserInfoService customerUserInfoService;
    @Autowired
    private CustomerUserRegister2AddCheckService customerUserRegister2AddCheckService;
    @Autowired
    private CustomerUserRegisterMobileService customerUserRegisterMobileService;
    @Autowired
    private CustomerUserRegisterUserNameService customerUserRegisterUserNameService;

    @Autowired
    private CustomerUserValidateService customerUserValidateService;



    @Override
    public CustomerUserLoginVO login(CustomerUserLoginThirdDTO params) {

        CustomerThirdUserInfo thirdUser= customerThirdUserInfoService.getOne(
                new QueryWrapper<CustomerThirdUserInfo>()
                        .eq("params_id",params.getParamsId())
                        // .eq("open_id",params.getOpenId())
                        .and(i -> { // 登录本平台一定有openid；由互联网医院同步的数据可能没有openid但有unionid；判断是否有患者数据是依据openid或unionid
                            i.eq("open_id",params.getOpenId());
                            if (StrUtil.isNotBlank(params.getUnionId())) { // 此判断容错，万一unionid获取不到
                                i.or().eq("union_id", params.getUnionId());
                            }
                        })
        );

        if(thirdUser==null){
            DataScopeMode dataScope = customerUserValidateService.getCustDataScope();
            if (DataScopeMode.CompanyDataScope == dataScope) {
                if(StrUtil.isEmpty(params.getCompanyNo())){
                    throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"机构不能为空");
                }
            }
            thirdUser= CustomerUserLoginHelper.getCustomerThirdUserInfo(params);
            if(params.isReg()){ //注册
                log.info("新的三方用户注册openid={}", thirdUser.getOpenId());
                CustomerUserInfo customerUser = customerUserRegisterUserNameService.register2UserNameByThird(thirdUser);
                thirdUser.setUserId(customerUser.getId());
                customerThirdUserInfoService.save(thirdUser);
                return CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser,customerUser);
            }else {
                customerThirdUserInfoService.save(thirdUser);
                return CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser);
            }
        } else {
            thirdUser.setOpenId(params.getOpenId()); // 更新openid：同步的数据可能没有openid
            if (StrUtil.isNotEmpty(params.getUnionId())) {
                // 更新thirdUser中的unionid
                thirdUser.setUnionId(params.getUnionId());
            }
            customerThirdUserInfoService.updateById(thirdUser);
        }

        if(XXCustUserConstant.CustThirdUserId.equals(thirdUser.getUserId())){
            if(params.isReg()){ //注册
                log.info("原三方用户userid=0时注册,openid={}", thirdUser.getOpenId());
                CustomerUserInfo customerUser = customerUserRegisterUserNameService.register2UserNameByThird(thirdUser);
                thirdUser.setUserId(customerUser.getId());
                customerThirdUserInfoService.updateById(thirdUser);
                return CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser,customerUser);
            }else {
                return CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser);
            }
        }

        CustomerUserInfo customerUser= customerUserInfoService.getById(thirdUser.getUserId());
        if(customerUser==null){
            if(params.isReg()){ //注册
                log.info("三方用户对应的userid不存在时注册,openid={}", thirdUser.getOpenId());
                CustomerUserInfo customerUserReg = customerUserRegisterUserNameService.register2UserNameByThird(thirdUser);
                thirdUser.setUserId(customerUserReg.getId());
                customerThirdUserInfoService.updateById(thirdUser);
                return CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser,customerUserReg);
            }else {
                thirdUser.setUserId(XXCustUserConstant.CustThirdUserId);
                customerThirdUserInfoService.updateById(thirdUser);
                return CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser);
            }
        }
        if(!CustomerUserStatusEnum.CUS_Normal.getCode().equals(customerUser.getStatus())){
            throw new BusinessException(XXCustUserErrorEnum.CustUser_Freeze);
        }
        return CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser,customerUser);
    }

    @Override
    public CustomerUserLoginVO bind(BindByMobileDTO params) {
        ValidatorUtils.validateEntity(params);
        CustomerThirdUserInfo thirdUser=customerThirdUserInfoService.getById(params.getThirdId());
        if(thirdUser==null){
            throw new BusinessException(XXCustUserErrorEnum.ThirdUser_NotExists);
        }
        if(!XXCustUserConstant.CustThirdUserId.equals(thirdUser.getUserId())){
            throw new BusinessException(XXCustUserErrorEnum.ThirdUser_Bind);
        }
        CustomerUserInfo customerUser= customerUserRegister2AddCheckService.getByMobile(thirdUser.getCompanyNo(),params.getMobile());

        if(customerUser!=null){ //绑定
            thirdUser.setUserId(customerUser.getId());
            customerThirdUserInfoService.updateById(thirdUser);
            return  CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser,customerUser);
        }else {
            if(!params.isReg()){
                throw new BusinessException(XXCustUserErrorEnum.ThirdUser_NotBindByUser);
            }else { //注册用户
                customerUser = customerUserRegisterMobileService.innerRegister2MobileByThird(params.getMobile(),thirdUser);
                thirdUser.setUserId(customerUser.getId());
                customerThirdUserInfoService.updateById(thirdUser);
                return  CustomerUserLoginHelper.getCustomerUserLoginVO(thirdUser,customerUser);
            }
        }
    }


}
