package com.xxh.cloud.log.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.BrowseLogGetBatchDTO;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.BrowseLogGetDTO;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.BrowseLogQueryDTO;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.BrowseLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.service.BrowseLogInfoBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

//@SaCheckPermission("store:file:upload")
@RestController
@RequestMapping("/sys/log/browse")
@Api(tags="系统WEB浏览日志管理接口")
public class BrowseLogManagerController {

    @Autowired
    private BrowseLogInfoBizService browseLogInfoBizService;


    @PostMapping("query")
    @ApiOperation(value = "查询" ,response = BrowseLogInfoVO.class)
    @SaCheckPermission("sys:log:browse:query")
    public Result<PageData<BrowseLogInfoVO>> query(@RequestBody BrowseLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<BrowseLogInfoVO> data= browseLogInfoBizService.query(params);
        return new Result<PageData<BrowseLogInfoVO>>().ok(data);
    }



    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:log:browse:info")
    public Result<BrowseLogInfoVO> info(@RequestBody BrowseLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        BrowseLogInfoVO data=browseLogInfoBizService.get(params.getId());
        return new Result<BrowseLogInfoVO>().ok(data);
    }



    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:log:browse:del")
    public Result<String> del(@RequestBody BrowseLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        browseLogInfoBizService.del(params.getId());
        return new Result<String>().ok("ok");
    }


    @PostMapping("delBatch")
    @ApiOperation("批量删除")
    @SaCheckPermission("sys:log:browse:delBatch")
    public Result<String> delBatch(@RequestBody BrowseLogGetBatchDTO params) {
        ValidatorUtils.validateEntity(params);
        browseLogInfoBizService.delBatch(params.getIds());
        return new Result<String>().ok("ok");
    }


}
