package com.xxh.cloud.log.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.*;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.BrowseLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.ErrorLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.service.ErrorLogInfoBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

//@SaCheckPermission("store:file:upload")
@RestController
@RequestMapping("/sys/log/error")
@Api(tags="系统错误日志管理接口")
public class ErrorLogManagerController {

    @Autowired
    private ErrorLogInfoBizService errorLogInfoBizService;


    @PostMapping("query")
    @ApiOperation(value = "查询" ,response = ErrorLogInfoVO.class)
    @SaCheckPermission("sys:log:error:query")
    public Result<PageData<ErrorLogInfoVO>> query(@RequestBody ErrorLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<ErrorLogInfoVO> data= errorLogInfoBizService.query(params);
        return new Result<PageData<ErrorLogInfoVO>>().ok(data);
    }



    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:log:error:info")
    public Result<ErrorLogInfoVO> info(@RequestBody ErrorLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        ErrorLogInfoVO data=errorLogInfoBizService.get(params.getId());
        return new Result<ErrorLogInfoVO>().ok(data);
    }



    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:log:error:del")
    public Result<String> del(@RequestBody ErrorLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        errorLogInfoBizService.del(params.getId());
        return new Result<String>().ok("ok");
    }


    @PostMapping("delBatch")
    @ApiOperation("批量删除")
    @SaCheckPermission("sys:log:error:delBatch")
    public Result<String> delBatch(@RequestBody ErrorLogGetBatchDTO params) {
        ValidatorUtils.validateEntity(params);
        errorLogInfoBizService.delBatch(params.getIds());
        return new Result<String>().ok("ok");
    }





}
