package com.xxh.cloud.log.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.log.framework.modules.biz.entity.dto.*;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.ErrorLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.entity.vos.LoginLogInfoVO;
import com.xxh.cloud.log.framework.modules.biz.service.LoginLogInfoBizService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

//@SaCheckPermission("store:file:upload")
@RestController
@RequestMapping("/sys/log/loginOut")
@Api(tags="用户登录退出日志管理接口")
public class LoginLogManagerController {

    @Autowired
    private LoginLogInfoBizService loginLogInfoBizService;


    @PostMapping("query")
    @ApiOperation(value = "查询" ,response = LoginLogInfoVO.class)
    @SaCheckPermission("sys:log:loginOut:query")
    public Result<PageData<LoginLogInfoVO>> query(@RequestBody LoginLogQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<LoginLogInfoVO> data= loginLogInfoBizService.query(params);
        return new Result<PageData<LoginLogInfoVO>>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:log:loginOut:info")
    public Result<LoginLogInfoVO> info(@RequestBody LoginLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        LoginLogInfoVO data=loginLogInfoBizService.get(params.getId());
        return new Result<LoginLogInfoVO>().ok(data);
    }

    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:log:loginOut:del")
    public Result<String> del(@RequestBody LoginLogGetDTO params) {
        ValidatorUtils.validateEntity(params);
        loginLogInfoBizService.del(params.getId());
        return new Result<String>().ok("ok");
    }


    @PostMapping("delBatch")
    @ApiOperation("批量删除")
    @SaCheckPermission("sys:log:loginOut:delBatch")
    public Result<String> delBatch(@RequestBody LoginLogGetBatchDTO params) {
        ValidatorUtils.validateEntity(params);
        loginLogInfoBizService.delBatch(params.getIds());
        return new Result<String>().ok("ok");
    }




}
