package com.xxh.cloud.store.framework.modules.service.impl;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.OssAliConfigBO;
import com.xxh.cloud.store.framework.modules.service.AliyunCloudStorageService;
import com.xxh.cloud.store.framework.modules.utils.StoreFileNameUtils;
import org.springframework.stereotype.Service;

import java.io.InputStream;

@Service
public class AliyunCloudStorageServiceImpl implements AliyunCloudStorageService {


    @Override
    public String upload(OssAliConfigBO ossAliConfig, InputStream inputStream, String path) {
        OSS client = new OSSClientBuilder().build(ossAliConfig.getEndPoint(),
                ossAliConfig.getAccessKeyId(), ossAliConfig.getAccessKeySecret());
        try {
            client.putObject(ossAliConfig.getBucketName(), path, inputStream, null);
            client.shutdown();
        } catch (Exception e){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR, "上传文件到OSS失败");
        } finally {
            if(client!=null){ client.shutdown();}
        }
        return this.getUrl(ossAliConfig, path);
    }

    @Override
    public String uploadSuffix(OssAliConfigBO ossAliConfig, InputStream inputStream, String suffix) {
        return upload(ossAliConfig, inputStream, StoreFileNameUtils.getPath("", suffix));
    }

    private  String getUrl(OssAliConfigBO ossAliConfig, String path){
        String domain = "https://" + ossAliConfig.getBucketName() + "." + ossAliConfig.getEndPoint()+"/";
        if(!path.startsWith(domain) ) {
            path = domain + path;
        }
        return path;
    }
}
