package com.xxh.cloud.store.framework.modules.utils;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.aliyun.oss.model.MatchMode;
import com.aliyun.oss.model.PolicyConditions;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.OssAliConfigBO;
import com.xxh.cloud.store.framework.modules.constant.OssErrorEnum;

import java.net.URL;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

public class AliYunOssUtils {

	/**
	 * 读取OSS私有读文件
	 * */
	public static String getOssPrivate(OssAliConfigBO ossAliConfigBO, String fileUrl){
		String[] split = fileUrl.split("\\.");
		String bucketName = split[0].split("//")[1];
		if (bucketName.startsWith("http")) {
			bucketName = split[0].split("//")[1];
		}
		if (!ossAliConfigBO.getBucketName().equals(bucketName)) {
			return fileUrl;
		}
		String objectName = fileUrl.split("aliyuncs.com/")[1];
		OSS client = new OSSClientBuilder().build("https://" + ossAliConfigBO.getEndPoint(), ossAliConfigBO.getAccessKeyId(), ossAliConfigBO.getAccessKeySecret());
		// 设置签名URL过期时间为120秒
		Date expiration = new Date(new Date().getTime() + 120 * 1000);

		URL url = client.generatePresignedUrl(bucketName, objectName, expiration);
		client.shutdown();
		return url.toString();
	}

	/**
	 * OSS签名
	 * @param ossAliConfigBO：
	 * @param companyNo：
	 * @param userId：
	 * @param widths :图片压缩尺寸
	 * @param quality :图片质量处理
	 * */
	public static Map<String, String> getOssSign(OssAliConfigBO ossAliConfigBO,
												 String companyNo,
												 String userId,
												 Set<Integer> widths,
												 int quality,
												 String isImg

	){
		OSS client = new OSSClientBuilder().build(ossAliConfigBO.getEndPoint(),
				ossAliConfigBO.getAccessKeyId(), ossAliConfigBO.getAccessKeySecret());
		try {
			long expireTime = 30;
			long expireEndTime = System.currentTimeMillis() + expireTime * 1000;
			Date expiration = new Date(expireEndTime);
			PolicyConditions policyConds = new PolicyConditions();
			policyConds.addConditionItem(PolicyConditions.COND_CONTENT_LENGTH_RANGE, 0, 1048576000);
			policyConds.addConditionItem(MatchMode.StartWith, PolicyConditions.COND_KEY, "");

			String postPolicy = client.generatePostPolicy(expiration, policyConds);
			byte[] binaryData = postPolicy.getBytes("utf-8");
			String encodedPolicy = BinaryUtil.toBase64String(binaryData);
			String postSignature = client.calculatePostSignature(postPolicy);

			Map<String, String> respMap = new LinkedHashMap<String, String>();
			respMap.put("accessid", ossAliConfigBO.getAccessKeyId());
			respMap.put("policy", encodedPolicy);
			respMap.put("signature", postSignature);
			respMap.put("dir", "");
			respMap.put("host", ossAliConfigBO.getHost());
			respMap.put("expire", String.valueOf(expireEndTime / 1000));
			String base64CallbackBody= AliYunOssUtils.getJasonCallback(ossAliConfigBO, companyNo,userId,widths,quality,isImg);
			respMap.put("callback", base64CallbackBody);
			return respMap;
		} catch (Exception e) {
			System.out.println("+++++++++++++++AliYunOSS callback模式生成签名错误++++++++++++ ");
			e.printStackTrace();
			throw new BusinessException(OssErrorEnum.OSS_Sign_Error);
		}

	}

	public static AliYunOssCallbackModel getCallbackModel(String callbackBody) {
		JSONObject jsonObj= JSONObject.parseObject(callbackBody);
		AliYunOssCallbackModel data=new AliYunOssCallbackModel();
		data.setFileName(jsonObj.getString("fileName"));
		data.setFormat(jsonObj.getString("format"));
		data.setMimeType(jsonObj.getString("mimeType"));
		data.setBucket(jsonObj.getString("bucket"));
		data.setEtag(jsonObj.getString("etag"));
		data.setSize(jsonObj.getLongValue("size"));
		data.setHeight(jsonObj.getIntValue("height"));
		data.setWidth(jsonObj.getIntValue("width"));
		data.setBucket(jsonObj.getString("bucket"));
		data.setTenantCode(jsonObj.getString("x:tenant"));
		data.setUserId(jsonObj.getString("x:userId"));
		data.setResizeWidth(jsonObj.getString("x:resize"));
		data.setQuality(jsonObj.getIntValue("x:quality"));
		return data;
	}


	public static String getJasonCallback(OssAliConfigBO ossAliConfigBO,
										  String tenantCode,String userId,
										  Set<Integer> widths, int quality,
										  String isImg) {
		String resize = "";
		if(CollectionUtil.isNotEmpty(widths)){
			for (Integer w : widths) {
				resize = "," + w;
			}
			resize = resize.substring(1);
		}else{
			resize="0";
		}
		if(quality<1 || quality>=100){
			quality=0;
		}
		JSONObject jasonCallback = new JSONObject();
		jasonCallback.put("callbackUrl", ossAliConfigBO.getCallbackUrl());
		jasonCallback.put("callbackBodyType", "application/json");

		if(StrUtil.isNotEmpty(isImg) && YN.YES.getCode().equals(isImg)){
			String callbackBody= StrUtil.format("{"
							+ "\"fileName\":{},"
							+ "\"mimeType\":{},"
							+"\"bucket\":{},"
							+"\"etag\":{},"
							+ "\"size\":{},"
							+"\"height\":{},"
							+"\"width\":{},"
							+"\"format\":{},"
							+"\"x:tenant\":\"{}\","
							+"\"x:bucketType\":\"{}\","
							+"\"x:userId\":\"{}\","
							+"\"x:resize\":\"{}\","
							+"\"x:quality\":\"{}\""
							+ "}"
					,"${object}"
					,"${mimeType}"
					,"${bucket}"
					,"${etag}"
					,"${size}"
					,"${imageInfo.height}"
					,"${imageInfo.width}"
					,"${imageInfo.format}"
					,tenantCode
					,"1"
					,userId
					,resize
					,quality
			);
			jasonCallback.put("callbackBody",callbackBody);
		}else{
			String callbackBody= StrUtil.format("{"
							+ "\"fileName\":{},"
							+ "\"mimeType\":{},"
							+"\"bucket\":{},"
							+"\"etag\":{},"
							+ "\"size\":{},"
							+"\"x:tenant\":\"{}\","
							+"\"x:userId\":\"{}\","
							+"\"x:resize\":\"{}\","
							+"\"x:quality\":\"{}\""
							+ "}"
					,"${object}"
					,"${mimeType}"
					,"${bucket}"
					,"${etag}"
					,"${size}"
					,tenantCode
					,userId
					,resize
					,quality
			);
			jasonCallback.put("callbackBody",callbackBody);
		}
		System.out.println("回调参数base64CallbackBody:" + jasonCallback.toString());
		String base64CallbackBody = BinaryUtil.toBase64String(jasonCallback.toString().getBytes());
		return base64CallbackBody;
	}






	public static void main(String[] args) {
//		String callbackBody= StrUtil.format("{"
//				+ "\"filename\":{},"
//				+ "\"mimeType\":{},"
//				+ "\"size\":{},"
//				+"\"height\":{},"
//				+"\"width\":{},"
//				+"\"format\":{},"
//				+"\"x:company\":\"{}\","
//				+"\"x:userId\":\"{}\""
//				+ "}"
//				, "${object}"
//				,"${mimeType}"
//				,"${size}"
//				,"${imageInfo.height}"
//				,"${imageInfo.width}"
//				,"${imageInfo.format}"
//				,"wjjj"
//				,"111"
//		);
//		System.out.println(callbackBody);

//		String url="http://image-demo.oss-cn-hangzhou.aliyuncs.com/f.jpg";
//		String parameters="x-oss-process=image/info";
//		String data;
//		if(url.indexOf("?")>0){
//			data=StrUtil.format("{}&{}",url,parameters);
//		}else{
//			data=StrUtil.format("{}?{}",url,parameters);
//		}
//		System.out.println(data);
	}






}
