package com.xxh.cloud.store.manager.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.param.bos.OssAliConfigBO;
import com.xxh.cloud.framework.common.param.constant.ParamValTypeEnum;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.store.framework.modules.base.entity.dto.OssCallbackSignParam;
import com.xxh.cloud.store.framework.modules.config.StoreConfig;
import com.xxh.cloud.store.framework.modules.utils.AliYunOssUtils;
import com.xxh.cloud.store.framework.modules.utils.StoreHttpContextUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.Map;

@SaCheckLogin
//@SaCheckPermission("store:file:upload")
@RestController
@RequestMapping("/store/sign")
@Api(tags="文件上传签名模式")
public class FileStoreSignController {

    @Autowired
    private StoreConfig storeConfig;
    @Autowired
    private ParamsClientService paramsClientService;

    @PostMapping("/getAliyunOss")
    @ApiOperation("获取阿里云OSS签名")
    public Result<Map<String, String>> getAliyunOss() {
        // 获取oss配置数据
        OssAliConfigBO ossAliConfig = paramsClientService.getParamConfig(storeConfig.getAliOss(), ParamValTypeEnum.Ali_Oss.getCode(), new OssAliConfigBO());
        LoginUser userInfo= AuthContextUtils.getCurrentLoginUser();
        Map<String, String> respMap = AliYunOssUtils.getOssSign(ossAliConfig,
                userInfo.getCompanyNo(),userInfo.getId(),null,0, YN.NO.getCode());
        HttpServletResponse response= StoreHttpContextUtils.getResponse();
//        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Methods", "GET, POST");
        return new Result<Map<String, String>>().ok(respMap);
    }

    @PostMapping("/getAliyunOssSign")
    @ApiOperation("获取阿里云OSS签名(图片参数)")
    public Result<Map<String, String>> getAliyunOssSign(@RequestBody OssCallbackSignParam params) {
        ValidatorUtils.validateEntity(params);
        LoginUser userInfo= AuthContextUtils.getCurrentLoginUser();
        // 获取oss配置数据
        OssAliConfigBO ossAliConfig =
                paramsClientService.getParamConfig(storeConfig.getAliOss(), ParamValTypeEnum.Ali_Oss.getCode(), new OssAliConfigBO());
        Map<String, String> respMap = AliYunOssUtils.getOssSign(ossAliConfig,
                userInfo.getCompanyNo(),userInfo.getId(),params.getResizeWidht(),params.getQuality(),params.getIsImg());
        HttpServletResponse response= StoreHttpContextUtils.getResponse();
//        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Methods", "GET, POST");
        return new Result<Map<String, String>>().ok(respMap);
    }

}
