package com.xxh.cloud.third.framework.modules.captcha.service.impl;

import cn.hutool.captcha.CaptchaUtil;
import cn.hutool.captcha.CircleCaptcha;
import cn.hutool.captcha.LineCaptcha;
import cn.hutool.captcha.ShearCaptcha;
import cn.hutool.captcha.generator.RandomGenerator;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.third.framework.modules.captcha.entity.dto.ImageCaptchaCheckDTO;
import com.xxh.cloud.third.framework.modules.captcha.entity.dto.ImageCaptchaCreateDTO;
import com.xxh.cloud.third.framework.modules.captcha.entity.enums.XXThirdErrorEnum;
import com.xxh.cloud.third.framework.modules.captcha.entity.vos.ImageCaptchaVO;
import com.xxh.cloud.third.framework.modules.captcha.service.ImageCaptchaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class ImageCaptchaServiceImpl implements ImageCaptchaService {

    @Autowired
    private RedisUtils redisUtils;



    private void check(ImageCaptchaCreateDTO params){
        if(params.getWidth()<=0){
            params.setWidth(160);
        }

        if(params.getHeight()<=0){
            params.setHeight(60);
        }

        if(params.getCacheExpire()<=0 || params.getCacheExpire()>=3600){
            params.setCacheExpire(300);
        }


        if(StrUtil.isEmpty(params.getType())){
            params.setType("LineCaptcha");
            params.setNum(150);
        }else if("LineCaptcha".equals(params.getType())){
            if(params.getNum()<=0){
                params.setNum(150);
            }
        }else if("CircleCaptcha".equals(params.getType())){
            if(params.getNum()<=0){
                params.setNum(20);
            }
        }else if("ShearCaptcha".equals(params.getType())){
            if(params.getNum()<=0){
                params.setNum(4);
            }
        }else {
            params.setType("LineCaptcha");
            params.setNum(150);
        }
    }


    @Override
    public ImageCaptchaVO createImageCaptcha(ImageCaptchaCreateDTO params) {
        RandomGenerator randomGenerator =new RandomGenerator("0123456789",4);
        String cacheKey=StrUtil.format("IC{}", IdUtil.fastSimpleUUID());
        if("LineCaptcha".equals(params.getType())){
            LineCaptcha lineCaptcha = CaptchaUtil.createLineCaptcha(params.getWidth(),params.getHeight(),4,params.getNum());
            lineCaptcha.setGenerator(randomGenerator);
            lineCaptcha.createCode();
            redisUtils.set(cacheKey,lineCaptcha.getCode(),params.getCacheExpire());
            return new ImageCaptchaVO(lineCaptcha.getImageBase64(), cacheKey);
        }else if("CircleCaptcha".equals(params.getType())){
            CircleCaptcha lineCaptcha = CaptchaUtil.createCircleCaptcha(params.getWidth(),params.getHeight(),4,params.getNum());
            lineCaptcha.setGenerator(randomGenerator);
            lineCaptcha.createCode();
            redisUtils.set(cacheKey,lineCaptcha.getCode(),params.getCacheExpire());
            return new ImageCaptchaVO(lineCaptcha.getImageBase64(), cacheKey);
        }else if("ShearCaptcha".equals(params.getType())){
            ShearCaptcha lineCaptcha = CaptchaUtil.createShearCaptcha(params.getWidth(),params.getHeight(),4,params.getNum());
            lineCaptcha.setGenerator(randomGenerator);
            lineCaptcha.createCode();
            redisUtils.set(cacheKey,lineCaptcha.getCode(),params.getCacheExpire());
            return new ImageCaptchaVO(lineCaptcha.getImageBase64(), cacheKey);
        }
        return null;
    }

    @Override
    public boolean check(ImageCaptchaCheckDTO params) {
        Object obj = redisUtils.get(params.getCaptchaVerification());
        if(obj==null){
            return false;
        }

        if(params.getCode().toLowerCase().equals(obj.toString().toLowerCase())){
            redisUtils.delete(params.getCaptchaVerification()); //ok-del
            return true;
        }else {
            return false;
        }
    }

    @Override
    public void validate(ImageCaptchaCheckDTO params) {
        Object obj = redisUtils.get(params.getCaptchaVerification());
        if(obj==null){
            throw new BusinessException(XXThirdErrorEnum.Vcode_Expire);
        }
        if(!params.getCode().toLowerCase().equals(obj.toString().toLowerCase())){
            throw new BusinessException(XXThirdErrorEnum.Vcode_Check_Error);
        }
        redisUtils.delete(params.getCaptchaVerification()); //ok-del
    }

    @Override
    public void validate(String captchaVerification, String code) {
        this.validate(new ImageCaptchaCheckDTO(captchaVerification,code));
    }


}
