package com.xxh.cloud.third.framework.modules.captcha.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.basic.client.api.bean.bos.ParamsBO;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.SmsVcodeConfigBO;
import com.xxh.cloud.framework.common.param.constant.ParamValTypeEnum;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.third.framework.modules.captcha.entity.enums.XXThirdErrorEnum;
import com.xxh.cloud.third.framework.modules.captcha.service.SmsCaptchaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.security.Key;
import java.util.Map;

@Service
public class SmsCaptchaServiceImpl implements SmsCaptchaService {

    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private ParamsClientService paramsClientService;

    private String getCacheKey(String type, String mobile){
        return StrUtil.format("sms:captcha:{}:{}",type,mobile);
    }



    /**
     * 验证码发送配置
     * */
    private SmsVcodeConfigBO getSmsVcodeConfig(){
        ParamsBO params= paramsClientService.getParamsById(ParamValTypeEnum.VCode_Sms.getCode());
        if(params==null){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR);
        }
        if(MapUtil.isEmpty(params.getParamValue())){
            throw new BusinessException(BusinessBaseErrorEnum.Config_GET_ERROR);
        }
        SmsVcodeConfigBO config= BeanUtil.fillBeanWithMap(params.getParamValue(),new SmsVcodeConfigBO(),false);
        return config;
    }



//    public static <T> T fillBeanWithMap(Map<?, ?> map, T bean, boolean isIgnoreError) {
//        return fillBeanWithMap(map, bean, false, isIgnoreError);
//    }


    /**
     * 生成手机验证码
     * */
    private String getSmsCode(SmsVcodeConfigBO config){
        //todo:发送
        return "8888";
    }


    /**
     * 发送手机验证码
     * */
    private void sendSmsCode(SmsVcodeConfigBO config,String code){
        //todo:发送
    }

    /**
     * @param  cacheExpire : (秒)
     * */
    @Override
    public String send(String type, String mobile,long cacheExpire) {
        //配置
        SmsVcodeConfigBO config=this.getSmsVcodeConfig();
        //生成
        String vcode=this.getSmsCode(config);
        //缓存
        String cacheKey=this.getCacheKey(type,mobile);
        //todo:code转大写
        redisUtils.hSet(cacheKey,vcode,vcode,cacheExpire);
        //发送
        this.sendSmsCode(config,vcode);
        return vcode;
    }

    @Override
    public void validate(String type,String mobile,String code) {
        String cacheKey=this.getCacheKey(type,mobile);
        Map<String, Object> cacheMap= redisUtils.hGetAll(cacheKey);
        if(MapUtil.isEmpty(cacheMap)){
            throw new BusinessException(XXThirdErrorEnum.Vcode_Expire);
        }
        //todo:code转大写
        if(cacheMap.keySet().contains(code)){
            //验证成功
           redisUtils.delete(cacheKey);
        }else {
            //验证失败
            throw new BusinessException(XXThirdErrorEnum.Vcode_Check_Error);
        }
    }

    @Override
    public boolean check(String type,String mobile, String code) {
        String cacheKey=this.getCacheKey(type,mobile);
        Map<String, Object> cacheMap= redisUtils.hGetAll(cacheKey);
        if(MapUtil.isEmpty(cacheMap)){
            return false;
        }
        //todo:code转大写
        if(cacheMap.keySet().contains(code)){
            //验证成功
            redisUtils.delete(cacheKey);
            return true;
        }else {
            //验证失败
            return false;
        }
    }
}
