package com.xxh.cloud.third.framework.modules.thirdHis;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.third.framework.modules.thirdHis.dto.HisGenerateSchemeDTO;
import com.xxh.cloud.third.framework.modules.thirdHis.dto.HisPatientDTO;
import com.xxh.cloud.third.framework.modules.thirdHis.dto.HisPromotionCodeDTO;
import com.xxh.cloud.third.framework.modules.thirdHis.model.*;
import lombok.extern.slf4j.Slf4j;

import org.springframework.stereotype.Component;

@Slf4j
@Component
public class HisWebApiHelper {

    // 获取accessToken
    private static String accessTokenUrl = "/api/his/getAccessToken";

    // 获取患者信息
    private static String patientByUnionIdUrl = "/api/his/getPatientByUnionId";

    // 获取患者信息
    private static String patientByThirdId = "/api/his/getPatientByThirdId";

    // 批量获取就诊人信息
    private static String patientByTimestamp = "/api/his/getPatientByTimestamp";

    // 获取科室信息
    private static String deptsInfoUrl = "/api/his/getDeptsInfo";

    // 获取医生信息
    private static String doctorsInfoUrl = "/api/his/getDoctorsInfo";

    // 获取schemeurl
    private static String schemeUrl = "/api/his/getWxMiniSchemeUrl";

    // 获取并保存推广码
    private static String promotionCodeUrl = "/api/his/getAndSavePromotionCodeUrl";


    /**公众号accessToken*/
    public static RespBaseThird<HisWxTokenModel> getAccessToken(HealthyFrontendConfig config) {
        log.info("获取公众号token>>>> config={}", JSON.toJSONString(config));
        // String result = HttpRequest.get(config.getHost() + accessTokenUrl + "?type=1").execute().body();
        String result = loopGetHttp(config.getHost() + accessTokenUrl + "?type=1", 0);
        log.info("获取公众号token<<<< {}", result);
        return new RespBaseThird<>(result, HisWxTokenModel.class);
    }

    /**小程序accessToken*/
    public static RespBaseThird<HisWxTokenModel> getMiniAccessToken(HealthyFrontendConfig config) {
        log.info("获取小程序token>>>> config={}", JSON.toJSONString(config));
        // String result = HttpRequest.get(config.getHost() + accessTokenUrl + "?type=2").execute().body();
        String result = loopGetHttp(config.getHost() + accessTokenUrl + "?type=2", 0);
        log.info("获取小程序token<<<< {}", result);
        return new RespBaseThird<>(result, HisWxTokenModel.class);
    }

    /**查询绑定患者的数据*/
    public static RespBaseThird<PatBindModel> getPatBind(HealthyFrontendConfig config, String unionId) {
        log.info("获取关联就诊卡号>>>> config={}; params={}", JSON.toJSONString(config), unionId);
        // String result = HttpRequest.get(config.getHost() + patientByUnionIdUrl + "?unionid=" + unionId).execute().body();
        String result = loopGetHttp(config.getHost() + patientByUnionIdUrl + "?unionid=" + unionId, 0);
        log.info("获取关联就诊卡号<<<< {}", result);
        return new RespBaseThird<>(result, PatBindModel.class);
    }

    /**查询绑定患者的数据*/
    public static RespBaseThird<PatBindModel> getPatBindByThirdId(HealthyFrontendConfig config, HisPatientDTO params) {
        log.info("获取关联就诊卡号>>>> config={}; params={}", JSON.toJSONString(config), JSON.toJSONString(params));
        // String result = HttpRequest.post(config.getHost() + patientByThirdId).body(JSON.toJSONString(params)).execute().body();
        String result = loopPostHttp(config.getHost() + patientByThirdId, JSON.toJSONString(params), 0);
        log.info("获取关联就诊卡号<<<< {}", result);
        return new RespBaseThird<>(result, PatBindModel.class);
    }

    /**查询所有患者的数据*/
    public static RespBaseThird<HisPatientListModel> getPatientByTimestamp(HealthyFrontendConfig config, HisPatientDTO params) {
        log.info("获取就诊人信息>>>> config={}; params={}", JSON.toJSONString(config), JSON.toJSONString(params));
        // String result = HttpRequest.post(config.getHost() + patientByTimestamp).body(JSON.toJSONString(params)).execute().body();
        String result = loopPostHttp(config.getHost() + patientByTimestamp, JSON.toJSONString(params), 0);
        log.info("获取就诊人信息<<<< {}", result);
        return new RespBaseThird<>(result, HisPatientListModel.class);

    }

    /**查询科室的数据*/
    public static RespBaseThird<HisDeptListModel> getDeptsInfo(HealthyFrontendConfig config) {
        log.info("获取科室>>>>" + JSON.toJSONString(config));
        // String result = HttpRequest.get(config.getHost() + deptsInfoUrl).execute().body();
        String result = loopGetHttp(config.getHost() + deptsInfoUrl, 0);
        log.info("获取科室<<<< {}", result);
        return new RespBaseThird<>(result, HisDeptListModel.class);
    }

    /**查询医生的数据*/
    public static RespBaseThird<HisDoctorListModel> getDoctorsInfo(HealthyFrontendConfig config, String showModel) {
        log.info("获取医生>>>> config={}; params={}", JSON.toJSONString(config), showModel);
        // String result = HttpRequest.get(config.getHost() + doctorsInfoUrl + "?showModel=" + showModel).execute().body();
        String result = loopGetHttp(config.getHost() + doctorsInfoUrl + "?showModel=" + showModel, 0);
        log.info("获取医生<<<< {}", result);
        return new RespBaseThird<>(result, HisDoctorListModel.class);
    }

    /** 获取schemeurl */
    public static RespBaseThird<HisSchemeUrlModel> generateScheme(HealthyFrontendConfig config, HisGenerateSchemeDTO params) {
        log.info("获取schemeurl>>>> config{}; params={}", JSON.toJSONString(config), JSON.toJSONString(params));
        // String result = HttpRequest.post(config.getHost() + schemeUrl).body(JSONObject.toJSONString(params)).execute().body();
        String result = loopPostHttp(config.getHost() + schemeUrl, JSONObject.toJSONString(params), 0);
        log.info("获取schemeurl<<<< {}", result);
        return new RespBaseThird<>(result, HisSchemeUrlModel.class);
    }

    /** 获取并保存推广码 */
    public static RespBaseThird<HisPromotionCodeModel> getAndSavePromotionCodeUrl(HealthyFrontendConfig config, HisPromotionCodeDTO params) {
        log.info("获取并保存推广码>>>> config{}; params={}", JSON.toJSONString(config), JSON.toJSONString(params));
        // String result = HttpRequest.post(config.getHost() + promotionCodeUrl).body(JSONObject.toJSONString(params)).execute().body();
        String result = loopPostHttp(config.getHost() + promotionCodeUrl, JSONObject.toJSONString(params), 0);
        log.info("获取并保存推广码<<<< {}", result);
        return new RespBaseThird<>(result, HisPromotionCodeModel.class);
    }

    // 调用失败则暂停5秒重新请求，最多重调5次
    private static String loopGetHttp(String url, Integer loopNum) {
        try {
            return HttpRequest.get(url).execute().body();
        } catch (Exception e) {
            log.info("GET调用异常: {}", url);
            e.printStackTrace();
            
            // 最多循环5次
            if (loopNum < 5) {
                // 先暂停
                log.info("休眠5000ms");
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                }
                // 出现异常再次调用
                return loopGetHttp(url, loopNum + 1);
            }
        }
        return null;
    }

    // 调用失败则暂停5秒重新请求，最多重调5次
    private static String loopPostHttp(String url, String body, Integer loopNum) {
        try {
            return HttpRequest.post(url).body(body).execute().body();
        } catch (Exception e) {
            log.info("POST调用异常: {} ", url);
            e.printStackTrace();
            
            // 最多循环5次
            if (loopNum < 5) {
                // 先暂停
                log.info("休眠5000ms");
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                }
                // 出现异常再次调用
                return loopPostHttp(url, body, loopNum + 1);
            }
        }
        return null;
    }
}
