package com.xxh.cloud.third.framework.modules.thirdWX.service;

import cn.hutool.core.util.StrUtil;
//import com.company.cloud.sdk.weixin.mp.WxMpApi;
//import com.company.cloud.sdk.weixin.mp.WxMpApi1;
import com.alibaba.fastjson.JSONObject;
import com.company.cloud.sdk.weixin.mp.model.AccessTokenModel;
import com.company.cloud.sdk.weixin.mp.model.AccessTokenResp;
import com.company.cloud.sdk.weixin.mp.model.JsapiTicketModel;
import com.company.cloud.sdk.weixin.mp.model.JsapiTicketResp;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.WxApiConfig;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.third.framework.modules.thirdHis.HisApiHelper;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HisAccessTokenModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.WeixinApiHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;

/**
 * 微信公众号和小程序 :全局唯一接口调用凭据 AccessToken
 * 公众号: 全局 jsapi_ticket的有效期为7200秒
 * */
@Log4j2
@Service
public class ThirdWeixinAccessTokenService {

    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private ParamsClientService paramsClientService;


    private String getAccessTokenCacheKey(String appid){
        return StrUtil.format("wx:AccessToken:{}",appid);
    }

    private String getJsapiTokenCacheKey(String appid){
        return StrUtil.format("wx:JsapiToken:{}",appid);
    }


    /**
     * 移除 微信AccessToken
     * */
    public void removeAccessToken(WxApiConfig config){
        String cacheKey=this.getAccessTokenCacheKey(config.getAppid());
        redisUtils.delete(cacheKey);
    }


    /**
     * 重新获取 微信AccessToken
     * */
    public String reloadAccessToken(WxApiConfig config){
        String cacheKey=this.getAccessTokenCacheKey(config.getAppid());
        redisUtils.delete(cacheKey);
        AccessTokenResp accessTokenResp=this._getAccessToken(config);
        if(accessTokenResp==null){
            //todo:报错
            return null;
        }
        redisUtils.set(cacheKey,accessTokenResp.getAccess_token(),accessTokenResp.getExpires_in());
        return accessTokenResp.getAccess_token();
    }

    /**
     * 获取 微信AccessToken
     * */
    public String getAccessToken(WxApiConfig config){
//        String cacheKey=this.getAccessTokenCacheKey(config.getAppid());
//        Object accessToken= redisUtils.get(cacheKey);
//        if(accessToken!=null){
//            return accessToken.toString();
//        }
//        AccessTokenResp accessTokenResp=this._getAccessToken(config);
//        if(accessTokenResp==null){
//            return null;
//        }
//        redisUtils.set(cacheKey,accessTokenResp.getAccess_token(),accessTokenResp.getExpires_in());
//        return accessTokenResp.getAccess_token();
        // 调用中转机获取token 本地不缓存 20231125 mj-li
        AccessTokenResp accessTokenResp=this._getAccessToken(config);
        if(accessTokenResp==null){
            //todo:报错
            return null;
        }
        return accessTokenResp.getAccess_token();
    }

    private AccessTokenResp _getAccessToken(WxApiConfig config){
        AccessTokenModel param=new AccessTokenModel();
        param.setAppid(config.getAppid());
        param.setSecret(config.getSecret());
//        AccessTokenResp resp= WxMpApi.accessToken(param); 调用微信获取AccessToken

        // 改成调用三方系统
        HealthyFrontendConfig healthyFrontend = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        RespBaseThird<HisAccessTokenModel> respBaseThird = HisApiHelper.getAccessToken(healthyFrontend);
        AccessTokenResp resp = new AccessTokenResp();
        resp.setAccess_token(respBaseThird.getData().getAccToken());
        return resp;
    }



    //================================================
    /**
     * 获取 微信jsapi_ticket
     * */
    public String getJsapiToken(WxApiConfig config){
        String cacheKey=this.getJsapiTokenCacheKey(config.getAppid());
        Object jsapiToken= redisUtils.get(cacheKey);
        if(jsapiToken!=null){
            return jsapiToken.toString();
        }
        String accessToken= this.getAccessToken(config);

        JsapiTicketResp jsapiTicketResp=this.getJsapiToken(accessToken);
        if(jsapiTicketResp==null){
            //todo:报错
            return null;
        }
        redisUtils.set(cacheKey,jsapiTicketResp.getTicket(),jsapiTicketResp.getExpires_in());
        return jsapiTicketResp.getTicket();
    }

    /**
     * 重新获取 微信jsapi_ticket
     * */


    /**
     * 移除 微信jsapi_ticket
     * */

    private JsapiTicketResp getJsapiToken(String accessToken){
        JsapiTicketModel wxReq=new JsapiTicketModel();
        wxReq.setAccess_token(accessToken);
//        JsapiTicketResp resp= WxMpApi.jsapiTicket(wxReq); 调用微信的方法

        // 改成调用三方系统
        HealthyFrontendConfig config = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        RespBaseThird<JsapiTicketResp> resp = WeixinApiHelper.jsapiTicket(config, accessToken);
        log.info("getJsapiToken" + JSONObject.toJSONString(resp));
        return resp.getData();
    }





}
