package com.xxh.cloud.third.framework.modules.thirdWX.service;

import cn.hutool.core.util.StrUtil;

import com.alibaba.fastjson2.JSON;
import com.company.cloud.wx.sdk.work.inner.WxWorkInnerApi;
import com.company.cloud.wx.sdk.work.inner.model.Code2TicketModel;
import com.company.cloud.wx.sdk.work.inner.model.Code2TicketResp;
import com.company.cloud.wx.sdk.work.intra.WxWorkCommonApi;
import com.company.cloud.wx.sdk.work.model.common.AccessTokenModel;
import com.company.cloud.wx.sdk.work.model.common.AccessTokenResp;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.param.bos.WxApiConfig;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdUnifyUser.entity.bos.ThirdUnifyUserLoginBO;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.ThirdUserLoginWeixinConvert;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.WeixinWorkApiHelper;

import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Log4j2
@Service
public class ThirdWeixinLoginWorkService {

    @Autowired
    private ParamsClientService paramsClientService;
    @Autowired
    private RedisUtils redisUtils;

    public ThirdUnifyUserLoginBO getLoginAuthToken(WxApiConfig config, String code){
        
        String accessToken = this.getAccessToken(config);

        Code2TicketResp resp = this.getWorkUserInfo(accessToken, code);
        if (null == resp || resp.getErrcode() != 0) {
            throw new BusinessException("获取用户身份失败");
        }

        return ThirdUserLoginWeixinConvert.getThirdUnifyUserLoginBO(resp);
    }

    // 获取accessToken
    public String getAccessToken(WxApiConfig config) {
        String cacheKey=StrUtil.format("wx:work:AccessToken:{}", config.getAppid());
        Object accessToken = redisUtils.get(cacheKey);
        if(accessToken!=null){
            return accessToken.toString();
        }

        // 直接调用企业微信获取accessToken
        // AccessTokenModel accessTokenModel = new AccessTokenModel();
        // accessTokenModel.setCorpid(config.getAppid());
        // accessTokenModel.setCorpsecret(config.getSecret());
        // log.info("获取token >>>>>> {}", JSON.toJSONString(accessTokenModel));
        // AccessTokenResp accessTokenResp = WxWorkCommonApi.accessToken(accessTokenModel);
        // log.info("获取token <<<<<< {}", JSON.toJSONString(accessTokenResp));
        // if (accessTokenResp.getErrcode() != 0 || StrUtil.isBlank(accessTokenResp.getAccess_token())) {
        //     throw new BusinessException("获取token失败");
        // }
        // redisUtils.set(cacheKey,accessTokenResp.getAccess_token(), accessTokenResp.getExpires_in() - 200);
        // return accessTokenResp.getAccess_token();


        // 调用前置机访问企业微信
        HealthyFrontendConfig healthyFrontend = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        RespBaseThird<AccessTokenResp> resp = WeixinWorkApiHelper.getWorkAccessToken(healthyFrontend, config);
        redisUtils.set(cacheKey,resp.getData().getAccess_token(), resp.getData().getExpires_in() - 200);
        return resp.getData().getAccess_token();
    }

    // 获取企业微信用户信息
    public Code2TicketResp getWorkUserInfo(String accessToken, String code) {
        Code2TicketModel code2TicketModel = new Code2TicketModel();
        code2TicketModel.setAccess_token(accessToken);
        code2TicketModel.setCode(code);

        // 直接调用企业微信获取用户身份信息
        // Code2TicketResp resp = WxWorkInnerApi.getUserInfo(code2TicketModel);
        // log.info("获取user info <<<<<< {}", JSON.toJSONString(resp));
        // return resp;

        // 调用前置机访问微信
        HealthyFrontendConfig healthyFrontend = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());
        RespBaseThird<Code2TicketResp> resp = WeixinWorkApiHelper.getWorkUserInfo(healthyFrontend, code2TicketModel);
        log.info("获取user info <<<<<< {}", JSON.toJSONString(resp));
        return resp.getData();
    }




}
