package com.xxh.cloud.third.framework.modules.thirdWX.service;

import cn.hutool.core.util.StrUtil;
import com.company.cloud.sdk.weixin.mp.model.RefreshTokenModel;
import com.company.cloud.sdk.weixin.mp.model.RefreshTokenResp;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdWX.helper.WeixinApiHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 *  微信公众号，授权后 网页授权接口调用凭证
 * */
@Service
public class ThirdWeixinPageAccessTokenService {

    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private ParamsClientService paramsClientService;


    private String getPageAccessTokenCacheKey(String appId ,String openId){
        return StrUtil.format("WX:Page:AccessToken:{}:{}",appId,openId);
    }

    private String getPageRefreshTokenCacheKey(String appId ,String openId){
        return StrUtil.format("WX:Page:RefreshToken:{}:{}",appId,openId);
    }


    /**
     * 设置 页面access_token和refresh_token有效期为30天
     * */
    public void setPageAccessToken2RefreshToken(String appId,String openId, String access_token,int expires_in, String refresh_token){
        this.setPageAccessToken(appId,openId,access_token,expires_in);
        this.setPageRefreshToken(appId,openId,refresh_token);
    }

    /**
     * 设置 页面access_token
     * */
    public void setPageAccessToken(String appId,String openId, String access_token,int expires_in){
        String cacheKey=getPageAccessTokenCacheKey(appId,openId);
        redisUtils.set(cacheKey,access_token,expires_in);
    }

    /**
     * 获取 页面access_token
     * */
    public String getPageAccessToken(String appId,String openId){
        String cacheKey=getPageAccessTokenCacheKey(appId,openId);
        Object accessToken= redisUtils.get(cacheKey);
        if(accessToken!=null){
            return accessToken.toString();
        }
        return this.refreshPageAccessToken(appId,openId);
    }

    /**
     * 设置 refresh_token有效期为30天
     * */
    public void setPageRefreshToken(String appId, String openId, String refresh_token){
        String cacheKey=getPageRefreshTokenCacheKey(appId,openId);
        redisUtils.set(cacheKey,refresh_token,RedisUtils.Day_Thirty_EXPIRE);
    }


    /**
     * 刷新 网页授权调用凭证
     * */
    public String refreshPageAccessToken(String appId, String openId){
        String cacheKey=getPageRefreshTokenCacheKey(appId,openId);
        Object pageRefreshToken = redisUtils.get(cacheKey);
        if(pageRefreshToken==null){
            return null;
        }
        return this.refresh(appId,pageRefreshToken.toString());
    }



    /**
     *  刷新网页授权调用凭证
     * */
    private String refresh(String appId,String refresh_token){
        RefreshTokenModel param=new RefreshTokenModel();
        param.setAppid(appId);
        param.setRefresh_token(refresh_token);
//        RefreshTokenResp resp= WxMpApi.token2RefreshToken(param); 调用微信的方法

        // 改成调用三方系统
        HealthyFrontendConfig healthyFrontend = paramsClientService.getParamConfig("HEALTHY_FRONTEND", new HealthyFrontendConfig());

        RespBaseThird<RefreshTokenResp> respBaseThird = WeixinApiHelper.refreshTokenPage(healthyFrontend, param);
        if(respBaseThird.getData() == null){
            return null;
        }
        RefreshTokenResp resp = respBaseThird.getData();
        this.setPageAccessToken2RefreshToken(appId,resp.getOpenid(),resp.getAccess_token(),resp.getExpires_in(),resp.getRefresh_token());
        return resp.getAccess_token();
    }



//    /**
//     * 刷新全部网页授权调用凭证
//     * */
//    public void refreshPageAccessToken(){
//
//    }
//
//
//    /**
//     * 刷新指定的appid网页授权调用凭证
//     * */
//    public void refreshPageAccessToken(String appId){
//
//    }








}
