package com.xxh.cloud.user.company.application.controller;


import cn.dev33.satoken.annotation.SaCheckLogin;
import com.alibaba.fastjson2.JSON;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.user.framework.modules.base.biz.UserInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.UserMenuSearchDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.MenuVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.UserVO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserUpdateBySelfDTO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserUpdatePasswordBySelfDTO;
import com.xxh.cloud.user.framework.modules.login.service.UserPermissionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@SaCheckLogin
@RestController
@RequestMapping("/app/com/user")
@Api(tags="商户端用户(服务),用户服务")
public class UserComAppController {

    @Autowired
    private UserInfoBizService userInfoBizService;

    @Autowired
    private UserPermissionService userPermissionService;


    @PostMapping("getMenu")
    @ApiOperation("获取当前用户菜单")
    public Result<List<MenuVO>> getMenu() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        List<MenuVO> data= userPermissionService.getMenuTreeByUser(loginUser.getId());
        return new Result<List<MenuVO>>().ok(data);
    }

    @PostMapping("getMenuList")
    @ApiOperation("获取当前用户菜单列表")
    public Result<List<MenuVO>> getMenuList(@RequestBody UserMenuSearchDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        List<MenuVO> data= userPermissionService.getMenuList(loginUser.getId(), params);
        return new Result<List<MenuVO>>().ok(data);
    }

    @PostMapping("getInfoBySelf")
    @ApiOperation("获取当前用户信息")
    public Result<UserVO> getInfoBySelf() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        System.out.println("+++++++++++++++++++++++++++++ "+ JSON.toJSONString(loginUser));
        UserVO data= userInfoBizService.infoById(loginUser.getId());
        return new Result<UserVO>().ok(data);
    }

    @PostMapping("updateBySelf")
    @ApiOperation("修改当前用户信息")
    public Result<UserVO> updateBySelf(@RequestBody UserUpdateBySelfDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        UserVO data= userInfoBizService.update(params, loginUser);
        return new Result<UserVO>().ok(data);
    }

    @PostMapping("updatePasswordBySelf")
    @ApiOperation("修改当前用户密码")
    public Result<UserVO> updatePasswordBySelf(@RequestBody UserUpdatePasswordBySelfDTO params) {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        UserVO data= userInfoBizService.changePassword(params, loginUser);
        return new Result<UserVO>().ok(data);
    }


    @PostMapping("logout")
    @ApiOperation("注销当前用户")
    public Result<String> logout() {
        LoginUser loginUser= AuthContextUtils.getCurrentLoginUser();
        //AuthContextUtils.logout(loginUser.getId());
        System.out.println("++++ loginUser token : "+loginUser.getToken());
        AuthContextUtils.logoutByToken(loginUser.getToken());
        return new Result<String>().ok("logout ok");
    }






}
