package com.xxh.cloud.user.company.application.controller;


import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.third.framework.modules.captcha.entity.dto.ImageCaptchaCreateDTO;
import com.xxh.cloud.third.framework.modules.captcha.entity.vos.ImageCaptchaVO;
import com.xxh.cloud.third.framework.modules.captcha.service.ImageCaptchaService;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserPasswordLoginDTO;
import com.xxh.cloud.user.framework.modules.login.entity.vos.UserLoginVO;
import com.xxh.cloud.user.framework.modules.login.service.UserLoginService;
import com.xxh.cloud.user.framework.modules.login.service.UserPermissionService;
import com.xxh.cloud.user.framework.modules.register.entity.dto.UserRegisterDto;
import com.xxh.cloud.user.framework.modules.register.service.UserRegisterService;
import com.xxh.cloud.web.common.service.StpUserPermissionCacheService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Set;

@SaIgnore
@RestController
@RequestMapping("/app/com/userlogin")
@Api(tags="商户端用户(服务),用户登录服务")
public class UserLoginComAppController {


    @Autowired
    private UserLoginService userLoginService;
    @Autowired
    private UserRegisterService userRegisterService;
    @Autowired
    private UserPermissionService userPermissionService;
    @Autowired
    private StpUserPermissionCacheService stpUserPermissionCacheService;

    @Autowired
    private ImageCaptchaService imageCaptchaService;

    @PostMapping("loginByNamePassword")
    @ApiOperation("用户密码验证码登录")
    public Result<UserLoginVO> loginByNamePassword(@RequestBody UserPasswordLoginDTO params) {
        ValidatorUtils.validateEntity(params);
        imageCaptchaService.validate(params.getCaptchaVerification(), params.getWordVCode());// 校验图形验证码
        UserLoginVO data= userLoginService.loginBySys(params);
        Set<String> permissions= userPermissionService.getPermissionListByUser(data.getId());
        if (CollUtil.isNotEmpty(permissions)) {
            stpUserPermissionCacheService.addUserPermission(data.getId(),permissions);
        }
        return new Result<UserLoginVO>().ok(data);
    }

    @PostMapping("registerByUserName")
    @ApiOperation("用户密码验证码注册")
    public Result<Object> registerByUserName(@RequestBody UserRegisterDto params) {
        ValidatorUtils.validateEntity(params);
        imageCaptchaService.validate(params.getCaptchaVerification(), params.getWordVCode());// 校验图形验证码
        userRegisterService.registerByUserName(params);
        return new Result<>().success();
    }


    @PostMapping("getImageWordCaptcha")
    @ApiOperation("获取图片验证码")
    public Result<ImageCaptchaVO> getImageWordCaptcha(@RequestBody ImageCaptchaCreateDTO params) {
        params.setType("CircleCaptcha");
        params.setNum(20);
        params.setCacheExpire(300);
        ImageCaptchaVO data= imageCaptchaService.createImageCaptcha(params);
        return new Result<ImageCaptchaVO>().ok(data);
    }



}
