package com.xxh.cloud.user.framework.modules.base.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.PostInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.PostInfo;
import com.xxh.cloud.user.framework.modules.base.entity.dto.PostInfoAddDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.PostInfoEditDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.PostInfoListDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.PostInfoQueryDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.PostInfoVO;
import com.xxh.cloud.user.framework.modules.base.helper.PostInfoConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.PostInfoService;
import com.xxh.cloud.user.framework.modules.cache.service.PostInfoCacheService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Log4j2
@Service
public class PostInfoBizServiceImpl implements PostInfoBizService {

    @Autowired
    private PostInfoService postInfoService;
    @Autowired
    private PostInfoCacheService postInfoCacheService;

    @Override
    public PageData<PostInfoVO> query(PostInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        IPage<PostInfo> page= postInfoService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<PostInfo>()
                        .eq("rows_status", YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .orderByDesc("sort")
        );
        PageData<PostInfoVO> data= new PageConverter<PostInfo, PostInfoVO>() {
            @Override
            public PostInfoVO converter(PostInfo entity) {
                return PostInfoConvertHelper.getPostInfoVO(entity);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<PostInfoVO> list(PostInfoListDTO params) {
        List<PostInfo> list= postInfoService.list(
                new QueryWrapper<PostInfo>()
                        .eq("rows_status",YN.YES.getCode())
                        .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .like(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                        .orderByDesc("sort")
        );
        List<PostInfoVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
            return data;
        }
        list.forEach(item->{
            data.add(PostInfoConvertHelper.getPostInfoVO(item));
        });
        return data;
    }

    @Override
    public PostInfoVO add(PostInfoAddDTO params) {
        ValidatorUtils.validateEntity(params);
        PostInfo entity=new PostInfo();
        if(StrUtil.isEmpty(params.getId())){
            entity.setId(IdUtil.getSnowflake(5,1).nextIdStr());
        }else {
            entity.setId(params.getId());
        }
        entity.setCompanyNo(params.getCompanyNo());
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        entity.setRowsStatus(YN.YES.getCode());
        postInfoService.save(entity);
        return PostInfoConvertHelper.getPostInfoVO(entity);
    }

    @Override
    public PostInfoVO edit(PostInfoEditDTO params) {
        ValidatorUtils.validateEntity(params);
        PostInfo entity=postInfoService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setName(params.getName());
        entity.setType(params.getType());
        entity.setSort(params.getSort());
        entity.setRemark(params.getRemark());
        postInfoService.updateById(entity);
        postInfoCacheService.removeById(entity.getId());
        return PostInfoConvertHelper.getPostInfoVO(entity);
    }

    @Override
    public PostInfoVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"岗位ID不能为空");
        }
        PostInfo entity=postInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        return PostInfoConvertHelper.getPostInfoVO(entity);
    }



    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"岗位ID不能为空");
        }
        PostInfo entity=postInfoService.getById(id);
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        //todo:

        entity.setRowsStatus(YN.NO.getCode());
        postInfoService.updateById(entity);
        postInfoCacheService.removeById(entity.getId());
    }




}
