package com.xxh.cloud.user.framework.modules.base.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.client.api.CompanyClientService;
import com.xxh.cloud.basic.client.api.bean.bos.CompanyBO;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.framework.modules.base.biz.RoleBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.Role;
import com.xxh.cloud.user.framework.modules.base.entity.dos.RoleMenu;
import com.xxh.cloud.user.framework.modules.base.entity.dos.RoleUser;
import com.xxh.cloud.user.framework.modules.base.entity.dto.*;
import com.xxh.cloud.user.framework.modules.base.entity.vos.RoleVO;
import com.xxh.cloud.user.framework.modules.base.helper.RoleConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.RoleMenuService;
import com.xxh.cloud.user.framework.modules.base.service.RoleService;
import com.xxh.cloud.user.framework.modules.base.service.RoleUserService;
import com.xxh.cloud.user.framework.modules.cache.service.RoleInfoCacheService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Log4j2
@Service
public class RoleBizServiceImpl implements RoleBizService {

    @Autowired
    private RoleService roleService;
    @Autowired
    private RoleUserService roleUserService;
    @Autowired
    private RoleMenuService roleMenuService;
    @Autowired
    private RoleInfoCacheService roleInfoCacheService;

    @Autowired
    private CompanyClientService companyClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;



    @Override
    public PageData<RoleVO> query(RoleQueryDTO params) {
        IPage<Role> page= roleService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<Role>()
                        .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getName()),"name",params.getName())
                        .orderByDesc("create_date")
        );

        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> companyNos=new HashSet<>();
        Set<String> opUserIds=new HashSet<>();

        page.getRecords().forEach(item->{
            if(StrUtil.isNotEmpty(item.getCompanyNo())){
                companyNos.add(item.getCompanyNo());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                opUserIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                opUserIds.add(item.getUpdater());
            }
        });

        Map<String, CompanyBO> companyMap= companyClientService.getCompanyMapByIds(companyNos);
        Map<String, AdminUserBO> opUserMap= adminUserClientService.getUserMapByIds(opUserIds);

        PageData<RoleVO> data= new PageConverter<Role, RoleVO>() {
            @Override
            public RoleVO converter(Role entity) {
                return RoleConvertHelper.getRoleVO(entity,companyMap,opUserMap);
            }
        }.getPageData(page);
        return data;
    }

    @Override
    public List<RoleVO> list(RoleListDTO params) {
        List<Role> list= roleService.list(
                new QueryWrapper<Role>()
                        .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                        .orderByDesc("create_date")
        );
        List<RoleVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(list)){
          return data;
        }

        Set<String> companyNos=new HashSet<>();
        Set<String> opUserIds=new HashSet<>();
        list.forEach(item->{
            if(StrUtil.isNotEmpty(item.getCompanyNo())){
                companyNos.add(item.getCompanyNo());
            }
            if(StrUtil.isNotEmpty(item.getCreator())){
                opUserIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                opUserIds.add(item.getUpdater());
            }
        });
        Map<String, CompanyBO> companyMap= companyClientService.getCompanyMapByIds(companyNos);
        Map<String, AdminUserBO> opUserMap= adminUserClientService.getUserMapByIds(opUserIds);

        list.forEach(item->{
            data.add(RoleConvertHelper.getRoleVO(item,companyMap,opUserMap));
        });
        return data;
    }

    @Override
    public Map<String, RoleVO> roleMap(RoleListDTO params) {
        List<RoleVO> list=this.list(params);
        Map<String, RoleVO> data=new HashMap<>();
        list.forEach(item->{
            data.put(item.getId(),item);
        });
        return data;
    }

    @Override
    public RoleVO add(RoleAddDTO params) {
        ValidatorUtils.validateEntity(params);
        Role entity=new Role();
        entity.setId(params.getId());
        entity.setCompanyNo(params.getCompanyNo());
        entity.setName(params.getName());
        entity.setRemark(params.getRemark());
        roleService.save(entity);
        return RoleConvertHelper.getRoleVO(entity, new ArrayList<>());
    }

    @Override
    public RoleVO update(RoleUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        Role entity=roleService.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        if(StrUtil.isNotEmpty(params.getName())){
            entity.setName(params.getName());
        }
        entity.setRemark(params.getRemark());
        roleService.updateById(entity);
        roleInfoCacheService.removeById(entity.getId());
        return RoleConvertHelper.getRoleVO(entity);
    }

    @Override
    public RoleVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Role entity=roleService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        Set<String> companyNos=new HashSet<>();
        Set<String> opUserIds=new HashSet<>();

        if(StrUtil.isNotEmpty(entity.getCompanyNo())){
            companyNos.add(entity.getCompanyNo());
        }

        if(StrUtil.isNotEmpty(entity.getCreator())){
            opUserIds.add(entity.getCreator());
        }

        if(StrUtil.isNotEmpty(entity.getUpdater())){
            opUserIds.add(entity.getUpdater());
        }
        Map<String, CompanyBO> companyMap= companyClientService.getCompanyMapByIds(companyNos);
        Map<String, AdminUserBO> opUserMap= adminUserClientService.getUserMapByIds(opUserIds);

        // 查询关联菜单
        List<RoleMenu> roleMenus = roleMenuService.list(new QueryWrapper<RoleMenu>().eq("role_id", id));
        return RoleConvertHelper.getRoleVO(entity,companyMap,opUserMap,roleMenus);
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        Role entity=roleService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        this.roleUserLogout(id);

        roleService.removeById(id);
        roleUserService.remove(new QueryWrapper<RoleUser>()
                .eq("role_id",id)
        );
        roleMenuService.remove(new QueryWrapper<RoleMenu>()
                .eq("role_id",id)
        );
        roleInfoCacheService.removeById(entity.getId());
    }


    @Override
    public void updateMenu(RoleUpdateMenuDTO params) {
        ValidatorUtils.validateEntity(params);

        ValidatorUtils.validateEntity(params);
        Role entity=roleService.getById(params.getId());
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        List<RoleMenu> saveList=new ArrayList<>();

        params.getMenuIds().forEach(menuId->{
            RoleMenu roleMenu=new RoleMenu();
            roleMenu.setId(IdUtil.getSnowflake(2,1).nextIdStr());
            roleMenu.setRoleId(params.getId());
            roleMenu.setMenuId(menuId);
            saveList.add(roleMenu);
        });

        roleMenuService.remove(new QueryWrapper<RoleMenu>()
                .eq("role_id",params.getId())
        );
        if(saveList.size()>0){
            roleMenuService.saveBatch(saveList);
        }
        this.roleUserLogout(params.getId());

    }

    private void roleUserLogout(String roleId){
        List<RoleUser> roleUserList= roleUserService.list(
                new QueryWrapper<RoleUser>()
                        .eq("role_id",roleId)
        );
        if(CollUtil.isNotEmpty(roleUserList)){
            for(RoleUser roleUser:roleUserList){
                AuthContextUtils.logout(roleUser.getUserId());
            }
        }

    }



}
