package com.xxh.cloud.user.framework.modules.base.biz.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Validator;
import cn.hutool.core.util.StrUtil;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.basic.client.api.CompanyClientService;
import com.xxh.cloud.basic.client.api.bean.bos.CompanyBO;
import com.xxh.cloud.framework.common.config.ProjectConfig;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.constant.DataScopeMode;
import com.xxh.cloud.framework.common.constant.UserRegLoginMode;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.mybatisplus.page.PageConverter;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.user.LoginUser;
import com.xxh.cloud.framework.common.utils.PasswordCheckUtils;
import com.xxh.cloud.framework.common.utils.ProjectUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.DeptClientService;
import com.xxh.cloud.user.client.api.PostInfoClientService;
import com.xxh.cloud.user.client.api.RoleInfoClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.client.api.bean.bos.DeptBO;
import com.xxh.cloud.user.client.api.bean.bos.PostInfoBO;
import com.xxh.cloud.user.client.api.bean.bos.RoleInfoBO;
import com.xxh.cloud.user.framework.modules.base.biz.RoleBizService;
import com.xxh.cloud.user.framework.modules.base.biz.UserInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dos.Dept;
import com.xxh.cloud.user.framework.modules.base.entity.dos.RoleUser;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.entity.dto.*;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.RoleVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.UserVO;
import com.xxh.cloud.user.framework.modules.base.helper.UserConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.RoleUserService;
import com.xxh.cloud.user.framework.modules.base.service.UserService;
import com.xxh.cloud.user.framework.modules.cache.keys.UserCacheKeyUtil;
import com.xxh.cloud.user.framework.modules.cache.service.UserCacheService;
import com.xxh.cloud.user.framework.modules.constant.UserStatus;
import com.xxh.cloud.user.framework.modules.constant.XXUserErrorEnum;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserUpdateBySelfDTO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserUpdatePasswordBySelfDTO;
import com.xxh.cloud.user.framework.modules.login.helper.UserLoginHelper;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Log4j2
@Service
public class UserInfoBizServiceImpl implements UserInfoBizService {

    @Autowired
    private UserService userService;

    @Autowired
    private RoleUserService roleUserService;

    @Autowired
    private ProjectConfig projectConfig;


    @Autowired
    private DeptClientService deptClientService;
    @Autowired
    private PostInfoClientService postInfoClientService;
    @Autowired
    private RoleInfoClientService roleInfoClientService;
    @Autowired
    private AdminUserClientService adminUserClientService;
    @Autowired
    private UserCacheService userCacheService;
    @Autowired
    private CompanyClientService companyClientService;


    @Override
    public PageData<UserVO> query(UserQueryDTO params) {
        IPage<User> page=userService.page(new Page<>(params.getPage(), params.getRows()),
                new QueryWrapper<User>()
                        .eq(StrUtil.isNotEmpty(params.getCompanyNo()),"company_no",params.getCompanyNo())
                        .ne(StrUtil.isNotEmpty(params.getNeCompanyNo()),"company_no",params.getNeCompanyNo())
                        .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                        .eq(StrUtil.isNotEmpty(params.getStatus()),"status",params.getStatus())
                        .eq(StrUtil.isNotEmpty(params.getDeptId()),"dept_id",params.getDeptId())

                        .or(StrUtil.isNotEmpty(params.getName()),
                                i->i.like("real_name",params.getName())
                                        .or().like("user_name",params.getName())
                                        .or().eq("mobile",params.getName())
                                        .or().eq("email",params.getName())
                        )
                        .eq(StrUtil.isNotEmpty(params.getSubjectCode()), "subject_code", params.getSubjectCode())
                        .eq(StrUtil.isNotEmpty(params.getSubjectRole()), "subject_role", params.getSubjectRole())
                        .eq("rows_status", YN.YES.getCode())
                        .orderByDesc("create_date")
        );



        if(page==null || CollUtil.isEmpty(page.getRecords())){
            return null;
        }

        Set<String> ids=new HashSet<>();
        Set<String> opUserIds=new HashSet<>();
        Set<String> deptIds=new HashSet<>();
        Set<String> postIs=new HashSet<>();
        Set<String> companyNos=new HashSet<>();
        Set<String> ruleIds=new HashSet<>();
        Map<String,Set<String>> userRoleMap=new HashMap<>();

        page.getRecords().forEach(item->{
            ids.add(item.getId());

            if(StrUtil.isNotEmpty(item.getCompanyNo())){
                companyNos.add(item.getCompanyNo());
            }

            if(StrUtil.isNotEmpty(item.getDeptId())){
                deptIds.add(item.getDeptId());
            }

            if(StrUtil.isNotEmpty(item.getCreator())){
                opUserIds.add(item.getCreator());
            }
            if(StrUtil.isNotEmpty(item.getUpdater())){
                opUserIds.add(item.getUpdater());
            }

            if(StrUtil.isNotEmpty(item.getPostIds())){
                Set<String> postSet= StrJoinSplitUtils.split2Set(item.getPostIds());
                postIs.addAll(postSet);
            }

        });


        List<RoleUser> roleUsers = roleUserService.list(new QueryWrapper<RoleUser>().in("user_id",ids));
        if(CollUtil.isNotEmpty(roleUsers)){
            roleUsers.forEach(item->{
                Set<String> userRoleSet= userRoleMap.get(item.getUserId());
                if(userRoleSet==null){
                    userRoleSet=new HashSet<>();
                    userRoleMap.put(item.getUserId(),userRoleSet);
                }
                userRoleSet.add(item.getRoleId());
                ruleIds.add(item.getRoleId());
            });
        }

        Map<String, DeptBO> deptMap= deptClientService.getDeptMapByIds(deptIds);
        Map<String, PostInfoBO> postMap= postInfoClientService.getPostMapByIds(postIs);
        Map<String, AdminUserBO> opUserMap= adminUserClientService.getUserMapByIds(opUserIds);
        Map<String, CompanyBO> companyMap= companyClientService.getCompanyMapByIds(companyNos);
        Map<String, RoleInfoBO> roleMap= roleInfoClientService.getRoleMapByIds(ruleIds);



        PageData<UserVO> data= new PageConverter<User, UserVO>() {
            @Override
            public UserVO converter(User user) {
                return UserConvertHelper.getUserVO(user,params.isDesensitize(),companyMap,deptMap,postMap,opUserMap,roleMap,userRoleMap);
            }
        }.getPageData(page);
        return data;
    }

    private void userCheck(UserAddDTO params) throws BusinessException {
        boolean isSysCom= ProjectUtils.isSysCompany(params.getCompanyNo());
        if(isSysCom){
            this.userNameCheck(params, DataScopeMode.CompanyDataScope);
        }else {
            UserRegLoginMode regMode= projectConfig.getComUserMgrRegModeEnum();
            if(regMode== UserRegLoginMode.NoneLoginRegAdd){
                throw new BusinessException(XXUserErrorEnum.Not_Add_User);
            }
            if(regMode== UserRegLoginMode.UserNamePassword){
                this.userNameCheck(params,projectConfig.getComUserDataScopeModeEnum());
            }
            if(regMode== UserRegLoginMode.MobileVcode){
                this.mobileCheck(params,projectConfig.getComUserDataScopeModeEnum());
            }
        }
    }

    /**
     * 用户名校验
     * */
    private void userNameCheck(UserAddDTO params , DataScopeMode dataScope){
        if(StrUtil.isEmpty(params.getUserName())){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"用户名不能为空");
        }
        if(StrUtil.isEmpty(params.getPassWord())){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"密码不能为空");
        }
        if(DataScopeMode.CompanyDataScope==dataScope){
            long num =  userService.count(new QueryWrapper<User>()
                    .eq("company_no",params.getCompanyNo())
                    .eq("user_name",params.getUserName())
            );
            if(num>0){
                throw new BusinessException(XXUserErrorEnum.UserName_Exists);
            }
        }
        if(DataScopeMode.TenantDataScope==dataScope){
            long num =  userService.count(new QueryWrapper<User>()
                    .ne("company_no", Constant.SysCompanyNo)
                    .eq("user_name",params.getUserName())
            );
            if(num>0){
                throw new BusinessException(XXUserErrorEnum.UserName_Exists);
            }
        }
    }

    private void mobileCheck(UserAddDTO params , DataScopeMode dataScope){
        if(StrUtil.isEmpty(params.getMobile())){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"手机号不能为空");
        }
        if(!Validator.isMobile(params.getMobile())){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"手机号格式错误");
        }
        if(DataScopeMode.CompanyDataScope==dataScope){
            long num =  userService.count(new QueryWrapper<User>()
                    .eq("company_no",params.getCompanyNo())
                    .eq("mobile",params.getMobile())
            );
            if(num>0){
                throw new BusinessException(XXUserErrorEnum.UserName_Exists);
            }
        }
        if(DataScopeMode.TenantDataScope==dataScope){
            long num =  userService.count(new QueryWrapper<User>()
                    .ne("company_no", Constant.SysCompanyNo)
                    .eq("mobile",params.getMobile())
            );
            if(num>0){
                throw new BusinessException(XXUserErrorEnum.UserName_Exists);
            }
        }

        params.setUserName(params.getMobile());
        if(StrUtil.isEmpty(params.getPassWord())){
            params.setPassWord("12345678");
        }
//        params.setEmail(null);
//        params.setMobileLiaison(params.getMobile());
    }




    @Transactional(rollbackFor = Exception.class)
    @Override
    public UserVO add(UserAddDTO params) {
        this.userCheck(params);
        User entity= UserConvertHelper.getUserEntity_Add(params);
        userService.save(entity);
        updateRole(entity, params.getRoleIds());  // 更新用户角色
        return UserConvertHelper.getUserVO(entity,false);
    }

    @Override
    public UserVO update(UserUpdateBySelfDTO params, LoginUser loginUser) {
        ValidatorUtils.validateEntity(params);
        User entity= userService.getById(loginUser.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRealName(params.getRealName());
        entity.setHeadUrl(params.getHeadUrl());
        entity.setGender(params.getGender());
        userService.updateById(entity);
        userCacheService.removeById(entity.getId());
        return UserConvertHelper.getUserVO(entity,false);
    }


    @Override
    public UserVO update(UserUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        User entity= userService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        UserConvertHelper.getUserEntity_Edit(entity,params);
        userService.updateById(entity);

        // 更新用户角色
        updateRole(entity, params.getRoleIds());
        userCacheService.removeById(entity.getId());
        AuthContextUtils.logout(entity.getId());

        return UserConvertHelper.getUserVO(entity,false);
    }

    @Override
    public UserVO updateStatus(UserUpdateStatusDTO params) {
        ValidatorUtils.validateEntity(params);
        User entity= userService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setStatus(params.getStatus());
        userService.updateById(entity);
        userCacheService.removeById(entity.getId());
        AuthContextUtils.logout(entity.getId());

        return UserConvertHelper.getUserVO(entity,false);
    }

    @Override
    public UserVO changePassword(UserUpdatePasswordBySelfDTO params, LoginUser loginUser) {
        ValidatorUtils.validateEntity(params);
        User entity= userService.getById(loginUser.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        // 校验旧密码
        UserLoginHelper.checkUserPassword(entity.getPassWord(),params.getOldPassWord());
        entity.setPassWord(UserLoginHelper.getPasswordMd5(params.getNewPassWord()));
        userService.updateById(entity);
        return UserConvertHelper.getUserVO(entity,false);
    }

    @Override
    public UserVO updatePassword(UserUpdatePasswordDTO params) {
        ValidatorUtils.validateEntity(params);
        User entity= userService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setPassWord(UserLoginHelper.getPasswordMd5(params.getPassWord()));
        userService.updateById(entity);
        return UserConvertHelper.getUserVO(entity,false);
    }

    @Override
    public void updateRole(UserUpdateRoleDTO params) {
        ValidatorUtils.validateEntity(params);
        User entity= userService.getById(params.getId());
        if(entity==null || YN.NO.getCode().equals(entity.getRowsStatus())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        this.updateRole(entity, params.getRoleIds());
        userCacheService.removeById(entity.getId());
        AuthContextUtils.logout(entity.getId());
    }

    @Override
    public void delById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        User entity=userService.getById(id);
        if(entity==null){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }
        entity.setRowsStatus(YN.NO.getCode());
        userService.updateById(entity);
        userCacheService.removeById(entity.getId());
        AuthContextUtils.logout(entity.getId());
    }

    @Override
    public UserVO infoById(String id) {
        if(StrUtil.isEmpty(id)){
            throw new BusinessException(BusinessBaseErrorEnum.PARAMS_GET_ERROR,"ID不能为空");
        }
        User entity=userService.getById(id);
        if(entity==null || entity.getRowsStatus().equals(YN.NO.getCode())){
            throw new BusinessException(BusinessBaseErrorEnum.DB_RECORD_NOT_EXISTS);
        }

        Set<String> opUserIds=new HashSet<>();
        Set<String> deptIds=new HashSet<>();
        Set<String> postIs=new HashSet<>();
        Set<String> companyNos=new HashSet<>();

        Set<String> ruleIds=new HashSet<>();
        Map<String,Set<String>> userRoleMap=new HashMap<>();



        if(StrUtil.isNotEmpty(entity.getCompanyNo())){
            companyNos.add(entity.getCompanyNo());
        }

        if(StrUtil.isNotEmpty(entity.getDeptId())){
            deptIds.add(entity.getDeptId());
        }

        if(StrUtil.isNotEmpty(entity.getCreator())){
            opUserIds.add(entity.getCreator());
        }
        if(StrUtil.isNotEmpty(entity.getUpdater())){
            opUserIds.add(entity.getUpdater());
        }
        if(StrUtil.isNotEmpty(entity.getPostIds())){
            Set<String> postSet= StrJoinSplitUtils.split2Set(entity.getPostIds());
            postIs.addAll(postSet);
        }

        List<RoleUser> roleUsers = roleUserService.list(new QueryWrapper<RoleUser>().eq("user_id",entity.getId()));
        if(CollUtil.isNotEmpty(roleUsers)){
            roleUsers.forEach(item->{
                Set<String> userRoleSet= userRoleMap.get(item.getUserId());
                if(userRoleSet==null){
                    userRoleSet=new HashSet<>();
                    userRoleMap.put(item.getUserId(),userRoleSet);
                }
                userRoleSet.add(item.getRoleId());
                ruleIds.add(item.getRoleId());
            });
        }

        Map<String, DeptBO> deptMap= deptClientService.getDeptMapByIds(deptIds);
        Map<String, PostInfoBO> postMap= postInfoClientService.getPostMapByIds(postIs);
        Map<String, AdminUserBO> opUserMap= adminUserClientService.getUserMapByIds(opUserIds);
        Map<String, CompanyBO> companyMap= companyClientService.getCompanyMapByIds(companyNos);
        Map<String, RoleInfoBO> roleMap= roleInfoClientService.getRoleMapByIds(ruleIds);

        UserVO data= UserConvertHelper.getUserVO(entity,false,companyMap,deptMap,postMap,opUserMap,roleMap,userRoleMap);
        return data;
    }


    public void updateRole(User entity, Set<String> roleIds) {
        if(CollUtil.isNotEmpty(roleIds)){
            List<RoleUser> roleUserList=new ArrayList<>();
            roleIds.forEach(roleId->{
                roleUserList.add(UserConvertHelper.getRoleUserEntity(entity, roleId));
            });
            roleUserService.remove(
                    new QueryWrapper<RoleUser>()
                            .eq("user_id",entity.getId())
            );
            roleUserService.saveBatch(roleUserList);
        }
    }

    // 同步医生
    @Transactional
    @Override
    public void addDoctorBatch(List<UserAddDTO> params) {
        log.info("1.同步医生>>>>>> {}", params.size());
        if (CollectionUtil.isEmpty(params)) {
            return;
        }
        params.forEach(i -> ValidatorUtils.validateEntity(i));

        // 查询要保存的数据是否存在
        Set<String> userIds = new HashSet<>();
        params.forEach(i -> {
            userIds.add(i.getId());
        });
        List<User> existList = userService.list(new QueryWrapper<User>()
                .in("id", userIds)
        );
        log.info("2.同步时要更新的医生数量>>>>>> {}", existList.size());
        Map<String, User> existMap = existList.stream().collect(Collectors.toMap(User::getId, i -> i));
        
        List<User> addList = new ArrayList<>(); // 待新增的信息
        List<User> updateList = new ArrayList<>(); // 待更新的信息
        params.forEach(i -> {
            User vo = existMap.get(i.getId());
            if (vo == null) {
                User entity = new User();
                entity.setId(i.getId());
                entity.setCompanyNo(i.getCompanyNo());
                entity.setUserName(i.getUserName()); // 账号用医生编号
                entity.setRealName(i.getRealName());
                entity.setDeptId(i.getDeptId());
                entity.setIsSuperAdmin(i.getIsSuperAdmin());
                entity.setStatus(i.getStatus());
                entity.setPassWord(UserLoginHelper.getPasswordMd5(i.getId() + "Aa@")); // 生成默认账号密码 // 密码用医生编号 + Abc
                entity.setRowsStatus(YN.YES.getCode());
                entity.setHeadUrl(i.getHeadUrl());
                entity.setPersonInfo(null == i.getPersonInfo() ? null : JSON.toJSONString(i.getPersonInfo()));
                entity.setUserShanc(i.getUserShanc());
                entity.setJobTitle(i.getJobTitle());
                entity.setJobPosition(i.getJobPosition());
                entity.setMobile(i.getMobile());
                entity.setEmpCode(i.getEmpCode());
                entity.setOaEmpCode(i.getOaEmpCode());
                addList.add(entity);
            } else {
                User entity = new User();
                entity.setId(i.getId());
                entity.setCompanyNo(i.getCompanyNo());
                // 更新部门号,头像，简介，职位职称
                // entity.setDeptId(i.getDeptId()); // 不要更新部门id
                entity.setHeadUrl(i.getHeadUrl());
                entity.setPersonInfo(null == i.getPersonInfo() ? null : JSON.toJSONString(i.getPersonInfo()));
                entity.setUserShanc(i.getUserShanc());
                entity.setJobTitle(i.getJobTitle());
                entity.setJobPosition(i.getJobPosition());
                entity.setMobile(i.getMobile());
                entity.setEmpCode(i.getEmpCode());
                entity.setOaEmpCode(i.getOaEmpCode());
                updateList.add(entity);
            }
        });

        log.info("3.新增的医生数量>>>>>> {}", addList.size());
        if (CollectionUtil.isNotEmpty(addList)) {
            userService.saveOrUpdateBatch(addList);
        }
        log.info("4.更新的医生数量>>>>>> {}", updateList.size());
        if (CollectionUtil.isNotEmpty(updateList)) {
            userService.updateBatchById(updateList);
        }
    }

    @Override
    public List<UserVO> simpleList(UserQueryDTO params) {
        List<User> list = userService.list(new QueryWrapper<User>()
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .eq(StrUtil.isNotEmpty(params.getDeptId()),"dept_id",params.getDeptId())
                .in(CollUtil.isNotEmpty(params.getDeptIds()),"dept_id",params.getDeptIds())
                .eq("rows_status", YN.YES.getCode())
                .orderByDesc("create_date")
        );

        if(CollUtil.isEmpty(list)){
            return new ArrayList<>(0);
        }

        return list.stream().map(user -> UserConvertHelper.getUserVO(user,true)).collect(Collectors.toList()); // 数据脱敏
    }

    @Override
    public UserVO simpleOne(UserInfoDTO params) {
        User user = userService.getOne(new QueryWrapper<User>()
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .eq("rows_status", YN.YES.getCode())
        );
        if(null == user){
            return null;
        }
        return UserConvertHelper.getUserVO(user, false); // 数据不脱敏
    }



}
