package com.xxh.cloud.user.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.entity.dto.UserQueryDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.UserVO;
import com.xxh.cloud.user.framework.modules.base.helper.UserConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.UserService;
import com.xxh.cloud.user.framework.modules.cache.service.UserCacheService;
import com.xxh.cloud.user.framework.modules.client.helper.AdminUserClientConvertHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Log4j2
@Service
public class AdminUserClientServiceImpl implements AdminUserClientService {

    @Autowired
    private UserCacheService userCacheService;
    @Autowired
    private UserService userService;


    @Override
    public AdminUserBO getUserById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        UserVO user = userCacheService.getById(id);
        return AdminUserClientConvertHelper.getAdminUserBO(user);
    }

    @Override
    public AdminUserBO getUserByUserName(String userName) {
        List<User> users = userService.list(new QueryWrapper<User>()
                .eq("user_name", userName)
                .eq("rows_status", YN.YES.getCode())
                .orderByDesc("create_date"));

        if(CollUtil.isEmpty(users)){
            return null;
        }
        return AdminUserClientConvertHelper.getAdminUserBO
                (UserConvertHelper.getUserVO(users.get(0),false));
    }

    @Override
    public List<AdminUserBO> getUserListByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        List<UserVO> list = userCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(AdminUserClientConvertHelper::getAdminUserBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, AdminUserBO> getUserMapByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new HashMap<>();
        }
        List<UserVO> list = userCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        Map<String, AdminUserBO> adminUserBOMap = new HashMap<>();
        list.forEach(i -> adminUserBOMap.put(i.getId(), AdminUserClientConvertHelper.getAdminUserBO(i)));
        return adminUserBOMap;
    }
    
    @Override
    public List<AdminUserBO> getUserListByDeptId(String deptId) {
        if(StrUtil.isBlank(deptId)){
            return new ArrayList<>();
        }

        UserQueryDTO userQueryDTO = new UserQueryDTO();
        userQueryDTO.setDeptId(deptId);
        List<UserVO> list = this.simpleList(userQueryDTO);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(AdminUserClientConvertHelper::getAdminUserBO).collect(Collectors.toList());
    }

    @Override
    public List<AdminUserBO> getUserListByDeptIds(List<String> deptIds) {
        if(CollUtil.isEmpty(deptIds)){
            return new ArrayList<>();
        }

        UserQueryDTO userQueryDTO = new UserQueryDTO();
        userQueryDTO.setDeptIds(deptIds);
        List<UserVO> list = this.simpleList(userQueryDTO);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(AdminUserClientConvertHelper::getAdminUserBO).collect(Collectors.toList());
    }

    private List<UserVO> simpleList(UserQueryDTO params) {
        List<User> list = userService.list(new QueryWrapper<User>()
                .eq(StrUtil.isNotEmpty(params.getId()),"id",params.getId())
                .eq(StrUtil.isNotEmpty(params.getDeptId()),"dept_id",params.getDeptId())
                .in(CollUtil.isNotEmpty(params.getDeptIds()),"dept_id",params.getDeptIds())
                .eq("rows_status", YN.YES.getCode())
                .orderByDesc("create_date")
        );

        if(CollUtil.isEmpty(list)){
            return new ArrayList<>(0);
        }

        return list.stream().map(user -> UserConvertHelper.getUserVO(user,true)).collect(Collectors.toList()); // 数据脱敏
    }

}
