package com.xxh.cloud.user.framework.modules.client.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.user.client.api.RoleInfoClientService;
import com.xxh.cloud.user.client.api.bean.bos.RoleInfoBO;
import com.xxh.cloud.user.framework.modules.base.biz.RoleBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.RoleListDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.PostInfoVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.RoleVO;
import com.xxh.cloud.user.framework.modules.cache.service.RoleInfoCacheService;
import com.xxh.cloud.user.framework.modules.client.helper.DeptClientConvertHelper;
import com.xxh.cloud.user.framework.modules.client.helper.PostInfoClientConvertHelper;
import com.xxh.cloud.user.framework.modules.client.helper.RoleInfoClientConvertHelper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Log4j2
@Service
public class RoleInfoClientServiceImpl implements RoleInfoClientService {


    @Autowired
    private RoleInfoCacheService roleInfoCacheService;

    @Autowired
    private RoleBizService roleBizService;


    @Override
    public RoleInfoBO getRoleById(String id) {
        if(StrUtil.isEmpty(id)){
            return null;
        }
        RoleVO role = roleInfoCacheService.getById(id);
        return RoleInfoClientConvertHelper.getRoleInfoBO(role);
    }

    @Override
    public List<RoleInfoBO> getRoleListByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new ArrayList<>();
        }
        List<RoleVO> list = roleInfoCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(RoleInfoClientConvertHelper::getRoleInfoBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, RoleInfoBO> getRoleMapByIds(Collection<String> ids) {
        if(CollUtil.isEmpty(ids)){
            return new HashMap<>();
        }
        List<RoleVO> list = roleInfoCacheService.getListByIds(ids);
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        return new HashMap<>(list.stream().collect(Collectors.toMap(RoleVO::getId, RoleInfoClientConvertHelper::getRoleInfoBO)));
    }

    @Override
    public List<RoleInfoBO> getRoleListByCompany(String companyNo) {
        if(StrUtil.isEmpty(companyNo)){
            return new ArrayList<>();
        }
        List<RoleVO> list = roleBizService.list(new RoleListDTO(companyNo));
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(RoleInfoClientConvertHelper::getRoleInfoBO).collect(Collectors.toList());
    }

    @Override
    public Map<String, RoleInfoBO> getRoleMapByCompany(String companyNo) {
        if(StrUtil.isEmpty(companyNo)){
            return new HashMap<>();
        }
        List<RoleVO> list = roleBizService.list(new RoleListDTO(companyNo));
        if (CollUtil.isEmpty(list)) {
            return new HashMap<>();
        }
        return new HashMap<>(list.stream().collect(Collectors.toMap(RoleVO::getId, RoleInfoClientConvertHelper::getRoleInfoBO)));
    }
}
