package com.xxh.cloud.user.framework.modules.login.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.basic.client.api.ParamsClientService;
import com.xxh.cloud.framework.common.config.ProjectConfig;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.constant.DataScopeMode;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.third.framework.modules.thirdHis.WbmdOaApiHelper;
import com.xxh.cloud.third.framework.modules.thirdHis.model.HealthyFrontendConfig;
import com.xxh.cloud.third.framework.modules.thirdHis.model.RespBaseThird;
import com.xxh.cloud.third.framework.modules.thirdHis.model.WbmdOaModel;
import com.xxh.cloud.third.framework.modules.thirdHis.model.WbmdOaResult;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.service.UserService;
import com.xxh.cloud.user.framework.modules.constant.UserStatus;
import com.xxh.cloud.user.framework.modules.constant.XXUserErrorEnum;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserPasswordLoginDTO;
import com.xxh.cloud.user.framework.modules.login.entity.vos.UserLoginVO;
import com.xxh.cloud.user.framework.modules.login.helper.UserLoginHelper;
import com.xxh.cloud.user.framework.modules.login.service.UserLoginService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UserLoginServiceImpl implements UserLoginService {

    @Autowired
    private UserService userService;
    @Autowired
    private ProjectConfig projectConfig;
    @Autowired
    private ParamsClientService paramsClientService;


    private User getUserById(String id){
        User user= userService.getById(id);
        if(user==null){
            throw new BusinessException(XXUserErrorEnum.User_Not_Exists);
        }
        return user;
    }


    /**
     * 商户唯一
     * */
    private User getUserByUserName(String companyNo, String userName){
        if(StrUtil.isEmpty(companyNo)){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL,"机构编号不能为空");
        }
        User user= userService.getOne(new QueryWrapper<User>()
                .eq("rows_status", YN.YES.getCode())
                .eq("company_no",companyNo)
                .eq("user_name",userName)
                .eq("status", UserStatus.US_Normal.getCode())
        );
        if(user==null){
            throw new BusinessException(XXUserErrorEnum.User_Not_Exists);
        }
        return user;
    }

    /**
     * 平台唯一
     * */
    private User getUserByUserName(String userName){
        User user= userService.getOne(new QueryWrapper<User>()
                .eq("rows_status", YN.YES.getCode())
                .eq("user_name",userName)
                .eq("status", UserStatus.US_Normal.getCode())
        );
        if(user==null){
            throw new BusinessException(XXUserErrorEnum.User_Not_Exists);
        }
        return user;
    }


    @Override
    public UserLoginVO loginByCom(UserPasswordLoginDTO params) {
        ValidatorUtils.validateEntity(params);
        User user=null;
        if(projectConfig.getComUserDataScopeModeEnum()== DataScopeMode.TenantDataScope){
            user=this.getUserByUserName(params.getUserName());
        }
        if(projectConfig.getComUserDataScopeModeEnum()== DataScopeMode.CompanyDataScope){
            user=this.getUserByUserName(params.getCompanyNo(), params.getUserName());
        }
        if(user==null){
            throw new BusinessException(XXUserErrorEnum.User_Not_Exists);
        }
        UserLoginHelper.checkUserPassword(user.getPassWord(),params.getPassword());

        // 记录日志
//        logLoginService.saveLog(new LogLoginSaveDTO(LogLoginOperation.Login.getCode(), LogLoginStatus.SUCCESS.getCode(),
//                user.getId(), user.getUserName()));

        return UserLoginHelper.getUserLoginVO(user,false);
    }

    @Override
    public UserLoginVO loginBySys(UserPasswordLoginDTO params) {
        User user=this.getUserByUserName(params.getUserName());
        UserLoginHelper.checkUserPassword(user.getPassWord(),params.getPassword());

        // 记录日志
//        logLoginService.saveLog(new LogLoginSaveDTO(LogLoginOperation.Login.getCode(), LogLoginStatus.SUCCESS.getCode(),
//                user.getId(), user.getUserName()));


        return UserLoginHelper.getUserLoginVO(user,false);
    }

    @Override
    public UserLoginVO loginByWbmdOa(String code) {
        WbmdOaModel model = WbmdOaApiHelper.wecomAuthorUrl(code);
        if (null == model || !"200".equals(model.getCode())) {
            return null;
        }
        User user = null;
        // 根据oa工号查
        String account = model.getResult().getAccount();
        if (StrUtil.isNotEmpty(account)) {
            user= userService.getOne(new QueryWrapper<User>()
                    .eq("rows_status", YN.YES.getCode())
                    .and(i->i.eq("user_name", account).or().eq("emp_code", account).or().eq("oa_emp_code", account))
                    .eq("status", UserStatus.US_Normal.getCode()));

        }
        if (null == user) {
            // 根据his工号查
            String accountHis = model.getResult().getAccountHis();
            user= userService.getOne(new QueryWrapper<User>()
                    .eq("rows_status", YN.YES.getCode())
                    .and(i->i.eq("user_name", accountHis).or().eq("emp_code", accountHis).or().eq("oa_emp_code", accountHis))
                    .eq("status", UserStatus.US_Normal.getCode()));
        }
        if (null == user) {
            throw new BusinessException(XXUserErrorEnum.User_Not_Exists);
        }
        return UserLoginHelper.getUserLoginVO(user,false);
    }
}
