package com.xxh.cloud.user.framework.modules.login.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.entity.dos.ThirdUser;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.service.ThirdUserService;
import com.xxh.cloud.user.framework.modules.base.service.UserService;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserBindThirdDTO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserLoginThirdDTO;
import com.xxh.cloud.user.framework.modules.login.entity.dto.UserPasswordLoginDTO;
import com.xxh.cloud.user.framework.modules.login.entity.vos.UserLoginVO;
import com.xxh.cloud.user.framework.modules.login.helper.UserLoginHelper;
import com.xxh.cloud.user.framework.modules.login.service.UserLoginService;
import com.xxh.cloud.user.framework.modules.login.service.UserLoginThirdService;

import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Log4j2
@Service
public class UserLoginThirdServiceImpl implements UserLoginThirdService {

    @Autowired
    private ThirdUserService thirdUserService;
    @Autowired
    private UserService userService;
    @Autowired
    private UserLoginService userLoginService;

    @Override
    public UserLoginVO login(UserLoginThirdDTO params) {

        ThirdUser thirdUser= thirdUserService.getOne(
                new QueryWrapper<ThirdUser>()
                        .eq("params_id",params.getParamsId())
                        .eq("open_id",params.getOpenId())
        );

        if(thirdUser==null){
            thirdUser= UserLoginHelper.getThirdUserInfo(params);
            thirdUserService.save(thirdUser);
            return UserLoginHelper.getUserLoginVO(thirdUser);
        } 
        // else if (StrUtil.isNotEmpty(params.getUnionId())) {
        //     // 更新thirdUser中的unionid
        //     thirdUser.setUnionId(params.getUnionId());
        //     thirdUserService.updateById(thirdUser);
        // }

        if("0".equals(thirdUser.getUserId())){
            return UserLoginHelper.getUserLoginVO(thirdUser);
        }

        User user= userService.getById(thirdUser.getUserId());
        
        if(!"normal".equals(user.getStatus())){
            throw new BusinessException("用户冻结");
        }
        return UserLoginHelper.getUserLoginVO(thirdUser,user);
    }

    @Override
    public UserLoginVO bind(String thirdUserId, UserBindThirdDTO params) {
        ValidatorUtils.validateEntity(params);
        ThirdUser thirdUser = thirdUserService.getById(thirdUserId);
        if(thirdUser==null){
            throw new BusinessException("三方用户不存在");
        }
        if(!"0".equals(thirdUser.getUserId())){
            throw new BusinessException("三方用户已绑定");
        }

        UserPasswordLoginDTO loginDTO = new UserPasswordLoginDTO();
        loginDTO.setUserName(params.getUserName());
        loginDTO.setPassword(params.getPassword());
        UserLoginVO loginUser = userLoginService.loginBySys(loginDTO);
        if (null == loginUser) {
            throw new BusinessException("绑定用户不存在");
        }

        // 判断绑定用户是否已被绑定
        Long count = thirdUserService.count(new QueryWrapper<ThirdUser>()
            .eq("user_id", loginUser.getId())
        );
        if (null != count && count > 0) {
            throw new BusinessException("用户已被绑定");
        }

        // 绑定
        thirdUser.setUserId(loginUser.getId());
        thirdUserService.updateById(thirdUser);

        loginUser.setThirdId(thirdUser.getId());
        loginUser.setThirdParamsId(thirdUser.getParamsId());
        loginUser.setThirdOpenId(thirdUser.getOpenId());
        loginUser.setThirdUnionId(thirdUser.getUnionId());
        loginUser.setId(thirdUser.getUserId());
        loginUser.setCompanyNo(thirdUser.getCompanyNo());
        return loginUser;
    }


}
