package com.xxh.cloud.user.framework.modules.login.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.utils.StrJoinSplitUtils;
import com.xxh.cloud.user.framework.modules.base.entity.dos.Menu;
import com.xxh.cloud.user.framework.modules.base.entity.dos.User;
import com.xxh.cloud.user.framework.modules.base.entity.dto.UserMenuSearchDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.MenuVO;
import com.xxh.cloud.user.framework.modules.base.helper.MenuConvertHelper;
import com.xxh.cloud.user.framework.modules.base.service.MenuService;
import com.xxh.cloud.user.framework.modules.base.service.UserService;
import com.xxh.cloud.user.framework.modules.cache.service.RoleMenuCacheService;
import com.xxh.cloud.user.framework.modules.constant.MenuType;
import com.xxh.cloud.user.framework.modules.login.mapper.UserPermissionQueryMapper;
import com.xxh.cloud.user.framework.modules.login.service.UserPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class UserPermissionServiceImpl implements UserPermissionService {

    @Autowired
    private UserPermissionQueryMapper userPermissionQueryMapper;
    @Autowired
    private UserService userService;
    @Autowired
    private MenuService menuService;
    @Autowired
    private RoleMenuCacheService roleMenuCacheService;

    @Override
    public Set<String> getPermissionListByUser(String userId) {
        Set<String> data=new HashSet<>();
        if(StrUtil.isEmpty(userId)){
            return data;
        }
        User user = userService.getById(userId);
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            // data.add("sys:*");
            data.add("*"); // 超级管理员拥有所有权限
            return data;
        }
        List<Menu> menuList= userPermissionQueryMapper.menuListByUserId(new QueryWrapper<Menu>()
                .eq("b.user_id", user.getId())
                .orderByDesc("sort")
        );
        if(CollUtil.isEmpty(menuList)){
            return data;
        }
        for(Menu menu:menuList){
            Set<String> permissions= StrJoinSplitUtils.split2Set(menu.getPermissions());
            data.addAll(permissions);
        }
        return data;
    }

    @Override
    public List<MenuVO> getMenuTreeByUser(String userId) {
        List<MenuVO> data =new ArrayList<>();

        if(StrUtil.isEmpty(userId)){
            return data;
        }

        List<MenuVO> returnData = roleMenuCacheService.getCacheMenuTree(userId);
        if (CollUtil.isNotEmpty(returnData)) {
            return returnData;
        }

        List<Menu> menuList ;
        User user = userService.getById(userId);

            if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
                menuList=menuService.list(
                        new QueryWrapper<Menu>()
                                .in("type", MenuType.MenuDir.getCode(), MenuType.Menu.getCode()));
            } else {
                menuList= userPermissionQueryMapper.menuListByUserId(new QueryWrapper<Menu>()
                        .eq("b.user_id", user.getId())
                        .in("c.type", MenuType.MenuDir.getCode(), MenuType.Menu.getCode())
                        .orderByDesc("sort"));
            }

            if(CollUtil.isEmpty(menuList)){
                return data;
            }

            List<MenuVO> dataList=new ArrayList<>();
            menuList.forEach(item->{
                dataList.add(MenuConvertHelper.getMenuVO(item,null));
            });

            data= TreeUtils.build(dataList);
            roleMenuCacheService.cacheMenuTree(userId, data);

        return data;
    }

    @Override
    public List<MenuVO> getMenuList(String userId, UserMenuSearchDTO params) {
        List<Menu> menuList = new ArrayList<>();
        if(StrUtil.isEmpty(userId)){
            return new ArrayList<>();
        }
        User user = userService.getById(userId);
        if (YN.YES.getCode().equals(user.getIsSuperAdmin())) {
            menuList.addAll(menuService.list(new QueryWrapper<Menu>()
                    .eq(StrUtil.isNotEmpty(params.getType()),"type",params.getType())
                    .eq(StrUtil.isNotEmpty(params.getPid()),"pid",params.getType())
            ));
        } else {
            menuList.addAll(userPermissionQueryMapper.menuListByUserId(new QueryWrapper<Menu>()
                    .eq("b.user_id", user.getId())
                    .eq(StrUtil.isNotEmpty(params.getType()),"c.type",params.getType())
                    .eq(StrUtil.isNotEmpty(params.getPid()),"c.pid",params.getType())
                    .orderByDesc("sort")));
        }
        if(CollUtil.isEmpty(menuList)){
            return new ArrayList<>();
        }
        return menuList.stream().map(MenuConvertHelper::getMenuVO).collect(Collectors.toList());
    }
}
