package com.xxh.cloud.user.framework.modules.mq.consumer;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.xxh.cloud.common.mq.bean.MQDoctorBean;
import com.xxh.cloud.common.mq.bean.MQWebDoctorBean;
import com.xxh.cloud.common.mq.constant.HMQQueueName;
import com.xxh.cloud.user.framework.modules.base.biz.UserInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.UserAddDTO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * 医生数据消费者
 * */
@Log4j2
@Component
@ConditionalOnProperty(name = "consumer.doctor.enable", havingValue = "true")
public class DoctorConsumer {

    @Autowired
    private UserInfoBizService userInfoBizService;

    @JmsListener(destination = HMQQueueName.Que_Doctor)
    public void queueDoctorData(String message) {
        log.info("1.医生消费者消费消息开始>>>>>> message={}", message);

        if (StrUtil.isBlank(message)) {
            return;
        }
        List<MQDoctorBean> mqMessageList = JSONArray.parseArray(message, MQDoctorBean.class);

        if (CollectionUtil.isEmpty(mqMessageList)) {
            return;
        }

        List<UserAddDTO> list = new ArrayList<>();
        mqMessageList.forEach(i -> {
            UserAddDTO userAddDTO = new UserAddDTO();
            userAddDTO.setCompanyNo("sys");
            userAddDTO.setId(i.getEmpCode());
            userAddDTO.setDeptId(i.getDeptId());
            userAddDTO.setUserName(i.getEmpCode());
            userAddDTO.setRealName(i.getEmpName());
            userAddDTO.setMobile(i.getPhoneNumber());
            userAddDTO.setEmpCode(i.getEmpCode());
            userAddDTO.setOaEmpCode(i.getOaEmpCode());
            list.add(userAddDTO);
        });


        log.info("2.医生消息信息保存>>>>>> {}", JSON.toJSONString(list));
        userInfoBizService.addDoctorBatch(list);

        log.info("3.医生消费者消费消息结束<<<<<<");
    }


    @JmsListener(destination = HMQQueueName.Que_WebDoctor)
    public void queueWebDoctorData(String message) {
        log.info("1.医生消费者消费消息开始>>>>>> message={}", message);

        if (StrUtil.isBlank(message)) {
            return;
        }
        List<MQWebDoctorBean> mqMessageList = JSONArray.parseArray(message, MQWebDoctorBean.class);

        if (CollectionUtil.isEmpty(mqMessageList)) {
            return;
        }

        List<UserAddDTO> list = new ArrayList<>();
        mqMessageList.forEach(i -> {
            UserAddDTO userAddDTO = new UserAddDTO();
            userAddDTO.setCompanyNo("sys");
            userAddDTO.setId(i.getEmpCode());
            userAddDTO.setUserName(i.getEmpCode());
            userAddDTO.setRealName(i.getName());
            userAddDTO.setHeadUrl(i.getImg());
            userAddDTO.setUserShanc(i.getShanc());
            userAddDTO.setJobTitle(i.getZhic());
            userAddDTO.setJobPosition(i.getZhiw());
            list.add(userAddDTO);
        });


        log.info("2.医生消息信息保存>>>>>> {}", JSON.toJSONString(list));
        userInfoBizService.addDoctorBatch(list);

        log.info("3.医生消费者消费消息结束<<<<<<");
    }


}
