package com.xxh.cloud.user.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.basic.client.api.CompanyClientService;
import com.xxh.cloud.basic.client.api.bean.bos.CompanyBO;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.tree.TreeUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.DeptBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.*;
import com.xxh.cloud.user.framework.modules.base.entity.vos.CompanyDeptVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.base.service.DeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

//@SaCheckPermission("sys:user:dept")
@RestController
@RequestMapping("/sys/user/dept")
@Api(tags="运营端用户(管理),部门管理")
public class DeptManagerController {

    @Autowired
    private DeptBizService deptBizService;

    @Autowired
    private CompanyClientService companyClientService;


    @PostMapping("company2deptTree")
    @ApiOperation("(公司+部门)树")
    @SaCheckPermission("sys:user:dept:company2deptTree")
    public Result<List<CompanyDeptVO>> company2deptTree() {
        List<CompanyDeptVO> list=new ArrayList<>();
        List<CompanyBO> companyList= companyClientService.getCompanyList();
        List<DeptVO> deptList= deptBizService.list(new DeptSearchDTO());
        if(CollUtil.isNotEmpty(companyList)){
            companyList.forEach(company->{
                list.add(new CompanyDeptVO(company.getId(),company.getName(),company.getSort(),company.getPid()));
            });
        }
        if(CollUtil.isNotEmpty(deptList)){
            deptList.forEach(dept->{
                if(Constant.RootParentNodeId.equals(dept.getPid())){
                    list.add(
                            new CompanyDeptVO(dept.getId(),dept.getCompanyNo(),dept.getName(),dept.getSort(),dept.getCompanyNo())
                    );
                }else {
                    list.add(
                            new CompanyDeptVO(dept.getId(),dept.getCompanyNo(),dept.getName(),dept.getSort(),dept.getPid())
                    );
                }
            });
        }

        List<CompanyDeptVO> data= TreeUtils.build(list);
        return new Result<List<CompanyDeptVO>>().ok(data);
    }


    @PostMapping("tree")
    @ApiOperation("部门树")
    @SaCheckPermission("sys:user:dept:tree")
    public Result<List<DeptVO>> tree(@RequestBody DeptTreeDTO params) {
        ValidatorUtils.validateEntity(params);
        List<DeptVO> data= deptBizService.getTree(params);
        return new Result<List<DeptVO>>().ok(data);
    }


    @PostMapping("query")
    @ApiOperation("部门列表")
    @SaCheckPermission("sys:user:dept:query")
    public Result<PageData<DeptVO>> query(@RequestBody DeptQueryPageDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<DeptVO> data = deptBizService.query(params);
        return new Result<PageData<DeptVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增部门")
    @SaCheckPermission("sys:user:dept:add")
    public Result<DeptVO> add(@RequestBody DeptAddDTO params) {
        ValidatorUtils.validateEntity(params);
        DeptVO data= deptBizService.add(params);
        return new Result<DeptVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改部门")
    @SaCheckPermission("sys:user:dept:update")
    public Result<DeptVO> update(@RequestBody DeptUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        DeptVO data= deptBizService.update(params);
        return new Result<DeptVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取部门")
    @SaCheckPermission("sys:user:dept:info")
    public Result<DeptVO> info(@RequestBody DeptInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        DeptVO data=deptBizService.infoById(params.getId());
        return new Result<DeptVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除部门")
    @SaCheckPermission("sys:user:dept:del")
    public Result<String> del(@RequestBody DeptInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        deptBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }










}
