package com.xxh.cloud.user.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.GroupInfoBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.*;
import com.xxh.cloud.user.framework.modules.base.entity.vos.GroupInfoVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.PostInfoVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.RoleVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/sys/user/group")
@Api(tags="运营端用户(管理),分组管理")
public class GroupManagerController {

    @Autowired
    private GroupInfoBizService groupInfoBizService;



    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:user:group:query")
    public Result<PageData<GroupInfoVO>> query(@RequestBody GroupInfoQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        PageData<GroupInfoVO> data= groupInfoBizService.query(params);
        return new Result<PageData<GroupInfoVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增")
    @SaCheckPermission("sys:user:group:add")
    public Result<GroupInfoVO> add(@RequestBody GroupInfoSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        GroupInfoVO data= groupInfoBizService.add(params);
        return new Result<GroupInfoVO>().ok(data);
    }


    @PostMapping("edit")
    @ApiOperation("修改")
    @SaCheckPermission("sys:user:group:edit")
    public Result<GroupInfoVO> edit(@RequestBody GroupInfoEditDTO params) {
        ValidatorUtils.validateEntity(params);
        GroupInfoVO data= groupInfoBizService.edit(params);
        return new Result<GroupInfoVO>().ok(data);
    }



    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:user:group:info")
    public Result<GroupInfoVO> info(@RequestBody GroupInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        GroupInfoVO data=groupInfoBizService.infoById(params.getId());
        return new Result<GroupInfoVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:user:group:del")
    public Result<String> del(@RequestBody GroupInfoGetDTO params) {
        ValidatorUtils.validateEntity(params);
        groupInfoBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }





}
