package com.xxh.cloud.user.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.entity.dto.MenuAddDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.MenuInfoDTO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.MenuUpdateDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.MenuVO;
import com.xxh.cloud.user.framework.modules.base.service.MenuService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:user:menu")
@RestController
@RequestMapping("/sys/user/menu")
@Api(tags="运营端用户(管理),菜单管理")
public class MenuManagerController {

    @Autowired
    private MenuService menuService;


    @PostMapping("tree")
    @ApiOperation("菜单树")
    @SaCheckPermission("sys:user:menu:tree")
    public Result<List<MenuVO>> tree() {
        List<MenuVO> data= menuService.getTree();
        return new Result<List<MenuVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增菜单")
    @SaCheckPermission("sys:user:menu:add")
    public Result<MenuVO> add(@RequestBody MenuAddDTO params) {
        ValidatorUtils.validateEntity(params);
        MenuVO data= menuService.add(params);
        return new Result<MenuVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改菜单")
    @SaCheckPermission("sys:user:menu:update")
    public Result<MenuVO> update(@RequestBody MenuUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        MenuVO data= menuService.update(params);
        return new Result<MenuVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取菜单")
    @SaCheckPermission("sys:user:menu:info")
    public Result<MenuVO> info(@RequestBody MenuInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        MenuVO data=menuService.infoById(params.getId());
        return new Result<MenuVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除菜单")
    @SaCheckPermission("sys:user:menu:del")
    public Result<String> del(@RequestBody MenuInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        menuService.delById(params.getId());
        return new Result<String>().ok("ok");
    }





}
