package com.xxh.cloud.user.sys.manager.controller;


import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.user.AuthContextUtils;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.client.api.AdminUserClientService;
import com.xxh.cloud.user.client.api.bean.bos.AdminUserBO;
import com.xxh.cloud.user.framework.modules.base.entity.dto.UserQueryDTO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.UserVO;
import com.xxh.cloud.user.framework.modules.user.entity.dto.UserKickOutDTO;
import com.xxh.cloud.user.framework.modules.user.entity.dto.UserOnlineQueryDTO;
import com.xxh.cloud.user.framework.modules.user.entity.vos.UserOnlineVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

@RestController
@RequestMapping("/sys/user/onlineUser")
@Api(tags="运营端用户(管理),在线用户管理")
public class OnlineUserManagerController {


    @Autowired
    private AdminUserClientService adminUserClientService;



    private List<UserOnlineVO> getUserOnlineList(List<String> sessionIds){
        List<UserOnlineVO> data=new ArrayList<>();
        if(CollUtil.isEmpty(sessionIds)){
            return data;
        }

        Set<String> userIds=new HashSet<>();
        Set<String> thirdUserIds=new HashSet<>();

        for (String sessionId:sessionIds){
            UserOnlineVO userOnlineVO=new UserOnlineVO();
            userOnlineVO.setSessionId(sessionId);
            userOnlineVO.setLoginUserId(this.getLoginUserId(sessionId));
            this.getUserId(userOnlineVO);
            if("U".equals(userOnlineVO.getUserType())){
                userIds.add(userOnlineVO.getUserId());
            }else if("T".equals(userOnlineVO.getUserType())){
                thirdUserIds.add(userOnlineVO.getLoginUserId());
            }
            data.add(userOnlineVO);
        }

        Map<String, AdminUserBO> userMap= adminUserClientService.getUserMapByIds(userIds);
        if(MapUtil.isNotEmpty(userMap)){
            data.forEach(item->{
                if("U".equals(item.getUserType())){
                    AdminUserBO userBO =  userMap.get(item.getUserId());
                    if(userBO!=null){
                        item.setUserName(userBO.getUserName());
                        item.setUserRealName(userBO.getRealName());
                    }
                }else if("T".equals(item.getUserType())){
                    //todo:
                }
            });
        }
        return data;
    }

    private String getLoginUserId(String sessionId){
        return sessionId.substring(sessionId.lastIndexOf(":")+1);
    }

    private void getUserId(UserOnlineVO userOnlineVO){
        String loginUserId=userOnlineVO.getLoginUserId();
        //用户ID
        if(!loginUserId.contains("NONEUSER")){
            userOnlineVO.setUserType("U");
            userOnlineVO.setUserId(loginUserId);
            return;
        }
        //三方用户ID
        if(!loginUserId.contains("NTU")){
            userOnlineVO.setUserType("T");
            userOnlineVO.setUserId(loginUserId.substring(loginUserId.indexOf("NONEUSER")+"NONEUSER".length()));
            return;
        }
        userOnlineVO.setUserType("N");
    }

    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:user:onlineUser:query")
    public Result<List<UserOnlineVO>> query(@RequestBody UserOnlineQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        List<String> sessionIds= AuthContextUtils.searchSessionId(params.getKeyword(),0,-1,true);
        List<UserOnlineVO> data=this.getUserOnlineList(sessionIds);
        return new Result<List<UserOnlineVO>>().ok(data);
    }


    @PostMapping("kickout")
    @ApiOperation("下线")
    @SaCheckPermission("sys:user:onlineUser:kickout")
    public Result<String> kickout(@RequestBody UserKickOutDTO params) {
        ValidatorUtils.validateEntity(params);
        AuthContextUtils.kickoutByUserId(params.getId());
        return new Result<String>().ok("ok");
    }







}
