package com.xxh.cloud.user.sys.manager.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.page.PageData;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.user.framework.modules.base.biz.DeptBizService;
import com.xxh.cloud.user.framework.modules.base.biz.RoleBizService;
import com.xxh.cloud.user.framework.modules.base.entity.dto.*;
import com.xxh.cloud.user.framework.modules.base.entity.vos.DeptVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.MenuVO;
import com.xxh.cloud.user.framework.modules.base.entity.vos.RoleVO;
import com.xxh.cloud.user.framework.modules.base.service.DeptService;
import com.xxh.cloud.user.framework.modules.base.service.MenuService;
import com.xxh.cloud.user.framework.modules.base.service.RoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

//@SaCheckPermission("sys:user:role")
@RestController
@RequestMapping("/sys/user/role")
@Api(tags="运营端用户(管理),角色管理")
public class RoleManagerController {

    @Autowired
    private RoleBizService roleBizService;
    @Autowired
    private MenuService menuService;

    @PostMapping("query")
    @ApiOperation("查询")
    @SaCheckPermission("sys:user:role:query")
    public Result<PageData<RoleVO>> query(@RequestBody RoleQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        if(StrUtil.isEmpty(params.getCompanyNo())){
            params.setCompanyNo(Constant.SysCompanyNo);
        }
        PageData<RoleVO> data= roleBizService.query(params);
        return new Result<PageData<RoleVO>>().ok(data);
    }


    @PostMapping("add")
    @ApiOperation("新增")
    @SaCheckPermission("sys:user:role:add")
    public Result<RoleVO> add(@RequestBody RoleAddDTO params) {
        ValidatorUtils.validateEntity(params);
        RoleVO data= roleBizService.add(params);
        return new Result<RoleVO>().ok(data);
    }


    @PostMapping("update")
    @ApiOperation("修改")
    @SaCheckPermission("sys:user:role:update")
    public Result<RoleVO> update(@RequestBody RoleUpdateDTO params) {
        ValidatorUtils.validateEntity(params);
        RoleVO data= roleBizService.update(params);
        return new Result<RoleVO>().ok(data);
    }


    @PostMapping("info")
    @ApiOperation("获取")
    @SaCheckPermission("sys:user:role:info")
    public Result<RoleVO> info(@RequestBody RoleInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        RoleVO data=roleBizService.infoById(params.getId());
        return new Result<RoleVO>().ok(data);
    }


    @PostMapping("del")
    @ApiOperation("删除")
    @SaCheckPermission("sys:user:role:del")
    public Result<String> del(@RequestBody RoleInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        roleBizService.delById(params.getId());
        return new Result<String>().ok("ok");
    }


    @PostMapping("getMenuTreeAllByRole")
    @ApiOperation("获取角色的所有菜单树")
    @SaCheckPermission("sys:role:getMenuTreeAllByRole")
    public Result<List<MenuVO>> getMenuTreeAllByRole(@RequestBody RoleInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        List<MenuVO> data= menuService.getTreeAllByRole(params.getId());
        return new Result<List<MenuVO>>().ok(data);
    }

    @PostMapping("getMenuTreeByRole")
    @ApiOperation("获取角色菜单树")
    @SaCheckPermission("sys:role:getMenuTreeByRole")
    public Result<List<MenuVO>> getMenuTreeByRole(@RequestBody RoleInfoDTO params) {
        ValidatorUtils.validateEntity(params);
        List<MenuVO> data=menuService.getTreeByRole(params.getId());
        return new Result<List<MenuVO>>().ok(data);
    }


    @PostMapping("getListByRole")
    @ApiOperation("获取角色菜单列表")
    @SaCheckPermission("sys:role:getListByRole")
    public Result<List<MenuVO>> getListByRole(@RequestBody MenuSearchDTO params) {
        List<MenuVO> data=menuService.getListByRole(params);
        return new Result<List<MenuVO>>().ok(data);
    }

    @PostMapping("setRoleMenu")
    @ApiOperation("绑定角色权限")
    @SaCheckPermission("sys:role:setRoleMenu")
    public Result setRoleMenu(@RequestBody RoleUpdateMenuDTO params) {
        ValidatorUtils.validateEntity(params);
        roleBizService.updateMenu(params);
        return new Result<>().success();
    }










}
